@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -48,7 +48,7 @@ public:
         if  ((size >0 && !isalpha(aname[0]))||
              (size >=3 && (aname[0]=='X' && aname[1]=='M' && aname[2]=='L')))
         {
-            attribute_name.insert(0,"ONE_");    
+            attribute_name.insert(0,"ONE_");
         }
     };
 
@@ -198,7 +198,7 @@ public:
      */
     Attribute* clone() const
     {
-        return new SingleAttribute(*this);   
+        return new SingleAttribute(*this);
     };
 
 private:
@@ -239,10 +239,22 @@ public:
     };
 
     /**
+     *  Returns the string value
+     *    @param name of the attribute
      *
+     *    @return the value of the attribute if found, empty otherwise
      */
     string vector_value(const char *name) const;
 
+    /**
+     *  Returns the boolean value
+     *    @param name of the attribute
+     *    @param value Bool value ("YES" is true)
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int vector_value(const char *name, bool& value) const;
+
     /**
      * Returns the integer value
      *
@@ -251,7 +263,49 @@ public:
      *
      * @return 0 on success, -1 otherwise
      */
-    int vector_value(const char *name, int & value) const;
+    int vector_value(const char *name, int& value) const;
+
+    /**
+     * Returns the long long value
+     *
+     * @param name Name of the attribute
+     * @param value Long long value
+     *
+     * @return 0 on success, -1 otherwise
+     */
+    int vector_value(const char *name, long long& value) const;
+
+    /**
+     * Returns the float value
+     *
+     * @param name Name of the attribute
+     * @param value Float value
+     *
+     * @return 0 on success, -1 otherwise
+     */
+    int vector_value(const char *name, float& value) const;
+
+    /**
+     * Returns the integer value
+     *
+     * @param name Name of the attribute
+     * @param value Integer value, if an error occurred the string returned is
+     * empty and value set to -1;
+     *
+     * @return the value in string form on success, "" otherwise
+     */
+    string vector_value_str(const char *name, int& value) const;
+
+    /**
+     * Returns the float value
+     *
+     * @param name Name of the attribute
+     * @param value Float value, if an error occurred the string returned is
+     * empty and value set to -1;
+     *
+     * @return the value in string form on success, "" otherwise
+     */
+    string vector_value_str(const char *name, float& value) const;
 
     /**
      *  Marshall the attribute in a single string. The string MUST be freed
@@ -290,7 +344,66 @@ public:
      *  Replace the value of the given vector attribute
      */
     void replace(const string& name, const string& value);
-    
+
+    /**
+     *  Replace the value of the given vector attribute
+     */
+    void replace(const string& name, int value)
+    {
+        ostringstream oss;
+
+        oss << value;
+
+        replace(name, oss.str());
+    }
+
+    /**
+     *  Replace the value of the given vector attribute
+     */
+    void replace(const string& name, long long value)
+    {
+        ostringstream oss;
+
+        oss << value;
+
+        replace(name, oss.str());
+    }
+
+    /**
+     *  Replace the value of the given vector attribute
+     */
+    void replace(const string& name, const char* value)
+    {
+        string svalue(value);
+
+        replace(name, svalue);
+    }
+
+    /**
+     *  Replace the value of the given vector attribute
+     */
+    void replace(const string& name, bool value)
+    {
+        string b_value;
+
+        if (value == true)
+        {
+            b_value = "YES";
+        }
+        else
+        {
+            b_value = "NO";
+        }
+
+        replace(name, b_value);
+    }
+
+    /**
+     * Removes given the vector attribute
+     * @param name of the vector attribute
+     */
+    void remove(const string& name);
+
     /**
      *  Returns the attribute type
      */
@@ -302,9 +415,9 @@ public:
     /**
      *  Clones the current attribute
      */
-    Attribute* clone() const
+    VectorAttribute* clone() const
     {
-        return new VectorAttribute(*this);   
+        return new VectorAttribute(*this);
     };
 
 private:
