@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -29,20 +29,23 @@ using namespace std;
 class ImagePool;
 class ImageManager;
 
+class DatastorePool;
+
 /**
- *  ImageManagerDriver represents the basic abstraction for Image Repository 
- *  drivers. It implements the protocol and recover functions from the Mad 
+ *  ImageManagerDriver represents the basic abstraction for Image Repository
+ *  drivers. It implements the protocol and recover functions from the Mad
  *  interface.
  */
 class ImageManagerDriver : public Mad
 {
 public:
 
-    ImageManagerDriver(int        userid, 
-                       const      map<string,string>& attrs, 
+    ImageManagerDriver(int        userid,
+                       const      map<string,string>& attrs,
                        bool       sudo,
-                       ImagePool* _ipool):
-            Mad(userid,attrs,sudo),ipool(_ipool){};
+                       ImagePool* _ipool,
+                       DatastorePool * _dspool):
+            Mad(userid,attrs,sudo),ipool(_ipool),dspool(_dspool){};
 
     virtual ~ImageManagerDriver(){};
 
@@ -50,7 +53,7 @@ public:
      *  Implements the Image Manager driver protocol.
      *    @param message the string read from the driver
      */
-    void protocol(string& message);
+    void protocol(const string& message) const;
 
     /**
      *  TODO: What do we need here? Check on-going xfr?
@@ -65,39 +68,52 @@ private:
      */
     ImagePool * ipool;
 
-	/**	
-	 *  Configuration file for the driver
-	 */
-	//Template	driver_conf;
+    /**
+     *  Reference to the DatastorePool
+     */
+    DatastorePool * dspool;
 
     /**
-     * Sends a copy request to the MAD
+     * Sends a copy request to the MAD.
      *    @param oid the image id.
      *    @param drv_msg xml data for the mad operation.
      */
     void cp(int oid, const string& drv_msg) const;
 
     /**
-     *  Sends a move request to the MAD: "MV IMAGE_ID SRC_PATH DST_PATH"
+     * Sends a clone request to the MAD.
      *    @param oid the image id.
-     *    @param destination is a driver specific location or "-" if not 
-     *           initialized
-     *    @param size_mb of the image to be created
+     *    @param drv_msg xml data for the mad operation.
      */
-    void mv(int oid, const string& source, const string& destination) const;
+    void clone(int oid, const string& drv_msg) const;
 
     /**
-     *  Sends a make filesystem request to the MAD: "MKFS IMAGE_ID PATH SIZE_MB"
+     *  Sends a stat  request to the MAD.
+     *    @param oid the id of the stat request
+     *    @param drv_msg xml data for the mad operation.
+     */
+    void stat(int oid, const string& drv_msg) const;
+
+    /**
+     *  Sends a make filesystem request to the MAD.
      *    @param oid the image id.
      *    @param drv_msg xml data for the mad operation.
      */
     void mkfs(int oid, const string& drv_msg) const;
+
     /**
      *  Sends a delete request to the MAD: "DELETE IMAGE_ID PATH"
      *    @param oid the image id.
      *    @param drv_msg xml data for the mad operation.
      */
     void rm(int oid, const string& drv_msg) const;
+
+    /**
+     *  Sends a monitor request to the MAD: "MONITOR DS_ID DS_XML"
+     *    @param oid the datastore id.
+     *    @param drv_msg xml data for the mad operation.
+     */
+    void monitor(int oid, const string& drv_msg) const;
 };
 
 /* -------------------------------------------------------------------------- */
