@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -18,6 +18,7 @@
 #define NEBULA_H_
 
 #include "SqlDB.h"
+#include "SystemDB.h"
 
 #include "NebulaTemplate.h"
 
@@ -29,6 +30,8 @@
 #include "GroupPool.h"
 #include "DatastorePool.h"
 #include "ClusterPool.h"
+#include "DocumentPool.h"
+#include "ZonePool.h"
 
 #include "VirtualMachineManager.h"
 #include "LifeCycleManager.h"
@@ -41,9 +44,17 @@
 #include "AclManager.h"
 #include "ImageManager.h"
 
+#include "DefaultQuotas.h"
+
 #include "Callbackable.h"
 
-class Nebula : public Callbackable
+
+/**
+ *  This is the main class for the OpenNebula daemon oned. It stores references
+ *  to the main modules and data pools. It also includes functions to bootstrap
+ *  the system and start all its components.
+ */
+class Nebula
 {
 public:
 
@@ -103,6 +114,16 @@ public:
         return clpool;
     };
 
+    DocumentPool * get_docpool()
+    {
+        return docpool;
+    };
+
+    ZonePool * get_zonepool()
+    {
+        return zonepool;
+    };
+
     // --------------------------------------------------------------
     // Manager Accessors
     // --------------------------------------------------------------
@@ -157,24 +178,61 @@ public:
     // --------------------------------------------------------------
 
     /**
-     *  Returns the value of DEBUG_LEVEL in oned.conf file
+     *  Returns the value of LOG->DEBUG_LEVEL in oned.conf file
      *      @return the debug level, to instantiate Log'ers
      */
     Log::MessageType get_debug_level() const
     {
-        Log::MessageType    clevel = Log::ERROR;
-        int                 log_level_int;
+        Log::MessageType            clevel = Log::ERROR;
+        vector<const Attribute *>   logs;
+        int                         rc;
+        int                         log_level_int;
 
-        nebula_configuration->get("DEBUG_LEVEL", log_level_int);
+        rc = nebula_configuration->get("LOG", logs);
 
-        if (0 <= log_level_int && log_level_int <= 3 )
+        if ( rc != 0 )
         {
-            clevel = static_cast<Log::MessageType>(log_level_int);
+            string value;
+            const VectorAttribute * log = static_cast<const VectorAttribute *>
+                                                          (logs[0]);
+            value = log->vector_value("DEBUG_LEVEL");
+
+            log_level_int = atoi(value.c_str());
+
+            if ( Log::ERROR <= log_level_int && log_level_int <= Log::DDDEBUG )
+            {
+                clevel = static_cast<Log::MessageType>(log_level_int);
+            }
         }
 
         return clevel;
     }
 
+    /**
+     *  Returns the value of LOG->SYSTEM in oned.conf file
+     *      @return the logging system CERR, FILE_TS or SYSLOG
+     */
+    NebulaLog::LogType get_log_system() const
+    {
+        vector<const Attribute *> logs;
+        int                       rc;
+        NebulaLog::LogType        log_system = NebulaLog::UNDEFINED;
+
+        rc = nebula_configuration->get("LOG", logs);
+
+        if ( rc != 0 )
+        {
+            string value;
+            const VectorAttribute * log = static_cast<const VectorAttribute *>
+                                                          (logs[0]);
+
+            value      = log->vector_value("SYSTEM");
+            log_system = NebulaLog::str_to_type(value);
+        }
+
+        return log_system;
+    };
+
     /**
      *  Returns the value of ONE_LOCATION env variable. When this variable is
      *  not defined the nebula location is "/".
@@ -228,39 +286,44 @@ public:
     };
 
     /**
-     *  Returns the default var location. When ONE_LOCATION is defined this path
-     *  points to $ONE_LOCATION/var, otherwise it is /var/lib/one.
-     *      @return the log location.
+     *
+     *
      */
-    const string& get_ds_location()
+    int get_ds_location(int cluster_id, string& dsloc)
     {
-        return ds_location;
-    };
-
-    /**
-     *  Returns the Transfer Manager for the system datastore
-     *      @return the tm name.
-     */
-    string get_system_ds_tm_mad()
-    {
-        Datastore * ds;
-        string      tm_mad = "";
-
-        ds = dspool->get(DatastorePool::SYSTEM_DS_ID, true);
-
-        if ( ds == 0 )
+        if ( cluster_id != -1 )
         {
-            NebulaLog::log("DaS", Log::ERROR, "Can not get system datastore");
-            return tm_mad;
+            Cluster * cluster = clpool->get(cluster_id, true);
+
+            if ( cluster == 0 )
+            {
+                return -1;
+            }
+
+            cluster->get_ds_location(dsloc);
+
+            cluster->unlock();
+        }
+        else
+        {
+            get_configuration_attribute("DATASTORE_LOCATION", dsloc);
         }
 
-        tm_mad = ds->get_tm_mad();
+        return 0;
+    }
 
-        ds->unlock();
-
-        return tm_mad;
+    /**
+     *  Returns the default vms location. When ONE_LOCATION is defined this path
+     *  points to $ONE_LOCATION/var/vms, otherwise it is /var/lib/one/vms. This
+     *  location stores vm related files: deployment, transfer, context, and
+     *  logs (in self-contained mode only)
+     *      @return the vms location.
+     */
+    const string& get_vms_location()
+    {
+        return vms_location;
     };
-    
+
     /**
      *  Returns the path of the log file for a VM, depending where OpenNebula is
      *  installed,
@@ -279,36 +342,281 @@ public:
     	}
     	else
     	{
-    		oss << nebula_location << "var/" << oid << "/vm.log";
+    		oss << vms_location << oid << "/vm.log";
     	}
 
     	return oss.str();
     };
 
+    /**
+     *  Returns the name of the host running oned
+     *    @return the name
+     */
     const string& get_nebula_hostname()
     {
         return hostname;
     };
 
+    /**
+     *  Returns the version of oned
+     *    @return the version
+     */
     static string version()
     {
-        return "OpenNebula 3.4.1";
+        return "OpenNebula " + code_version();
     };
 
-    static string db_version()
+    /**
+     *  Returns the version of oned
+     * @return
+     */
+    static string code_version()
     {
-        return "3.4.1";
+        return "4.5.80"; // bump version
     }
 
-    void start();
+    /**
+     * Version needed for the DB, shared tables
+     * @return
+     */
+    static string shared_db_version()
+    {
+        return "4.5.80";
+    }
 
+    /**
+     * Version needed for the DB, local tables
+     * @return
+     */
+    static string local_db_version()
+    {
+        return "4.5.80";
+    }
+
+    /**
+     *  Starts all the modules and services for OpenNebula
+     */
+    void start(bool bootstrap_only=false);
+
+    /**
+     *  Initialize the database
+     */
+    void bootstrap_db();
+
+    // --------------------------------------------------------------
+    // Federation
+    // --------------------------------------------------------------
+
+    bool is_federation_enabled()
+    {
+        return federation_enabled;
+    };
+
+    bool is_federation_master()
+    {
+        return federation_master;
+
+    };
+
+    bool is_federation_slave()
+    {
+        return federation_enabled && !federation_master;
+    };
+
+    int get_zone_id()
+    {
+        return zone_id;
+    };
+
+    const string& get_master_oned()
+    {
+        return master_oned;
+    };
+
+    // -----------------------------------------------------------------------
+    // Configuration attributes (read from oned.conf)
+    // -----------------------------------------------------------------------
+
+    /**
+     *  Gets a configuration attribute for oned
+     *    @param name of the attribute
+     *    @param value of the attribute
+     */
     void get_configuration_attribute(
         const char * name,
         string& value) const
     {
         string _name(name);
 
-        nebula_configuration->Template::get(_name,value);
+        nebula_configuration->Template::get(_name, value);
+    };
+
+    /**
+     *  Gets a configuration attribute for oned
+     *    @param name of the attribute
+     *    @param value of the attribute
+     */
+    void get_configuration_attribute(
+        const char * name,
+        long long& value) const
+    {
+        string _name(name);
+
+        nebula_configuration->Template::get(_name, value);
+    };
+
+    /**
+     *  Gets a configuration attribute for oned, bool version
+     */
+    void get_configuration_attribute(
+        const char * name,
+        bool& value) const
+    {
+        string _name(name);
+
+        nebula_configuration->Template::get(_name, value);
+    };
+
+    /**
+     *  Gets a TM configuration attribute
+     */
+    int get_tm_conf_attribute(
+        const string& tm_name,
+        const VectorAttribute* &value) const
+    {
+        vector<const Attribute*>::const_iterator it;
+        vector<const Attribute*> values;
+
+        nebula_configuration->Template::get("TM_MAD_CONF", values);
+
+        for (it = values.begin(); it != values.end(); it ++)
+        {
+            value = dynamic_cast<const VectorAttribute*>(*it);
+
+            if (value == 0)
+            {
+                continue;
+            }
+
+            if (value->vector_value("NAME") == tm_name)
+            {
+                return 0;
+            }
+        }
+
+        value = 0;
+        return -1;
+    };
+
+
+    /**
+     *  Gets an XML document with all of the configuration attributes
+     *    @return the XML
+     */
+    string get_configuration_xml() const
+    {
+        string xml;
+        return nebula_configuration->to_xml(xml);
+    };
+
+    // -----------------------------------------------------------------------
+    // Default Quotas
+    // -----------------------------------------------------------------------
+
+    /**
+     *  Get the default quotas for OpenNebula users
+     *    @return the default quotas
+     */
+    const DefaultQuotas& get_default_user_quota()
+    {
+        return default_user_quota;
+    };
+
+    /**
+     *  Set the default quotas for OpenNebula users
+     *    @param tmpl template with the default quotas
+     *    @param error describes the error if any
+     *
+     *    @return 0 if success
+     */
+    int set_default_user_quota(Template *tmpl, string& error)
+    {
+        int rc = default_user_quota.set(tmpl, error);
+
+        if ( rc == 0 )
+        {
+            rc = default_user_quota.update();
+        }
+
+        return rc;
+    };
+
+    /**
+     *  Get the default quotas for OpenNebula for groups
+     *    @return the default quotas
+     */
+    const DefaultQuotas& get_default_group_quota()
+    {
+        return default_group_quota;
+    };
+
+    /**
+     *  Set the default quotas for OpenNebula groups
+     *    @param tmpl template with the default quotas
+     *    @param error describes the error if any
+     *
+     *    @return 0 if success
+     */
+    int set_default_group_quota(Template *tmpl, string& error)
+    {
+        int rc = default_group_quota.set(tmpl, error);
+
+        if ( rc == 0 )
+        {
+            rc = default_group_quota.update();
+        }
+
+        return rc;
+    };
+
+    // -----------------------------------------------------------------------
+    // System attributes
+    // -----------------------------------------------------------------------
+    /**
+     *  Reads a System attribute from the DB
+     *    @param attr_name name of the attribute
+     *    @param cb Callback that will receive the attribute in XML
+     *    @return 0 on success
+     */
+    int select_sys_attribute(const string& attr_name, string& attr_xml)
+    {
+        return system_db->select_sys_attribute(attr_name, attr_xml);
+    };
+
+    /**
+     *  Writes a system attribute in the database.
+     *    @param db pointer to the db
+     *    @return 0 on success
+     */
+    int insert_sys_attribute(
+        const string& attr_name,
+        const string& xml_attr,
+        string&       error_str)
+    {
+        return system_db->insert_sys_attribute(attr_name, xml_attr, error_str);
+    };
+
+    /**
+     *  Updates the system attribute in the database.
+     *    @param db pointer to the db
+     *    @return 0 on success
+     */
+    int update_sys_attribute(
+        const string& attr_name,
+        const string& xml_attr,
+        string&       error_str)
+    {
+        return system_db->update_sys_attribute(attr_name, xml_attr, error_str);
     };
 
 private:
@@ -317,9 +625,22 @@ private:
     //Constructors and = are private to only access the class through instance
     // -----------------------------------------------------------------------
 
-    Nebula():nebula_configuration(0),db(0),vmpool(0),hpool(0),vnpool(0),
-        upool(0),ipool(0),gpool(0),tpool(0),dspool(0),clpool(0),
-        lcm(0),vmm(0),im(0),tm(0),dm(0),rm(0),hm(0),authm(0),aclm(0),imagem(0)
+    Nebula():nebula_configuration(0),
+        default_user_quota( "DEFAULT_USER_QUOTAS",
+                            "/DEFAULT_USER_QUOTAS/DATASTORE_QUOTA",
+                            "/DEFAULT_USER_QUOTAS/NETWORK_QUOTA",
+                            "/DEFAULT_USER_QUOTAS/IMAGE_QUOTA",
+                            "/DEFAULT_USER_QUOTAS/VM_QUOTA"),
+        default_group_quota("DEFAULT_GROUP_QUOTAS",
+                            "/DEFAULT_GROUP_QUOTAS/DATASTORE_QUOTA",
+                            "/DEFAULT_GROUP_QUOTAS/NETWORK_QUOTA",
+                            "/DEFAULT_GROUP_QUOTAS/IMAGE_QUOTA",
+                            "/DEFAULT_GROUP_QUOTAS/VM_QUOTA"),
+        system_db(0), db(0),
+        vmpool(0), hpool(0), vnpool(0), upool(0), ipool(0), gpool(0), tpool(0),
+        dspool(0), clpool(0), docpool(0), zonepool(0),
+        lcm(0), vmm(0), im(0), tm(0), dm(0), rm(0), hm(0), authm(0),
+        aclm(0), imagem(0)
     {
         const char * nl = getenv("ONE_LOCATION");
 
@@ -332,7 +653,7 @@ private:
             log_location     = "/var/log/one/";
             var_location     = "/var/lib/one/";
             remotes_location = "/var/lib/one/remotes/";
-            ds_location      = "/var/lib/one/datastores/";
+            vms_location     = "/var/lib/one/vms/";
         }
         else
         {
@@ -348,120 +669,38 @@ private:
             log_location     = nebula_location + "var/";
             var_location     = nebula_location + "var/";
             remotes_location = nebula_location + "var/remotes/";
-            ds_location      = nebula_location + "var/datastores/";
+            vms_location     = nebula_location + "var/vms/";
         }
     };
 
     ~Nebula()
     {
-        if ( vmpool != 0)
-        {
-            delete vmpool;
-        }
-
-        if ( vnpool != 0)
-        {
-            delete vnpool;
-        }
-
-        if ( hpool != 0)
-        {
-            delete hpool;
-        }
-
-        if ( upool != 0)
-        {
-            delete upool;
-        }
-
-        if ( ipool != 0)
-        {
-            delete ipool;
-        }
-
-        if ( gpool != 0)
-        {
-            delete gpool;
-        }
-
-        if ( tpool != 0)
-        {
-            delete tpool;
-        }
-
-        if ( dspool != 0)
-        {
-            delete dspool;
-        }
-
-        if ( clpool != 0)
-        {
-            delete clpool;
-        }
-        
-        if ( vmm != 0)
-        {
-            delete vmm;
-        }
-
-        if ( lcm != 0)
-        {
-            delete lcm;
-        }
-
-        if ( im != 0)
-        {
-            delete im;
-        }
-
-        if ( tm != 0)
-        {
-            delete tm;
-        }
-
-        if ( dm != 0)
-        {
-            delete dm;
-        }
-
-        if ( rm != 0)
-        {
-            delete rm;
-        }
-
-        if ( hm != 0)
-        {
-            delete hm;
-        }
-
-        if ( authm != 0)
-        {
-            delete authm;
-        }
-
-        if ( aclm != 0)
-        {
-            delete aclm;
-        }
-
-        if ( imagem != 0)
-        {
-            delete imagem;
-        }
-
-        if ( nebula_configuration != 0)
-        {
-            delete nebula_configuration;
-        }
-
-        if ( db != 0 )
-        {
-            delete db;
-        }
+        delete vmpool;
+        delete vnpool;
+        delete hpool;
+        delete upool;
+        delete ipool;
+        delete gpool;
+        delete tpool;
+        delete dspool;
+        delete clpool;
+        delete docpool;
+        delete zonepool;
+        delete vmm;
+        delete lcm;
+        delete im;
+        delete tm;
+        delete dm;
+        delete rm;
+        delete hm;
+        delete authm;
+        delete aclm;
+        delete imagem;
+        delete nebula_configuration;
+        delete db;
+        delete system_db;
     };
 
-    Nebula(Nebula const&){};
-
     Nebula& operator=(Nebula const&){return *this;};
 
     // ---------------------------------------------------------------
@@ -476,7 +715,7 @@ private:
     string	var_location;
     string  hook_location;
     string  remotes_location;
-    string  ds_location;
+    string  vms_location;
 
     string	hostname;
 
@@ -484,7 +723,29 @@ private:
     // Configuration
     // ---------------------------------------------------------------
 
-    OpenNebulaTemplate *    nebula_configuration;
+    OpenNebulaTemplate * nebula_configuration;
+
+    // ---------------------------------------------------------------
+    // Federation
+    // ---------------------------------------------------------------
+
+    bool    federation_enabled;
+    bool    federation_master;
+    int     zone_id;
+    string  master_oned;
+
+    // ---------------------------------------------------------------
+    // Default quotas
+    // ---------------------------------------------------------------
+
+    DefaultQuotas default_user_quota;
+    DefaultQuotas default_group_quota;
+
+    // ---------------------------------------------------------------
+    // The system database
+    // ---------------------------------------------------------------
+
+    SystemDB * system_db;
 
     // ---------------------------------------------------------------
     // Nebula Pools
@@ -500,6 +761,8 @@ private:
     VMTemplatePool     * tpool;
     DatastorePool      * dspool;
     ClusterPool        * clpool;
+    DocumentPool       * docpool;
+    ZonePool           * zonepool;
 
     // ---------------------------------------------------------------
     // Nebula Managers
@@ -521,34 +784,6 @@ private:
     // ---------------------------------------------------------------
 
     friend void nebula_signal_handler (int sig);
-
-    /**
-     *  Bootstraps the database control tables
-     *
-     *    @return 0 on success
-     */
-    int bootstrap();
-
-    /**
-     *  Callback function for the check_db_version method. Stores the read
-     *  version in loaded_db_version
-     *    @param _loaded_db_version returned columns
-     *    @param num the number of columns read from the DB
-     *    @param names the column names
-     *    @param vaues the column values
-     *    @return 0 on success
-     */
-    int select_cb(void *_loaded_db_version, int num, char **values,
-                  char **names);
-
-    /**
-     * Reads the current DB version.
-     *
-     * @return  0 on success,
-     *          -1 if there is a version mismatch,
-     *          -2 if the DB needs a bootstrap
-     */
-    int check_db_version();
 };
 
 #endif /*NEBULA_H_*/
