@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,6 +20,7 @@
 #include <map>
 #include <string>
 #include <queue>
+#include <set>
 
 #include "SqlDB.h"
 #include "PoolObjectSQL.h"
@@ -44,9 +45,10 @@ public:
      *   @param _db a pointer to the database
      *   @param _table the name of the table supporting the pool (to set the oid
      *   counter). If null the OID counter is not updated.
+     *   @param cache True to enable the cache
      *   @param cache_by_name True if the objects can be retrieved by name
      */
-    PoolSQL(SqlDB * _db, const char * _table, bool cache_by_name);
+    PoolSQL(SqlDB * _db, const char * _table, bool cache, bool cache_by_name);
 
     virtual ~PoolSQL();
 
@@ -97,6 +99,20 @@ public:
         const char *    table,
         const string&   where);
 
+    /**
+     *  List the objects in the pool
+     *   @param oids a vector with the oids of the objects.
+     *   @param table the name of the DB table.
+     *
+     *   @return 0 on success
+     */
+    int list(
+        vector<int>&    oids,
+        const char *    table)
+    {
+        return search(oids, table, "");
+    }
+
     /**
      *  Updates the object's data in the data base. The object mutex SHOULD be
      *  locked.
@@ -135,6 +151,10 @@ public:
             error_msg = "SQL DB error";
             return -1;
         }
+        else
+        {
+            do_hooks(objsql, Hook::REMOVE);
+        }
 
         return 0;
     };
@@ -152,10 +172,79 @@ public:
      *
      *  @return 0 on success
      */
-    virtual int dump(ostringstream& oss, const string& where) = 0;
+    int dump(ostringstream& oss, const string& where)
+    {
+        return dump(oss, where, "");
+    }
 
+    /**
+     *  Dumps the pool in XML format. A filter and limit can be also added
+     *  to the query
+     *  @param oss the output stream to dump the pool contents
+     *  @param where filter for the objects, defaults to all
+     *  @param limit parameters used for pagination
+     *
+     *  @return 0 on success
+     */
+
+    virtual int dump(ostringstream& oss, const string& where,
+                     const string& limit) = 0;
+
+    // -------------------------------------------------------------------------
+    // Function to generate dump filters
+    // -------------------------------------------------------------------------
+
+    /**
+     *  Creates a filter for those objects (oids) or objects owned by a given
+     *  group that an user can access based on the ACL rules
+     *    @param uid the user id
+     *    @param user_groups Set of group IDs that the user is part of
+     *    @param auth_object object type
+     *    @param all returns if the user can access all objects
+     *    @param filter the resulting filter string
+     */
+    static void acl_filter(int                       uid,
+                           const set<int>&           user_groups,
+                           PoolObjectSQL::ObjectType auth_object,
+                           bool&                     all,
+                           string&                   filter);
+
+    /**
+     *  Creates a filter for the objects owned by a given user/group
+     *    @param uid the user id
+     *    @param user_groups Set of group IDs that the user is part of
+     *    @param filter_flag query type (ALL, MINE, GROUP)
+     *    @param all user can access all objects
+     *    @param filter the resulting filter string
+     */
+    static void usr_filter(int              uid,
+                           const set<int>&  user_groups,
+                           int              filter_flag,
+                           bool             all,
+                           const string&    acl_str,
+                           string&          filter);
+    /**
+     *  Creates a filter for a given set of objects based on their id
+     *    @param start_id first id
+     *    @param end_id last id
+     *    @param filter the resulting filter string
+     */
+    static void oid_filter(int     start_id,
+                           int     end_id,
+                           string& filter);
 protected:
 
+    /**
+     *  Register on "CREATE" and on "REMOVE" hooks for the pool. The hooks are
+     *  meant to be executed locally by the generic AllocateHook and RemoveHook
+     *  classes.
+     *    @param hook_mads, array of HOOKs to be installed
+     *    @param remotes_location, path to remotes in the front-end where the
+     *    hooks are installed
+     */
+    void register_hooks(vector<const Attribute *> hook_mads,
+                        const string&             remotes_location);
+
     /**
      *  Gets an object from the pool (if needed the object is loaded from the
      *  database).
@@ -172,6 +261,23 @@ protected:
      */
     SqlDB * db;
 
+    /**
+     *  Dumps the pool in XML format. A filter and limit can be also added
+     *  to the query
+     *  @param oss the output stream to dump the pool contents
+     *  @param elem_name Name of the root xml pool name
+     *  @param table Pool table name
+     *  @param where filter for the objects, defaults to all
+     *  @param limit parameters used for pagination
+     *
+     *  @return 0 on success
+     */
+    int dump(ostringstream& oss,
+             const string&  elem_name,
+             const char *   table,
+             const string&  where,
+             const string&  limit);
+
     /**
      *  Dumps the pool in XML format. A filter can be also added to the
      *  query
@@ -182,8 +288,34 @@ protected:
      *
      *  @return 0 on success
      */
-    int dump(ostringstream& oss, const string& elem_name,
-             const char * table, const string& where);
+    int dump(ostringstream& oss,
+             const string&  elem_name,
+             const char *   table,
+             const string&  where)
+    {
+        return dump(oss, elem_name, table, where, "");
+    }
+
+    /**
+     *  Dumps the output of the custom sql query into an xml
+     *
+     *   @param oss The output stream to dump the xml contents
+     *   @param root_elem_name Name of the root xml element name
+     *   @param sql_query The SQL query to execute
+     *
+     *   @return 0 on success
+     */
+    int dump(ostringstream&  oss,
+             const string&   root_elem_name,
+             ostringstream&  sql_query);
+
+    /**
+     * Child classes can add extra elements to the dump xml, right after all the
+     * pool objects
+     *
+     * @param oss The output stream to dump the xml contents
+     */
+    virtual void add_extra_xml(ostringstream&  oss){};
 
     /* ---------------------------------------------------------------------- */
     /* Interface to access the lastOID assigned by the pool                   */
@@ -237,6 +369,11 @@ private:
      */
     map<int,PoolObjectSQL *> pool;
 
+    /**
+     * Whether or not this pool uses the cache
+     */
+    bool cache;
+
     /**
      * Whether or not this pool uses the name_pool index
      */
@@ -284,11 +421,28 @@ private:
      */
     void replace();
 
+    /**
+     * Cleans all the objects in the cache, except the ones locked.
+     * The object with the given oid will not be ignored if locked, the
+     * method will wait for it to be unlocked and ensure it is erased from
+     * the cache
+     *
+     * @param oid
+     */
+    void flush_cache(int oid);
+
+    /**
+     * Same as flush_cache(int), but with the object name-uid key
+     *
+     * @param name_key
+     */
+    void flush_cache(const string& name_key);
+
     /**
      *  Generate an index key for the object
      *    @param name of the object
      *    @param uid owner of the object, only used if needed
-     *    
+     *
      *    @return the key, a string
      */
     virtual string key(const string& name, int uid)
