@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,8 +21,9 @@
 #include <xmlrpc-c/registry.hpp>
 
 #include "RequestManager.h"
-#include "AuthManager.h"
+#include "AuthRequest.h"
 #include "PoolObjectSQL.h"
+#include "Quotas.h"
 
 using namespace std;
 
@@ -72,9 +73,38 @@ protected:
         string uname;             /**< name of the user */
         string gname;             /**< name of the user's group */
 
+        set<int> group_ids;      /**< set of user's group ids */
+
         string session;           /**< Session from ONE XML-RPC API */
+        int    req_id;            /**< Request ID for log messages */
 
         xmlrpc_c::value * retval; /**< Return value from libxmlrpc-c */
+
+        RequestAttributes(){};
+
+        RequestAttributes(const RequestAttributes& ra)
+        {
+            uid = ra.uid;
+            gid = ra.gid;
+
+            uname = ra.uname;
+            gname = ra.gname;
+
+            session  = ra.session;
+            retval   = ra.retval;
+        };
+
+        RequestAttributes(int _uid, int _gid, const RequestAttributes& ra)
+        {
+            uid = _uid;
+            gid = _gid;
+
+            uname = "";
+            gname = "";
+
+            session  = ra.session;
+            retval   = ra.retval;
+        };
     };
 
     /* -------- Static (shared among request of the same method) -------- */
@@ -87,8 +117,8 @@ protected:
 
     /* -------------------- Constructors ---------------------------------- */
 
-    Request(const string& mn, 
-            const string& signature, 
+    Request(const string& mn,
+            const string& signature,
             const string& help): pool(0),method_name(mn)
     {
         _signature = signature;
@@ -102,8 +132,8 @@ protected:
 
     /**
      *  Performs a basic authorization for this request using the uid/gid
-     *  from the request. The function gets the object from the pool to get 
-     *  the public attribute and its owner. The authorization is based on 
+     *  from the request. The function gets the object from the pool to get
+     *  the public attribute and its owner. The authorization is based on
      *  object and type of operation for the request.
      *    @param oid of the object, can be -1 for objects to be created, or
      *    pools.
@@ -130,7 +160,52 @@ protected:
      */
     bool basic_authorization(int oid, AuthRequest::Operation op,
                              RequestAttributes& att);
-            
+
+    /**
+     *  Performs a basic quota check for this request using the uid/gid
+     *  from the request. Usage counters are updated for the user/group.
+     *  On case of error, the failure_response return values are set
+     *
+     *    @param tmpl describing the object
+     *    @param object type of the object
+     *    @param att the specific request attributes
+     *
+     *    @return true if the user is authorized.
+     */
+    bool quota_authorization(
+            Template *          tmpl,
+            Quotas::QuotaType   qtype,
+            RequestAttributes&  att);
+
+    /**
+     *  Performs a basic quota check for this request using the uid/gid
+     *  from the request. Usage counters are updated for the user/group.
+     *  On case of error, the failure_response return values is not set, instead
+     *  the error reason is returned in error_str
+     *
+     *    @param tmpl describing the object
+     *    @param object type of the object
+     *    @param att the specific request attributes
+     *
+     *    @param error_str Error reason, if any
+     *    @return true if the user is authorized.
+     */
+    bool quota_authorization(
+            Template *          tmpl,
+            Quotas::QuotaType   qtype,
+            RequestAttributes&  att,
+            string&             error_str);
+
+    /**
+     *  Performs rollback on usage counters for a previous  quota check operation
+     *  for the request.
+     *    @param tmpl describing the object
+     *    @param att the specific request attributes
+     */
+    void quota_rollback(Template *         tmpl,
+                        Quotas::QuotaType  qtype,
+                        RequestAttributes& att);
+
     /**
      *  Actual Execution method for the request. Must be implemented by the
      *  XML-RPC requests
@@ -230,6 +305,7 @@ protected:
      *
      * @param perms returns the object's permissions
      * @param name returns the object's name
+     * @param throw_error send error response to client if object not found
      *
      * @return 0 on success, -1 otherwise
      */
@@ -238,7 +314,70 @@ protected:
                   PoolObjectSQL::ObjectType type,
                   RequestAttributes&        att,
                   PoolObjectAuth&           perms,
-                  string&                   name);
+                  string&                   name,
+                  bool                      throw_error);
+
+    /**
+     * Logs the method invocation, including the arguments
+     *
+     * @param att the specific request attributes
+     * @param paramList list of XML parameters
+     */
+    virtual void log_method_invoked(
+            const RequestAttributes&    att,
+            const xmlrpc_c::paramList&  paramList);
+
+    /**
+     * Logs the method result, including the output data or error message
+     *
+     * @param att the specific request attributes
+     */
+    virtual void log_result(
+            const RequestAttributes&    att);
+
+    /**
+     * Formats and adds a xmlrpc_c::value input parameter to oss.
+     *
+     * @param v value to format
+     * @param oss stream to write v
+     * @param index parameter index
+     */
+    virtual void log_xmlrpc_param(
+            const xmlrpc_c::value&  v,
+            ostringstream&          oss,
+            const int&              index);
+
+    /**
+     * Formats and adds a xmlrpc_c::value to oss.
+     *
+     * @param v value to format
+     * @param oss stream to write v
+     */
+    virtual void log_xmlrpc_value(
+            const xmlrpc_c::value&  v,
+            ostringstream&          oss);
+
+private:
+
+    /* ------------- Functions to manage user and group quotas -------------- */
+
+    bool user_quota_authorization(Template * tmpl,
+                                  Quotas::QuotaType  qtype,
+                                  RequestAttributes& att,
+                                  string& error_str);
+
+    bool group_quota_authorization(Template * tmpl,
+                                   Quotas::QuotaType  qtype,
+                                   RequestAttributes& att,
+                                   string& error_str);
+
+    void user_quota_rollback(Template * tmpl,
+                             Quotas::QuotaType  qtype,
+                             RequestAttributes& att);
+
+    void group_quota_rollback(Template * tmpl,
+                              Quotas::QuotaType  qtype,
+                              RequestAttributes& att);
 };
 
 /* -------------------------------------------------------------------------- */
