@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -32,7 +32,7 @@ class RequestManagerDelete: public Request
 protected:
     RequestManagerDelete(const string& method_name,
                          const string& help)
-        :Request(method_name,"A:si",help)
+        :Request(method_name, "A:si", help)
     {
         auth_op = AuthRequest::MANAGE;
 
@@ -48,9 +48,9 @@ protected:
     void request_execute(xmlrpc_c::paramList const& _paramList,
                          RequestAttributes& att);
 
-    bool delete_authorization(int                oid,
-                              RequestAttributes& att);
-                              
+    bool delete_authorization(int                           oid,
+                              RequestAttributes&            att);
+
     /* -------------------------------------------------------------------- */
 
     virtual int drop(int oid, PoolObjectSQL * object, string& error_msg);
@@ -98,7 +98,7 @@ public:
     VirtualNetworkDelete():
         RequestManagerDelete("VirtualNetworkDelete",
                              "Deletes a virtual network")
-    {    
+    {
         Nebula& nd  = Nebula::instance();
         pool        = nd.get_vnpool();
         auth_object = PoolObjectSQL::NET;
@@ -127,7 +127,7 @@ class ImageDelete: public RequestManagerDelete
 public:
     ImageDelete():
         RequestManagerDelete("ImageDelete", "Deletes an image")
-    {    
+    {
         Nebula& nd  = Nebula::instance();
         pool        = nd.get_ipool();
         auth_object = PoolObjectSQL::IMAGE;
@@ -149,7 +149,7 @@ class HostDelete : public RequestManagerDelete
 public:
     HostDelete():
         RequestManagerDelete("HostDelete", "Deletes a host")
-    {    
+    {
         Nebula& nd  = Nebula::instance();
         pool        = nd.get_hpool();
         auth_object = PoolObjectSQL::HOST;
@@ -169,6 +169,10 @@ public:
     {
         return cluster->del_host(id, error_msg);
     };
+
+    /* -------------------------------------------------------------------- */
+
+    int drop(int oid, PoolObjectSQL * object, string& error_msg);
 };
 
 /* ------------------------------------------------------------------------- */
@@ -179,7 +183,7 @@ class GroupDelete: public RequestManagerDelete
 public:
     GroupDelete():
         RequestManagerDelete("GroupDelete", "Deletes a group")
-    {    
+    {
         Nebula& nd = Nebula::instance();
         pool       = nd.get_gpool();
 
@@ -202,7 +206,7 @@ class UserDelete: public RequestManagerDelete
 public:
     UserDelete():
         RequestManagerDelete("UserDelete", "Deletes a user")
-    {    
+    {
         Nebula& nd  = Nebula::instance();
         pool        = nd.get_upool();
         gpool       = nd.get_gpool();
@@ -268,6 +272,46 @@ public:
     };
 
     ~ClusterDelete(){};
+
+    int drop(int oid, PoolObjectSQL * object, string& error_msg);
+};
+
+/* ------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------- */
+
+class DocumentDelete : public RequestManagerDelete
+{
+public:
+    DocumentDelete():
+        RequestManagerDelete("DocumentDelete",
+                             "Deletes a generic document")
+    {
+        Nebula& nd  = Nebula::instance();
+        pool        = nd.get_docpool();
+        auth_object = PoolObjectSQL::DOCUMENT;
+    };
+
+    ~DocumentDelete(){};
+};
+
+/* ------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------- */
+
+class ZoneDelete: public RequestManagerDelete
+{
+public:
+    ZoneDelete():
+        RequestManagerDelete("ZoneDelete", "Deletes a zone")
+    {
+        Nebula& nd  = Nebula::instance();
+        pool        = nd.get_zonepool();
+        auth_object = PoolObjectSQL::ZONE;
+        auth_op     = AuthRequest::ADMIN;
+    };
+
+    ~ZoneDelete(){};
+
+    int drop(int oid, PoolObjectSQL * object, string& error_msg);
 };
 
 /* -------------------------------------------------------------------------- */
