@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -48,8 +48,7 @@ public:
      *  Implements the VM Manager driver protocol.
      *    @param message the string read from the driver
      */
-    void protocol(
-        string&     message);
+    void protocol(const string& message) const;
 
     /**
      *  TODO: What do we need here? Check on-going xfr?
