@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -32,11 +32,13 @@ class VirtualMachinePool : public PoolSQL
 {
 public:
 
-    VirtualMachinePool(SqlDB * db,
-                       vector<const Attribute *> hook_mads,
-                       const string& hook_location,
-                       const string& remotes_location,
-                       vector<const Attribute *>& restricted_attrs);
+    VirtualMachinePool(SqlDB *                      db,
+                       vector<const Attribute *>    hook_mads,
+                       const string&                hook_location,
+                       const string&                remotes_location,
+                       vector<const Attribute *>&   restricted_attrs,
+                       time_t                       expire_time,
+                       bool                         on_hold);
 
     ~VirtualMachinePool(){};
 
@@ -44,10 +46,14 @@ public:
      *  Function to allocate a new VM object
      *    @param uid user id (the owner of the VM)
      *    @param gid the id of the group this object is assigned to
+     *    @param uname user name
+     *    @param gname group name
+     *    @param umask permissions umask
      *    @param vm_template a VM Template object describing the VM
      *    @param oid the id assigned to the VM (output)
      *    @param error_str Returns the error reason, if any
      *    @param on_hold flag to submit on hold
+     *
      *    @return oid on success, -1 error inserting in DB or -2 error parsing
      *  the template
      */
@@ -56,6 +62,7 @@ public:
         int                      gid,
         const string&            uname,
         const string&            gname,
+        int                      umask,
         VirtualMachineTemplate * vm_template,
         int *                    oid,
         string&                  error_str,
@@ -75,6 +82,26 @@ public:
         return static_cast<VirtualMachine *>(PoolSQL::get(oid,lock));
     };
 
+    /**
+     *  Function to get a VM from the pool, string version for VM ID
+     */
+    VirtualMachine * get(
+        const string& oid_s,
+        bool          lock)
+    {
+        istringstream iss(oid_s);
+        int           oid;
+
+        iss >> oid;
+
+        if ( iss.fail() )
+        {
+            return 0;
+        }
+
+        return static_cast<VirtualMachine *>(PoolSQL::get(oid,lock));
+    };
+
     /**
      *  Function to get the IDs of running VMs
      *   @param oids a vector that contains the IDs
@@ -122,6 +149,38 @@ public:
         return vm->update_previous_history(db);
     }
 
+    /**
+     * Inserts the last monitoring, and deletes old monitoring entries for this
+     * VM
+     *
+     * @param vm pointer to the virtual machine object
+     * @return 0 on success
+     */
+    int update_monitoring(
+        VirtualMachine * vm)
+    {
+        if ( _monitor_expiration <= 0 )
+        {
+            return 0;
+        }
+
+        return vm->update_monitoring(db);
+    };
+
+    /**
+     * Deletes the expired monitoring entries for all VMs
+     *
+     * @return 0 on success
+     */
+    int clean_expired_monitoring();
+
+    /**
+     * Deletes all monitoring entries for all VMs
+     *
+     * @return 0 on success
+     */
+    int clean_all_monitoring();
+
     /**
      *  Bootstraps the database table(s) associated to the VirtualMachine pool
      *    @return 0 on success
@@ -137,12 +196,57 @@ public:
      *  pool
      *  @param oss the output stream to dump the pool contents
      *  @param where filter for the objects, defaults to all
+     *  @param limit parameters used for pagination
      *
      *  @return 0 on success
      */
-    int dump(ostringstream& oss, const string& where)
+    int dump(ostringstream& oss, const string& where, const string& limit)
     {
-        return PoolSQL::dump(oss, "VM_POOL", VirtualMachine::table, where);
+        return PoolSQL::dump(oss, "VM_POOL", VirtualMachine::table, where,
+                             limit);
+    };
+
+    /**
+     *  Dumps the VM accounting information in XML format. A filter can be also
+     *  added to the query as well as a time frame.
+     *  @param oss the output stream to dump the pool contents
+     *  @param where filter for the objects, defaults to all
+     *
+     *  @return 0 on success
+     */
+    int dump_acct(ostringstream& oss,
+                  const string&  where,
+                  int            time_start,
+                  int            time_end);
+
+    /**
+     *  Dumps the VM monitoring information entries in XML format. A filter
+     *  can be also added to the query.
+     *
+     *  @param oss the output stream to dump the pool contents
+     *  @param where filter for the objects, defaults to all
+     *
+     *  @return 0 on success
+     */
+    int dump_monitoring(ostringstream& oss,
+                        const string&  where);
+
+    /**
+     *  Dumps the VM monitoring information  for a single VM
+     *
+     *  @param oss the output stream to dump the pool contents
+     *  @param vmid id of the target VM
+     *
+     *  @return 0 on success
+     */
+    int dump_monitoring(ostringstream& oss,
+                        int            vmid)
+    {
+        ostringstream filter;
+
+        filter << "oid = " << vmid;
+
+        return dump_monitoring(oss, filter.str());
     }
 
 private:
@@ -152,8 +256,18 @@ private:
      */
     PoolObjectSQL * create()
     {
-        return new VirtualMachine(-1,-1,-1,"","",0);
+        return new VirtualMachine(-1,-1,-1,"","",0,0);
     };
+
+    /**
+     * Size, in seconds, of the historical monitoring information
+     */
+    static time_t _monitor_expiration;
+
+    /**
+     * True or false whether to submit new VM on HOLD or not
+     */
+    static bool _submit_on_hold;
 };
 
 #endif /*VIRTUAL_MACHINE_POOL_H_*/
