@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -36,7 +36,7 @@ using namespace std;
 /* -------------------------------------------------------------------------- */
 
 /**
- *  The Virtual Network class. It represents a Virtual Network at manages its 
+ *  The Virtual Network class. It represents a Virtual Network at manages its
  *  leases. One lease is formed by one IP and one MAC address.
  *  MAC address are derived from IP addresses.
  */
@@ -70,7 +70,7 @@ public:
     /**
      * Adds Leases to the virtual network (Only implemented for FIXED networks)
      *  @param leases template in the form LEASES = [IP=XX, MAC=XX].
-     *         MAC is optional. The template can only contain one LEASE 
+     *         MAC is optional. The template can only contain one LEASE
      *         definition.
      *  @param error_msg If the action fails, this message contains the reason.
      *  @return 0 on success
@@ -78,7 +78,7 @@ public:
     int add_leases(VirtualNetworkTemplate * leases, string& error_msg);
 
     /**
-     * Removes Leases from the virtual network; if they are not used.(Only 
+     * Removes Leases from the virtual network; if they are not used.(Only
      * implemented for FIXED networks)
      *  @param leases template in the form LEASES = [IP=XX].
      *         The template can only contain one LEASE definition.
@@ -117,8 +117,10 @@ public:
      */
     int get_lease(int vid, string& _ip, string& _mac, string& _bridge)
     {
+        unsigned int eui64[2];
+
         _bridge = bridge;
-        return leases->get(vid,_ip,_mac);
+        return leases->get(vid, _ip, _mac, eui64);
     };
 
     /**
@@ -131,8 +133,10 @@ public:
      */
     int set_lease(int vid, const string& _ip, string& _mac, string& _bridge)
     {
+        unsigned int eui64[2];
+
         _bridge = bridge;
-        return leases->set(vid,_ip,_mac);
+        return leases->set(vid, _ip, _mac, eui64);
     };
 
     /**
@@ -146,7 +150,7 @@ public:
     };
 
     /**
-     *  Check if a VM is the owner of the ip 
+     *  Check if a VM is the owner of the ip
      *    @param ip of the lease to be checked
      *    @param vid the ID of the VM
      *    @return true if the ip was already assigned
@@ -173,6 +177,15 @@ public:
      */
     string& to_xml(string& xml) const;
 
+    /**
+     * Function to print the object into a string in XML format
+     * base64 encoded
+     *  @param xml64 the resulting XML string
+     *  @param extended return the extended template or the simple one
+     *  @return a reference to the generated string
+     */
+    string& to_xml64(string &xml64, bool extended);
+
     /**
      * Function to print the VirtualNetwork object into a string in
      * XML format. The extended XML includes the LEASES
@@ -188,9 +201,21 @@ public:
      *  * BRIDGE: for this virtual network
      *  @param nic attribute for the VM template
      *  @param vid of the VM getting the lease
+     *  @param inherit_attrs Attributes to be inherited from the vnet template
+     *      into the nic
      *  @return 0 on success
      */
-    int nic_attribute(VectorAttribute * nic, int vid);
+    int nic_attribute(
+            VectorAttribute *       nic,
+            int                     vid,
+            const vector<string>&   inherit_attrs);
+
+    /**
+     *  Replace the template of the virtual network it also updates the BRIDGE,
+     *  PHY_DEV, VLAN_ID and VLAN attributes.
+     *    @param tmpl string representation of the template
+     */
+    int replace_template(const string& tmpl_str, string& error);
 
 private:
 
@@ -216,7 +241,7 @@ private:
      *  Name of the physical device the bridge should be attached to
      */
     string  phydev;
-    
+
     /**
      *  VLAN ID of the NIC
      */
@@ -227,6 +252,26 @@ private:
      */
     int     vlan;
 
+    /**
+     *  IPv6 address global unicast prefix
+     */
+     string global;
+
+    /**
+     *  Binary representation of the IPv6 address global unicast prefix
+     */
+     unsigned int global_bin[2];
+
+    /**
+     *  IPv6 address site unicast prefix
+     */
+     string site;
+
+    /**
+     *  Binary representation of the IPv6 address site unicast prefix
+     */
+     unsigned int site_bin[2];
+
     // -------------------------------------------------------------------------
     // Virtual Network Description
     // -------------------------------------------------------------------------
@@ -299,6 +344,7 @@ private:
                    int                      gid,
                    const string&            _uname,
                    const string&            _gname,
+                   int                      _umask,
                    int                      _cluster_id,
                    const string&            _cluster_name,
                    VirtualNetworkTemplate * _vn_template = 0);
@@ -326,8 +372,8 @@ private:
      *  Reads the Virtual Network (identified with its OID) from the database.
      *    @param db pointer to the db
      *    @param name of the network
-     *    @param uid of the owner 
-     * 
+     *    @param uid of the owner
+     *
      *    @return 0 on success
      */
     int select(SqlDB * db, const string& name, int uid);
