@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -19,8 +19,12 @@
 ####################################################
 # Script to implement host failure tolerance
 #   It can be set to
-#           -r resubmit VMs running in the host
+#           -r recreate VMs running in the host
 #           -d delete VMs running in the host
+#   Additional flags
+#           -f force resubmission of suspended VMs
+#           -p <n> avoid resubmission if host comes
+#                  back after n monitoring cycles
 ####################################################
 
 ONE_LOCATION=ENV["ONE_LOCATION"]
@@ -28,26 +32,50 @@ ONE_LOCATION=ENV["ONE_LOCATION"]
 if !ONE_LOCATION
     RUBY_LIB_LOCATION="/usr/lib/one/ruby"
     VMDIR="/var/lib/one"
+    CONFIG_FILE="/var/lib/one/config"
 else
     RUBY_LIB_LOCATION=ONE_LOCATION+"/lib/ruby"
     VMDIR=ONE_LOCATION+"/var"
+    CONFIG_FILE=ONE_LOCATION+"/var/config"
 end
 
 $: << RUBY_LIB_LOCATION
 
-require 'OpenNebula'
+require 'opennebula'
 include OpenNebula
 
+require 'getoptlong'
+
 if !(host_id=ARGV[0])
     exit -1
 end
 
-if !(mode=ARGV[1]) # By default, resubmit VMs
-    mode = "-r"
-end
+mode   = "-r" # By default, recreate VMs
+force  = "n"  # By default, don't recreate/delete suspended VMs
+repeat = nil  # By default, don't wait for monitorization cycles"
 
-if !(force=ARGV[2]) # By default, don't resubmit/finalize suspended VMs
-    force = "n"
+opts = GetoptLong.new(
+            ['--delete',   '-d',GetoptLong::NO_ARGUMENT],
+            ['--recreate', '-r',GetoptLong::NO_ARGUMENT],
+            ['--force',    '-f',GetoptLong::NO_ARGUMENT],
+            ['--pause',    '-p',GetoptLong::REQUIRED_ARGUMENT]
+        )
+
+begin
+    opts.each do |opt, arg|
+        case opt
+            when '--delete'
+                mode="-d"
+            when '--recreate'
+                mode="-r"
+            when '--force'
+                force  = "y"
+            when '--pause'
+                repeat = arg.to_i
+        end
+    end
+rescue Exception => e
+    exit(-1)
 end
 
 begin
@@ -59,15 +87,28 @@ end
 
 # Retrieve hostname
 host  =  OpenNebula::Host.new_with_id(host_id, client)
-exit -1 if OpenNebula.is_error?(host)
-host.info
+rc = host.info
+exit -1 if OpenNebula.is_error?(rc)
 host_name = host.name
 
+if repeat
+    # Retrieve host monitor interval
+    monitor_interval = nil
+    File.read(CONFIG_FILE).each{|line|
+         monitor_interval = line.split("=").last.to_i if /MONITORING_INTERVAL/=~line
+    }
+    # Sleep through the desired number of monitor interval
+    sleep (repeat * monitor_interval)
+
+    # If the host came back, exit! avoid duplicated VMs
+    exit 0 if host.state != 3
+end
+
 # Loop through all vms
 vms = VirtualMachinePool.new(client)
-exit -1 if OpenNebula.is_error?(vms)
+rc = vms.info_all
+exit -1 if OpenNebula.is_error?(rc)
 
-vms.info_all
 
 state = "STATE=3"
 state += " or STATE=5" if force == "y"
@@ -80,9 +121,9 @@ if vm_ids_array
         vm.info
 
         if mode == "-r"
-            vm.resubmit
+            vm.delete(true)
         elsif mode == "-d"
-            vm.finalize
+            vm.delete
         end
     end
 end
