@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,18 +15,55 @@
 /* -------------------------------------------------------------------------- */
 
 #include "AclRule.h"
-#include "AuthManager.h"
+#include "AuthRequest.h"
 #include "PoolObjectSQL.h"
-    
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 const long long AclRule::INDIVIDUAL_ID  = 0x0000000100000000LL;
 const long long AclRule::GROUP_ID       = 0x0000000200000000LL;
 const long long AclRule::ALL_ID         = 0x0000000400000000LL;
+const long long AclRule::CLUSTER_ID     = 0x0000000800000000LL;
 
 const long long AclRule::NONE_ID        = 0x1000000000000000LL;
 
+const int AclRule::num_pool_objects = 11;
+const PoolObjectSQL::ObjectType AclRule::pool_objects[] = {
+            PoolObjectSQL::VM,
+            PoolObjectSQL::HOST,
+            PoolObjectSQL::NET,
+            PoolObjectSQL::IMAGE,
+            PoolObjectSQL::USER,
+            PoolObjectSQL::TEMPLATE,
+            PoolObjectSQL::GROUP,
+            PoolObjectSQL::DATASTORE,
+            PoolObjectSQL::CLUSTER,
+            PoolObjectSQL::DOCUMENT,
+            PoolObjectSQL::ZONE
+};
+
+const int AclRule::num_auth_operations = 4;
+const AuthRequest::Operation AclRule::auth_operations[] = {
+            AuthRequest::USE,
+            AuthRequest::MANAGE,
+            AuthRequest::ADMIN,
+            AuthRequest::CREATE
+};
+
+const long long AclRule::INVALID_CLUSTER_OBJECTS =
+        PoolObjectSQL::VM | PoolObjectSQL::IMAGE | PoolObjectSQL::USER |
+        PoolObjectSQL::TEMPLATE | PoolObjectSQL::GROUP | PoolObjectSQL::ACL |
+        PoolObjectSQL::CLUSTER | PoolObjectSQL::DOCUMENT | PoolObjectSQL::ZONE;
+
+const long long AclRule::INVALID_GROUP_OBJECTS =
+        PoolObjectSQL::HOST | PoolObjectSQL::GROUP | PoolObjectSQL::CLUSTER |
+        PoolObjectSQL::ZONE;
+
+const long long AclRule::FEDERATED_OBJECTS =
+        PoolObjectSQL::USER | PoolObjectSQL::GROUP | PoolObjectSQL::ZONE |
+        PoolObjectSQL::ACL;
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -34,6 +71,7 @@ bool AclRule::malformed(string& error_str) const
 {
     ostringstream oss;
     bool error = false;
+    long long resource_type;
 
     // Check user
 
@@ -100,7 +138,11 @@ bool AclRule::malformed(string& error_str) const
 
     // Check resource
 
-    if ( (resource & INDIVIDUAL_ID) != 0 && (resource & GROUP_ID) != 0 )
+    if ( ( (resource & INDIVIDUAL_ID) != 0 && (resource & 0xF00000000LL) != INDIVIDUAL_ID ) ||
+         ( (resource & GROUP_ID)      != 0 && (resource & 0xF00000000LL) != GROUP_ID ) ||
+         ( (resource & CLUSTER_ID)    != 0 && (resource & 0xF00000000LL) != CLUSTER_ID ) ||
+         ( (resource & ALL_ID)        != 0 && (resource & 0xF00000000LL) != ALL_ID )
+        )
     {
         if ( error )
         {
@@ -108,10 +150,13 @@ bool AclRule::malformed(string& error_str) const
         }
 
         error = true;
-        oss << "[resource] INDIVIDUAL (#) and GROUP (@) bits are exclusive";
+        oss << "[resource] INDIVIDUAL (#), GROUP (@), CLUSTER (%) "
+            << "and ALL (*) bits are exclusive";
     }
 
-    if ( (resource & INDIVIDUAL_ID) != 0 && (resource & ALL_ID) != 0 )
+    resource_type = resource_code() & 0xFFFFFFF000000000LL;
+
+    if ((resource & CLUSTER_ID) && (resource_type & INVALID_CLUSTER_OBJECTS))
     {
         if ( error )
         {
@@ -119,10 +164,13 @@ bool AclRule::malformed(string& error_str) const
         }
 
         error = true;
-        oss << "[resource] INDIVIDUAL (#) and ALL (*) bits are exclusive";
+        oss << "[resource] CLUSTER(%) selector can be applied only to "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::DATASTORE) << ", "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::HOST) << " and "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::NET) << " types";
     }
 
-    if ( (resource & GROUP_ID) != 0 && (resource & ALL_ID) != 0 )
+    if ((resource & GROUP_ID) && (resource_type & INVALID_GROUP_OBJECTS))
     {
         if ( error )
         {
@@ -130,10 +178,14 @@ bool AclRule::malformed(string& error_str) const
         }
 
         error = true;
-        oss << "[resource] GROUP (@) and ALL (*) bits are exclusive";
+        oss << "[resource] GROUP(@) selector cannot be applied to "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::HOST) << ", "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::GROUP) << ", "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::CLUSTER) << " or "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::ZONE) << " types";
     }
 
-    if ( (resource & 0x700000000LL) == 0 )
+    if ( (resource & 0xF00000000LL) == 0 )
     {
         if ( error )
         {
@@ -141,7 +193,7 @@ bool AclRule::malformed(string& error_str) const
         }
 
         error = true;
-        oss << "[resource] is missing one of the INDIVIDUAL, GROUP or ALL bits";
+        oss << "[resource] is missing one of the INDIVIDUAL, GROUP, CLUSTER or ALL bits";
     }
 
     if ( resource_id() < 0 )
@@ -212,6 +264,75 @@ bool AclRule::malformed(string& error_str) const
         oss << "wrong [rights], it cannot be bigger than 0xF";
     }
 
+    // Check zone
+
+    if ( (zone & GROUP_ID) != 0 )
+    {
+        error = true;
+        oss << "[zone] GROUP (@) bit is not supported";
+    }
+
+    if ( (zone & INDIVIDUAL_ID) != 0 && (zone & ALL_ID) != 0 )
+    {
+        if ( error )
+        {
+            oss << "; ";
+        }
+
+        error = true;
+        oss << "[zone] INDIVIDUAL (#) and ALL (*) bits are exclusive";
+    }
+
+    if ( (zone & 0x700000000LL) == 0 )
+    {
+        if ( error )
+        {
+            oss << "; ";
+        }
+
+        error = true;
+        oss << "[zone] is missing one of the INDIVIDUAL or ALL bits";
+    }
+
+    if ( zone_id() < 0 )
+    {
+        if ( error )
+        {
+            oss << "; ";
+        }
+
+        error = true;
+        oss << "[zone] ID cannot be negative";
+    }
+
+    if ( (zone & ALL_ID) != 0 && zone_id() != 0 )
+    {
+        if ( error )
+        {
+            oss << "; ";
+        }
+
+        error = true;
+        oss << "when using the ALL bit, [zone] ID must be 0";
+    }
+
+    if ((zone & ALL_ID) &&
+        (resource & INDIVIDUAL_ID) &&
+        ( (resource_type & FEDERATED_OBJECTS) != resource_type ) )
+    {
+        if ( error )
+        {
+            oss << "; ";
+        }
+
+        error = true;
+        oss << "[resource] INDIVIDUAL(#) selector cannot be applied "
+            << "to ALL zones, except for "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::USER) << ", "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::GROUP) << " and "
+            << PoolObjectSQL::type_to_str(PoolObjectSQL::ZONE) << " types";
+    }
+
     if ( error )
     {
         error_str = oss.str();
@@ -246,30 +367,18 @@ void AclRule::build_str()
 
     oss << " ";
 
-    PoolObjectSQL::ObjectType objects[] = {
-            PoolObjectSQL::VM,
-            PoolObjectSQL::HOST,
-            PoolObjectSQL::NET,
-            PoolObjectSQL::IMAGE,
-            PoolObjectSQL::USER,
-            PoolObjectSQL::TEMPLATE,
-            PoolObjectSQL::GROUP,
-            PoolObjectSQL::DATASTORE,
-            PoolObjectSQL::CLUSTER
-    };
-
     bool prefix = false;
 
-    for ( int i = 0; i < 9; i++ )
+    for ( int i = 0; i < num_pool_objects; i++ )
     {
-        if ( (resource & objects[i]) != 0 )
+        if ( (resource & pool_objects[i]) != 0 )
         {
             if ( prefix )
             {
                 oss << "+";
             }
 
-            oss << PoolObjectSQL::type_to_str( objects[i] );
+            oss << PoolObjectSQL::type_to_str( pool_objects[i] );
             prefix = true;
         }
     }
@@ -284,6 +393,10 @@ void AclRule::build_str()
     {
         oss << "#" << resource_id();
     }
+    else if ( (resource & CLUSTER_ID) != 0 )
+    {
+        oss << "%" << resource_id();
+    }
     else if ( (resource & ALL_ID) != 0 )
     {
         oss << "*";
@@ -293,33 +406,39 @@ void AclRule::build_str()
         oss << "??";
     }
 
-
     oss << " ";
 
-
-    AuthRequest::Operation operations[] = {
-            AuthRequest::USE,
-            AuthRequest::MANAGE,
-            AuthRequest::ADMIN,
-            AuthRequest::CREATE
-    };
-
     prefix = false;
 
-    for ( int i = 0; i < 4; i++ )
+    for ( int i = 0; i < num_auth_operations; i++ )
     {
-        if ( (rights & operations[i]) != 0 )
+        if ( (rights & auth_operations[i]) != 0 )
         {
             if ( prefix )
             {
                 oss << "+";
             }
 
-            oss << AuthRequest::operation_to_str( operations[i] );
+            oss << AuthRequest::operation_to_str( auth_operations[i] );
             prefix = true;
         }
     }
 
+    oss << " ";
+
+    if ( (zone & INDIVIDUAL_ID) != 0 )
+    {
+        oss << "#" << zone_id();
+    }
+    else if ( (zone & ALL_ID) != 0 )
+    {
+        oss << "*";
+    }
+    else
+    {
+        oss << "??";
+    }
+
     str = oss.str();
 }
 
@@ -336,6 +455,7 @@ string& AclRule::to_xml(string& xml) const
         "<USER>"     << hex << user      << "</USER>"        <<
         "<RESOURCE>" << hex << resource  << "</RESOURCE>"    <<
         "<RIGHTS>"   << hex << rights    << "</RIGHTS>"      <<
+        "<ZONE>"     << hex << zone      << "</ZONE>"        <<
         "<STRING>"   << str              << "</STRING>"      <<
     "</ACL>";
 
@@ -359,7 +479,7 @@ int AclRule::from_xml(xmlNodePtr node)
             break;
         }
 
-        xmlNodePtr elem = acl->children; 
+        xmlNodePtr elem = acl->children;
 
         if ( elem->type != XML_TEXT_NODE )
         {
@@ -386,6 +506,10 @@ int AclRule::from_xml(xmlNodePtr node)
         {
             iss >> hex >> rights;
         }
+        else if (name == "ZONE")
+        {
+            iss >> hex >> zone;
+        }
         else if (name == "STRING")
         {
             str = iss.str();
@@ -397,4 +521,3 @@ int AclRule::from_xml(xmlNodePtr node)
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
-
