@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -32,6 +32,7 @@ $: << RUBY_LIB_LOCATION
 require 'scripts_common'
 require 'OpenNebulaDriver'
 require 'getoptlong'
+require 'shellwords'
 
 # This is a generic AuthZ/AuthN driver able to manage multiple authentication 
 # protocols (simultaneosly). It also supports the definition of custom 
@@ -84,6 +85,16 @@ class AuthDriver < OpenNebulaDriver
         end
     end
 
+
+    # Works the same as log_method but changes the password by '****'.
+    # The last word is the password for authentication.
+    def log_method_no_password(num, secret)
+        lambda {|message|
+            m=message.gsub(/ #{Regexp.escape(secret)}$/, ' ****')
+            log(num, m)
+        }
+    end
+
     # Authenticate a user based in a string of the form user:secret when using the 
     # driver secret is protocol:token
     # @param [String] the id for this request, used by OpenNebula core
@@ -110,9 +121,12 @@ class AuthDriver < OpenNebulaDriver
         authN_path = File.join(@local_scripts_path, driver)
 
         command = File.join(authN_path, ACTION[:authN].downcase)
-        command << " '" << user.gsub("'", '\'"\'"\'') << "' '" << password.gsub("'", '\'"\'"\'') << "' " << secret
+        command << ' ' << ([user, password, secret].map do |p|
+            Shellwords.escape(p)
+        end.join(' '))
 
-        rc = LocalCommand.run(command, log_method(request_id))
+        rc = LocalCommand.run(command,
+            log_method_no_password(request_id, Shellwords.escape(secret)))
 
         result , info = get_info_from_execution(rc)
 
