@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -15,6 +15,7 @@
 #--------------------------------------------------------------------------- #
 
 require 'one_helper'
+require 'one_helper/onevm_helper'
 
 class OneVNetHelper < OpenNebulaHelper::OneHelper
     def self.rname
@@ -40,12 +41,12 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
             end
 
             column :USER, "Username of the Virtual Network owner", :left,
-                    :size=>8 do |d|
+                    :size=>12 do |d|
                 helper.user_name(d, options)
             end
 
             column :GROUP, "Group of the Virtual Network", :left,
-                    :size=>8 do |d|
+                    :size=>12 do |d|
                 helper.group_name(d, options)
             end
 
@@ -54,7 +55,7 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
                 d["NAME"]
             end
 
-            column :CLUSTER, "Name of the Cluster", :left, :size=>8 do |d|
+            column :CLUSTER, "Name of the Cluster", :left, :size=>10 do |d|
                 OpenNebulaHelper.cluster_str(d["CLUSTER"])
             end 
 
@@ -66,8 +67,8 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
                 d["SIZE"]
             end
 
-            column :BRIDGE, "Bridge associated to the Virtual Network",
-                    :size=>6 do |d|
+            column :BRIDGE, "Bridge associated to the Virtual Network", :left,
+                    :size=>8 do |d|
                 d["BRIDGE"]
             end
 
@@ -76,7 +77,7 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
                 d["TOTAL_LEASES"]
             end
 
-            default :ID, :USER, :GROUP, :NAME, :TYPE, :BRIDGE, :LEASES
+            default :ID, :USER, :GROUP, :NAME, :CLUSTER, :TYPE, :BRIDGE, :LEASES
         end
 
         table
@@ -97,7 +98,7 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
         OpenNebula::VirtualNetworkPool.new(@client, user_flag)
     end
 
-    def format_resource(vn)
+    def format_resource(vn, options = {})
         str_h1="%-80s"
         CLIHelper.print_header(str_h1 %
             ["VIRTUAL NETWORK #{vn.id.to_s} INFORMATION"])
@@ -111,8 +112,10 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
         puts str % ["TYPE", vn.type_str]
         puts str % ["BRIDGE", vn["BRIDGE"]]
         puts str % ["VLAN", OpenNebulaHelper.boolean_to_str(vn['VLAN'])]
-        puts str % ["PHYSICAL DEVICE", vn["PHYDEV"]] if vn["PHYDEV"]
-        puts str % ["VLAN ID", vn["VLAN_ID"]] if vn["VLAN_ID"]
+        puts str % ["PHYSICAL DEVICE", vn["PHYDEV"]] if !vn["PHYDEV"].empty?
+        puts str % ["VLAN ID", vn["VLAN_ID"]] if !vn["VLAN_ID"].empty?
+        puts str % ["GLOBAL PREFIX", vn["GLOBAL_PREFIX"]] if !vn["GLOBAL_PREFIX"].empty?
+        puts str % ["SITE PREFIX", vn["SITE_PREFIX"]] if !vn["SITE_PREFIX"].empty?
         puts str % ["USED LEASES", vn['TOTAL_LEASES']]
         puts
 
@@ -152,5 +155,19 @@ class OneVNetHelper < OpenNebulaHelper::OneHelper
                 puts leases_str
             end
         }
+
+        puts
+        CLIHelper.print_header("VIRTUAL MACHINES", false)
+        puts
+
+        vms=vn.retrieve_elements("LEASES/LEASE/VID")
+
+        if vms
+            vms=vms.delete_if {|vm| vm=="-1" }
+            vms.map!{|e| e.to_i }
+            onevm_helper=OneVMHelper.new
+            onevm_helper.client=@client
+            onevm_helper.list_pool({:ids=>vms}, false)
+        end
     end
 end
