@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -30,11 +30,8 @@ require 'ImagePoolOCCI'
 require 'UserOCCI'
 require 'UserPoolOCCI'
 
-require 'OpenNebulaVNC'
-
 require 'pp'
 
-
 ##############################################################################
 # The OCCI Server provides an OCCI implementation based on the
 # OpenNebula Engine
@@ -58,7 +55,7 @@ class OCCIServer < CloudServer
         if config[:ssl_server]
             @base_url=config[:ssl_server]
         else
-            @base_url="http://#{config[:server]}:#{config[:port]}"
+            @base_url="http://#{config[:host]}:#{config[:port]}"
         end
 
         @client = client
@@ -298,6 +295,73 @@ class OCCIServer < CloudServer
         return "", 204
     end
 
+    # Attach a disk to an existing COMPUTE
+    # @param [Hash] request hash containing the data of the request
+    # @param [Hash] params hash containing the params of the request
+    # @param [OpenNebula::XMLElement] action_xml contains the body of the request
+    # @return [[String, OpenNebula:Error], Integer]
+    def attach_disk(request, params, action_xml)
+        # --- Get the VM ---
+        vm = VirtualMachineOCCI.new(
+                    VirtualMachine.build_xml(params[:id]),
+                    @client)
+
+        image_href = action_xml.attr('PARAMS/STORAGE','href')
+        if !image_href
+            error = OpenNebula::Error.new("You have to specify an STORAGE " <<
+                "to be attached")
+            return error, CloudServer::HTTP_ERROR_CODE[error.errno]
+        end
+
+        image_id = image_href.split('/').last
+        target = action_xml['PARAMS/TARGET']
+
+        if target
+            template = "DISK = [ IMAGE_ID = #{image_id}, TARGET = #{target} ]"
+        else
+            template = "DISK = [ IMAGE_ID = #{image_id}, DEV_PREFIX = sd ]"
+        end
+
+        # --- Attach DISK ---
+        result = vm.attachdisk(template)
+        if OpenNebula.is_error?(result)
+            return result, CloudServer::HTTP_ERROR_CODE[result.errno]
+        end
+
+        vm.info
+
+        return to_occi_xml(vm, :code=>202)
+    end
+
+    # Detach a DISK from an existing COMOUTE
+    # @param [Hash] request hash containing the data of the request
+    # @param [Hash] params hash containing the params of the request
+    # @param [OpenNebula::XMLElement] action_xml contains the body of the request
+    # @return [[String, OpenNebula:Error], Integer]
+    def detach_disk(request, params, action_xml)
+        # --- Get the VM ---
+        vm = VirtualMachineOCCI.new(
+                    VirtualMachine.build_xml(params[:id]),
+                    @client)
+
+        disk_id = action_xml.attr('PARAMS/DISK','id')
+        if !disk_id
+            error = OpenNebula::Error.new("You have to specify a DISK " <<
+                "to be detached")
+            return error, CloudServer::HTTP_ERROR_CODE[error.errno]
+        end
+
+        # --- Detach DISK ---
+        result = vm.detachdisk(disk_id.to_i)
+        if OpenNebula.is_error?(result)
+            return result, CloudServer::HTTP_ERROR_CODE[result.errno]
+        end
+
+        vm.info
+
+        return to_occi_xml(vm, :code=>202)
+    end
+
     # Updates a COMPUTE resource
     # request:: _Hash_ hash containing the data of the request
     # [return] _String_,_Integer_ Update confirmation msg or error,
@@ -481,6 +545,31 @@ class OCCIServer < CloudServer
         return to_occi_xml(image, :code=>200)
     end
 
+    # Clone a STORAGE resource
+    # request:: _Hash_ hash containing the data of the request
+    # [return] _String_,_Integer_ STORAGE occi representation or error,
+    #                             status code
+    def clone_storage(request, params, action_xml)
+        # --- Get the Image ---
+        image = ImageOCCI.new(
+                        Image.build_xml(params[:id]),
+                        @client)
+
+        rc = image.clone(action_xml["PARAMS/NAME"])
+        if OpenNebula.is_error?(rc)
+            return rc, CloudServer::HTTP_ERROR_CODE[rc.errno]
+        end
+
+        new_image = ImageOCCI.new(
+                Image.build_xml(rc),
+                @client)
+
+        new_image.info
+
+        # --- Prepare XML Response ---
+        return to_occi_xml(new_image, :code=>201)
+    end
+
     # Deletes a STORAGE resource (Not yet implemented)
     # request:: _Hash_ hash containing the data of the request
     # [return] _String_,_Integer_ Delete confirmation msg or error,
@@ -553,33 +642,4 @@ class OCCIServer < CloudServer
 
         return to_occi_xml(user, :code=>200)
     end
-
-    ############################################################################
-    # VNC Methods
-    ############################################################################
-
-    def startvnc(id,config)
-        vm = VirtualMachineOCCI.new(VirtualMachine.build_xml(id), @client)
-        rc = vm.info
-
-        if OpenNebula.is_error?(rc)
-            error =  "Error starting VNC session, "
-            error << "could not retrieve Virtual Machine"
-            return [404, error]
-        end
-
-        vnc_proxy = OpenNebulaVNC.new(config, logger, {:json_errors => false})
-        return vnc_proxy.start(vm)
-    end
-
-    def stopvnc(pipe)
-        begin
-            OpenNebulaVNC.stop(pipe)
-        rescue Exception => e
-            logger.error {e.message}
-            return [500, "Error stopping VNC. Please check server logs."]
-        end
-        
-        return [200,nil]
-    end
 end
