@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,7 +15,7 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
-require 'OpenNebula'
+require 'opennebula'
 
 include OpenNebula
 
@@ -27,8 +28,8 @@ class VirtualMachineOCCI < VirtualMachine
             <CPU><%= self['TEMPLATE/CPU'] %></CPU>
             <MEMORY><%= self['TEMPLATE/MEMORY'] %></MEMORY>
             <NAME><%= self.name%></NAME>
-            <% if self['TEMPLATE/INSTANCE_TYPE'] %>
-            <INSTANCE_TYPE href="<%= base_url %>/instance_type/<%= self['TEMPLATE/INSTANCE_TYPE'] %>"><%= self['TEMPLATE/INSTANCE_TYPE'] %></INSTANCE_TYPE>
+            <% if self['USER_TEMPLATE/INSTANCE_TYPE'] %>
+            <INSTANCE_TYPE href="<%= base_url %>/instance_type/<%= self['USER_TEMPLATE/INSTANCE_TYPE'] %>"><%= self['USER_TEMPLATE/INSTANCE_TYPE'] %></INSTANCE_TYPE>
             <% end %>
             <STATE><%= self.state_str %></STATE>
             <% self.each('TEMPLATE/DISK') do |disk| %>
@@ -67,11 +68,15 @@ class VirtualMachineOCCI < VirtualMachine
     OCCI_ACTION = {
         "STOPPED"   => { :from => ["ACTIVE"], :action => :stop},
         "SUSPENDED" => { :from => ["ACTIVE"], :action => :suspend},
-        "RESUME"    => { :from => ["STOPPED", "SUSPENDED"], :action => :resume},
+        "RESUME"    => { :from => ["STOPPED", "SUSPENDED", "POWEROFF"], :action => :resume},
         "CANCEL"    => { :from => ["ACTIVE"], :action => :cancel},
         "REBOOT"    => { :from => ["ACTIVE"], :action => :reboot},
+        "RESET"     => { :from => ["ACTIVE"], :action => :reset},
         "SHUTDOWN"  => { :from => ["ACTIVE"], :action => :shutdown},
-        "DONE"      => { :from => VM_STATE, :action => :finalize}
+        "RESTART"   => { :from => ["ACTIVE"], :action => :restart},
+        "RESUBMIT"  => { :from => ["ACTIVE", "FAILED"], :action => :resubmit},
+        "POWEROFF"  => { :from => ["ACTIVE"], :action => :poweroff},
+        "DONE"      => { :from => VM_STATE,   :action => :finalize}
     }
 
     # Class constructor
