@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -28,11 +28,12 @@
 
 const string ClusterPool::NONE_CLUSTER_NAME = "";
 const int    ClusterPool::NONE_CLUSTER_ID   = -1;
+const int    ClusterPool::ALL_RESOURCES = 10;
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-ClusterPool::ClusterPool(SqlDB * db):PoolSQL(db, Cluster::table, true)
+ClusterPool::ClusterPool(SqlDB * db):PoolSQL(db, Cluster::table, true, true)
 {
     ostringstream oss;
     string        error_str;
@@ -50,19 +51,17 @@ ClusterPool::ClusterPool(SqlDB * db):PoolSQL(db, Cluster::table, true)
 
 int ClusterPool::allocate(string name, int * oid, string& error_str)
 {
-    Cluster *       cluster;
-    ostringstream   oss;
+    Cluster * cluster;
+    string    error;
 
-    if ( name.empty() )
+    ostringstream oss;
+
+    // Check name
+    if ( !PoolObjectSQL::name_is_valid(name, error_str) )
     {
         goto error_name;
     }
 
-    if ( name.length() > 128 )
-    {
-        goto error_name_length;
-    }
-
     // Check for duplicates
     cluster = get(name, false);
 
@@ -72,28 +71,21 @@ int ClusterPool::allocate(string name, int * oid, string& error_str)
     }
 
     // Build a new Cluster object
-    cluster = new Cluster(-1, name);
+    cluster = new Cluster(-1, name, 0);
 
     // Insert the Object in the pool
     *oid = PoolSQL::allocate(cluster, error_str);
 
     return *oid;
 
-error_name:
-    oss << "NAME cannot be empty.";
-    goto error_common;
-
-error_name_length:
-    oss << "NAME is too long; max length is 128 chars.";
-    goto error_common;
 
 error_duplicated:
     oss << "NAME is already taken by CLUSTER " << cluster->get_oid() << ".";
-
-error_common:
-    *oid = -1;
     error_str = oss.str();
 
+error_name:
+    *oid = -1;
+
     return *oid;
 }
 
