@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,6 +20,7 @@
 
 #include "Attribute.h"
 
+#define TO_UPPER(S) transform(S.begin(),S.end(),S.begin(),(int(*)(int))toupper)
 
 const char * VectorAttribute::magic_sep      = "@^_^@";
 const int    VectorAttribute::magic_sep_size = 5;
@@ -145,7 +146,7 @@ void VectorAttribute::unmarshall(const string& sattr, const char * _sep)
     	{
     		continue;
     	}
-        
+
         if ( mpos + 1 == tmp.size() )
         {
             attribute_value.insert(make_pair(tmp.substr(0,mpos),""));
@@ -154,7 +155,7 @@ void VectorAttribute::unmarshall(const string& sattr, const char * _sep)
         {
             attribute_value.insert(make_pair(tmp.substr(0,mpos),
                                              tmp.substr(mpos+1)));
-        } 
+        }
     }
 }
 /* -------------------------------------------------------------------------- */
@@ -173,18 +174,33 @@ void VectorAttribute::replace(const string& name, const string& value)
     map<string,string>::iterator it;
 
     it = attribute_value.find(name);
-    
+
     if ( it != attribute_value.end() )
     {
         attribute_value.erase(it);
     }
-    
+
     attribute_value.insert(make_pair(name,value));
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void VectorAttribute::remove(const string& name)
+{
+    map<string,string>::iterator it;
+
+    it = attribute_value.find(name);
+
+    if ( it != attribute_value.end() )
+    {
+        attribute_value.erase(it);
+    }
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 string VectorAttribute::vector_value(const char *name) const
 {
     map<string,string>::const_iterator it;
@@ -200,6 +216,37 @@ string VectorAttribute::vector_value(const char *name) const
         return it->second;
     }
 }
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VectorAttribute::vector_value(const char *name, bool& value) const
+{
+    map<string,string>::const_iterator it;
+
+    value = false;
+    it    = attribute_value.find(name);
+
+    if (it == attribute_value.end())
+    {
+        return -1;
+    }
+
+    if (it->second.empty())
+    {
+        return -1;
+    }
+
+    string tmp = it->second;
+
+    TO_UPPER(tmp);
+
+    if (tmp == "YES")
+    {
+        value = true;
+    }
+
+    return 0;
+}
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -223,5 +270,139 @@ int VectorAttribute::vector_value(const char *name, int & value) const
     istringstream iss(it->second);
     iss >> value;
 
+    if (iss.fail() || !iss.eof())
+    {
+        return -1;
+    }
+
     return 0;
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VectorAttribute::vector_value(const char *name, long long& value) const
+{
+    map<string,string>::const_iterator it;
+
+    it = attribute_value.find(name);
+
+    if ( it == attribute_value.end() )
+    {
+        return -1;
+    }
+
+    if ( it->second.empty() )
+    {
+        return -1;
+    }
+
+    istringstream iss(it->second);
+    iss >> value;
+
+    if (iss.fail() || !iss.eof())
+    {
+        return -1;
+    }
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VectorAttribute::vector_value(const char *name, float & value) const
+{
+    map<string,string>::const_iterator it;
+
+    it = attribute_value.find(name);
+
+    if ( it == attribute_value.end() )
+    {
+        return -1;
+    }
+
+    if ( it->second.empty() )
+    {
+        return -1;
+    }
+
+    istringstream iss(it->second);
+    iss >> value;
+
+    if (iss.fail() || !iss.eof())
+    {
+        return -1;
+    }
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string VectorAttribute::vector_value_str(const char *name, int& value) const
+{
+    map<string,string>::const_iterator it;
+
+    it = attribute_value.find(name);
+
+    if ( it == attribute_value.end() )
+    {
+        value = -1;
+        return  "";
+    }
+
+    if ( it->second.empty() )
+    {
+        value = -1;
+        return "";
+    }
+
+    istringstream iss(it->second);
+    iss >> value;
+
+    if (iss.fail() || !iss.eof())
+    {
+        value = -1;
+        return "";
+    }
+
+    return it->second;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+string VectorAttribute::vector_value_str(const char *name, float& value) const
+{
+    map<string,string>::const_iterator it;
+
+    it = attribute_value.find(name);
+
+    if ( it == attribute_value.end() )
+    {
+        value = -1;
+        return  "";
+    }
+
+    if ( it->second.empty() )
+    {
+        value = -1;
+        return "";
+    }
+
+    istringstream iss(it->second);
+    iss >> value;
+
+    if (iss.fail() || !iss.eof())
+    {
+        value = -1;
+        return "";
+    }
+
+    return it->second;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
