@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -21,7 +21,6 @@
 #    @param $1 - Datastore base_path
 #    @param $2 - Restricted directories
 #    @param $3 - Safe dirs
-#    @param $4 - Umask for new file creation (default: 0007)
 #    @return sets the following environment variables
 #      - RESTRICTED_DIRS: Paths that cannot be used to register images
 #      - SAFE_DIRS: Paths that are safe to specify image paths
@@ -34,7 +33,6 @@ function set_up_datastore {
 	BASE_PATH="$1"
 	RESTRICTED_DIRS="$2"
 	SAFE_DIRS="$3"
-	UMASK="$4"
 
 	if [ -z "${ONE_LOCATION}" ]; then
 	    VAR_LOCATION=/var/lib/one/
@@ -52,19 +50,13 @@ function set_up_datastore {
 	export BASE_PATH
 	export RESTRICTED_DIRS
 	export SAFE_DIRS
-
-	if [ -n "$UMASK" ]; then
-		umask $UMASK
-	else
-		umask 0007
-	fi
 }
 
 #-------------------------------------------------------------------------------
-# Generates an unique image path. Requires BASE_PATH to be set
-#   @return path for the image (empty if error)
+# Generates an unique image hash. Requires BASE_PATH to be set
+#   @return hash for the image (empty if error)
 #-------------------------------------------------------------------------------
-function generate_image_path {
+function generate_image_hash {
 
 	CANONICAL_STR="`$DATE +%s`:$ID"
 
@@ -80,30 +72,78 @@ EOF
 		exit 1
 	fi
 
+	echo "${IMAGE_HASH}"
+}
+
+#-------------------------------------------------------------------------------
+# Generates an unique image path. Requires BASE_PATH to be set
+#   @return path for the image (empty if error)
+#-------------------------------------------------------------------------------
+function generate_image_path {
+	IMAGE_HASH=`generate_image_hash`
 	echo "${BASE_PATH}/${IMAGE_HASH}"
 }
 
 #-------------------------------------------------------------------------------
-# Computes the size of an image
-#   @param $1 - Path to the image
-#   @return size of the image in Mb
+# Set up the arguments for the downloader script
+#   @param $1 - MD5 string
+#   @param $2 - SHA1 string
+#   @param $3 - NO_DECOMPRESS
+#   @param $4 - BW LIMIT
+#   @param $5 - SRC
+#   @param $6 - DST
+#   @return downloader.sh util arguments
 #-------------------------------------------------------------------------------
-function fs_du {
-	if [ -d "$1" ]; then
-		SIZE=`du -sb "$1" | cut -f1`
-		error=$?
-	else
-		SIZE=`stat -c %s "$1"`
-		error=$?
+function set_downloader_args {
+	HASHES=" "
+
+	if [ -n "$1" ]; then
+	    HASHES="--md5 $1"
 	fi
 
-	if [ $error -ne 0 ]; then
-		SIZE=0
-	else
-		SIZE=$((($SIZE+1048575)/1048576))
+	if [ -n "$2" ]; then
+	    HASHES="$HASHES --sha1 $2"
 	fi
 
-	echo "$SIZE"
+	if [ "$3" = "yes" -o "$3" = "Yes" -o "$3" = "YES" ]; then
+	    HASHES="$HASHES --nodecomp"
+	fi
+
+	if [ -n "$4" ]; then
+		HASHES="$HASHES --limit $4"
+	fi
+
+	echo "$HASHES $5 $6"
+}
+
+#------------------------------------------------------------------------------
+# Gets the size in bytes of a file
+#   @param $1 - Path to the image
+#   @return size of the image in bytes
+#------------------------------------------------------------------------------
+
+function file_size {
+    stat --version &> /dev/null
+
+    if [ $? = 0 ]; then
+        # Linux
+        STAT_CMD="stat -c %s"
+    else
+        # Darwin
+        STAT_CMD="stat -f %z"
+    fi
+
+    $STAT_CMD "$*"
+}
+
+#------------------------------------------------------------------------------
+# Gets the size in bytes of a gzipped file
+#   @param $1 - Path to the image
+#   @return size of the image in bytes
+#------------------------------------------------------------------------------
+
+function gzip_file_size {
+    gzip -l "$1" | tail -n 1 | awk '{print $2}'
 }
 
 #-------------------------------------------------------------------------------
@@ -111,15 +151,52 @@ function fs_du {
 #   @param $1 - Path to the image
 #   @return size of the image in Mb
 #-------------------------------------------------------------------------------
-function qemu_size {
-	DISK="$1"
+function fs_size {
 
-	SIZE=`$QEMU_IMG info $DISK|grep "^virtual size:"|\
-        sed 's/^.*(\([0-9]\+\) bytes.*$/\1/g'`
+    case $1 in
+    http://*)
+        HEADERS=`curl -LIk $1 2>&1`
 
-	SIZE=$((($SIZE+1048575)/1048576))
+        if echo "$HEADERS" | grep -q "OpenNebula-AppMarket-Size"; then
+            # An AppMarket/Marketplace URL
+            SIZE=$(echo "$HEADERS" | grep "^OpenNebula-AppMarket-Size:" | tail -n1 | cut -d: -f2)
+        else
+            # Not an AppMarket/Marketplace URL
+            SIZE=$(echo "$HEADERS" | grep "^Content-Length:" | tail -n1 | cut -d: -f2)
+        fi
+        error=$?
+        ;;
+    *)
+        if [ -d "$1" ]; then
+            SIZE=`du -sb "$1" | cut -f1`
+            error=$?
+        else
+            TYPE=$(cat "$1" | head -n 1024 | file -b - | tr A-Z a-z)
+            case "$TYPE" in
+            *gzip*)
+                SIZE=$(gzip_file_size "$1")
+                ;;
+            *qcow*)
+                SIZE=$($QEMU_IMG info "$1" | sed -n 's/.*(\([0-9]*\) bytes).*/\1/p')
+                ;;
+            *)
+                SIZE=$(file_size "$1")
+                ;;
+            esac
+            error=$?
+        fi
+        ;;
+    esac
 
-	echo "$SIZE"
+    SIZE=$(echo $SIZE | tr -d "\r")
+
+    if [ $error -ne 0 ]; then
+        SIZE=0
+    else
+        SIZE=$((($SIZE+1048575)/1048576))
+    fi
+
+    echo "$SIZE"
 }
 
 #-------------------------------------------------------------------------------
@@ -144,3 +221,22 @@ function check_restricted {
 
   	echo 0
 }
+
+#-------------------------------------------------------------------------------
+# Gets the ESX host to be used as bridge to register a VMware disk
+# Implements a round robin for the bridges
+#   @param $1 - Image ID to be used to round-robin between ESX Bridges
+#   @return host to be used as bridge
+#-------------------------------------------------------------------------------
+function get_destination_host {
+    HOSTS_ARRAY=($BRIDGE_LIST)
+    N_HOSTS=${#HOSTS_ARRAY[@]}
+
+    if [ -n "$1" ]; then
+        ARRAY_INDEX=$(($1 % ${N_HOSTS}))
+    else
+        ARRAY_INDEX=$((RANDOM % ${N_HOSTS}))
+    fi
+
+    echo ${HOSTS_ARRAY[$ARRAY_INDEX]}
+}
