@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -17,6 +17,8 @@
 #include "DispatchManager.h"
 #include "NebulaLog.h"
 
+#include "Nebula.h"
+
 void  DispatchManager::suspend_success_action(int vid)
 {
     VirtualMachine *    vm;
@@ -68,12 +70,21 @@ void  DispatchManager::stop_success_action(int vid)
     }
 
     if ((vm->get_state() == VirtualMachine::ACTIVE) &&
-        (vm->get_lcm_state() == VirtualMachine::EPILOG_STOP))
+        (vm->get_lcm_state() == VirtualMachine::EPILOG_STOP ||
+         vm->get_lcm_state() == VirtualMachine::PROLOG_RESUME))
     {
         vm->set_state(VirtualMachine::STOPPED);
 
         vm->set_state(VirtualMachine::LCM_INIT);
 
+        //Set history action field to perform the right TM command on resume
+        if (vm->get_action() == History::NONE_ACTION)
+        {
+            vm->set_action(History::STOP_ACTION);
+
+            vmpool->update_history(vm);
+        }
+
         vmpool->update(vm);
 
         vm->log("DiM", Log::INFO, "New VM state is STOPPED");
@@ -95,9 +106,100 @@ void  DispatchManager::stop_success_action(int vid)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void  DispatchManager::undeploy_success_action(int vid)
+{
+    VirtualMachine *    vm;
+
+    vm = vmpool->get(vid,true);
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    if ((vm->get_state() == VirtualMachine::ACTIVE) &&
+        (vm->get_lcm_state() == VirtualMachine::EPILOG_UNDEPLOY ||
+         vm->get_lcm_state() == VirtualMachine::PROLOG_UNDEPLOY))
+    {
+        vm->set_state(VirtualMachine::UNDEPLOYED);
+
+        vm->set_state(VirtualMachine::LCM_INIT);
+
+        //Set history action field to perform the right TM command on resume
+        if (vm->get_action() == History::NONE_ACTION)
+        {
+            vm->set_action(History::UNDEPLOY_ACTION);
+
+            vmpool->update_history(vm);
+        }
+
+        vmpool->update(vm);
+
+        vm->log("DiM", Log::INFO, "New VM state is UNDEPLOYED");
+    }
+    else
+    {
+        ostringstream oss;
+
+        oss << "undeploy_success action received but VM " << vid
+            << " not in ACTIVE state";
+        NebulaLog::log("DiM",Log::ERROR,oss);
+    }
+
+    vm->unlock();
+
+    return;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void  DispatchManager::poweroff_success_action(int vid)
+{
+    VirtualMachine *    vm;
+
+    vm = vmpool->get(vid,true);
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    if ((vm->get_state() == VirtualMachine::ACTIVE) &&
+        (vm->get_lcm_state() == VirtualMachine::SHUTDOWN_POWEROFF))
+    {
+        vm->set_state(VirtualMachine::POWEROFF);
+
+        vm->set_state(VirtualMachine::LCM_INIT);
+
+        vmpool->update(vm);
+
+        vm->log("DiM", Log::INFO, "New VM state is POWEROFF");
+    }
+    else
+    {
+        ostringstream oss;
+
+        oss << "poweroff_success action received but VM " << vid
+            << " not in ACTIVE state";
+        NebulaLog::log("DiM",Log::ERROR,oss);
+    }
+
+    vm->unlock();
+
+    return;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 void  DispatchManager::done_action(int vid)
 {
-    VirtualMachine *         vm;
+    VirtualMachine * vm;
+    Template *       tmpl;
+
+    int uid;
+    int gid;
 
     VirtualMachine::LcmState lcm_state;
     VirtualMachine::VmState  dm_state;
@@ -115,8 +217,12 @@ void  DispatchManager::done_action(int vid)
     if ((dm_state == VirtualMachine::ACTIVE) &&
           (lcm_state == VirtualMachine::EPILOG ||
            lcm_state == VirtualMachine::CANCEL ||
-           lcm_state == VirtualMachine::CLEANUP ))
+           lcm_state == VirtualMachine::CLEANUP_DELETE))
     {
+        vm->release_network_leases();
+
+        vm->release_disk_images();
+
         vm->set_state(VirtualMachine::DONE);
 
         vm->set_state(VirtualMachine::LCM_INIT);
@@ -127,9 +233,15 @@ void  DispatchManager::done_action(int vid)
 
         vm->log("DiM", Log::INFO, "New VM state is DONE");
 
-        vm->release_network_leases();
+        uid  = vm->get_uid();
+        gid  = vm->get_gid();
+        tmpl = vm->clone_template();
 
-        vm->release_disk_images();
+        vm->unlock();
+
+        Quotas::vm_del(uid, gid, tmpl);
+
+        delete tmpl;
     }
     else
     {
@@ -137,9 +249,9 @@ void  DispatchManager::done_action(int vid)
 
         oss << "done action received but VM " << vid << " not in ACTIVE state";
         NebulaLog::log("DiM",Log::ERROR,oss);
-    }
 
-    vm->unlock();
+        vm->unlock();
+    }
 
     return;
 }
@@ -191,7 +303,7 @@ void  DispatchManager::resubmit_action(int vid)
         return;
     }
 
-    if (vm->get_lcm_state() == VirtualMachine::CLEANUP)
+    if (vm->get_lcm_state() == VirtualMachine::CLEANUP_RESUBMIT)
     {
 
         vm->set_state(VirtualMachine::LCM_INIT);
