@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)           */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs      */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -27,6 +27,7 @@
 
 #include "AuthManager.h"
 #include "UserPool.h"
+#include "NebulaUtil.h"
 
 #define TO_UPPER(S) transform(S.begin(),S.end(),S.begin(),(int(*)(int))toupper)
 
@@ -38,9 +39,11 @@ Image::Image(int             _uid,
              int             _gid,
              const string&   _uname,
              const string&   _gname,
+             int             _umask,
              ImageTemplate * _image_template):
         PoolObjectSQL(-1,IMAGE,"",_uid,_gid,_uname,_gname,table),
         type(OS),
+        disk_type(FILE),
         regtime(time(0)),
         source(""),
         path(""),
@@ -48,8 +51,12 @@ Image::Image(int             _uid,
         size_mb(0),
         state(INIT),
         running_vms(0),
+        cloning_ops(0),
+        cloning_id(-1),
         ds_id(-1),
-        ds_name("")
+        ds_name(""),
+        vm_collection("VMS"),
+        img_clone_collection("CLONES")
 {
     if (_image_template != 0)
     {
@@ -59,14 +66,13 @@ Image::Image(int             _uid,
     {
         obj_template = new ImageTemplate;
     }
+
+    set_umask(_umask);
 }
 
 Image::~Image()
 {
-    if (obj_template != 0)
-    {
-        delete obj_template;
-    }
+    delete obj_template;
 }
 
 /* ************************************************************************ */
@@ -79,7 +85,7 @@ const char * Image::db_names =
         "oid, name, body, uid, gid, owner_u, group_u, other_u";
 
 const char * Image::db_bootstrap = "CREATE TABLE IF NOT EXISTS image_pool ("
-    "oid INTEGER PRIMARY KEY, name VARCHAR(128), body TEXT, uid INTEGER, "
+    "oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, "
     "gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, "
     "UNIQUE(name,uid) )";
 
@@ -96,7 +102,9 @@ int Image::insert(SqlDB *db, string& error_str)
     string dev_prefix;
     string source_attr;
     string saved_id;
+    string size_attr;
 
+    istringstream iss;
     ostringstream oss;
 
     // ---------------------------------------------------------------------
@@ -121,25 +129,53 @@ int Image::insert(SqlDB *db, string& error_str)
         goto error_type;
     }
 
-    // ------------ PERSISTENT --------------------
+    // ------------ PERSISTENT & PREFIX --------------------
 
     erase_template_attribute("PERSISTENT", persistent_attr);
 
-    TO_UPPER(persistent_attr);
-
-    persistent_img = (persistent_attr == "YES");
-
-    // ------------ PREFIX --------------------
-
-    get_template_attribute("DEV_PREFIX", dev_prefix);
-
-    if( dev_prefix.empty() )
+    switch (type)
     {
-        SingleAttribute * dev_att = new SingleAttribute("DEV_PREFIX",
-                                          ImagePool::default_dev_prefix());
-        obj_template->set(dev_att);
+        case OS:
+        case DATABLOCK:
+        case CDROM:
+            TO_UPPER(persistent_attr);
+            persistent_img = (persistent_attr == "YES");
+
+            get_template_attribute("DEV_PREFIX", dev_prefix);
+
+            if( dev_prefix.empty() )
+            {
+                if (type == CDROM)
+                {
+                    dev_prefix = ImagePool::default_cdrom_dev_prefix();
+                }
+                else
+                {
+                    dev_prefix = ImagePool::default_dev_prefix();
+                }
+
+                SingleAttribute * dev_att =
+                        new SingleAttribute("DEV_PREFIX", dev_prefix);
+
+                obj_template->set(dev_att);
+            }
+        break;
+
+        case KERNEL: // Files are always non-persistent with no dev_prefix
+        case RAMDISK:
+        case CONTEXT:
+            persistent_img = false;
+            erase_template_attribute("DEV_PREFIX", dev_prefix);
+        break;
     }
 
+    // ------------ SIZE --------------------
+
+    erase_template_attribute("SIZE", size_attr);
+
+    iss.str(size_attr);
+    iss >> size_mb;
+
     // ------------ PATH & SOURCE --------------------
 
     erase_template_attribute("PATH", path);
@@ -149,25 +185,17 @@ int Image::insert(SqlDB *db, string& error_str)
     {
         if ( source.empty() && path.empty() )
         {
-            string        size_attr;
-            istringstream iss;
-
-            erase_template_attribute("SIZE",   size_attr);
-            erase_template_attribute("FSTYPE", fs_type);
-
-            // DATABLOCK image needs SIZE and FSTYPE
-            if (type != DATABLOCK || size_attr.empty() || fs_type.empty())
+            if (type != DATABLOCK)
             {
                 goto error_no_path;
             }
 
-            iss.str(size_attr);
+            erase_template_attribute("FSTYPE", fs_type);
 
-            iss >> size_mb;
-
-            if (iss.fail() == true)
+            // DATABLOCK image needs FSTYPE
+            if (fs_type.empty())
             {
-                goto error_size_format;
+                fs_type = "raw";
             }
         }
         else if ( !source.empty() && !path.empty() )
@@ -177,20 +205,7 @@ int Image::insert(SqlDB *db, string& error_str)
     }
     else
     {
-        string        size_attr;
-        istringstream iss;
-
         fs_type = "save_as";
-        erase_template_attribute("SIZE",   size_attr);
-
-        iss.str(size_attr);
-
-        iss >> size_mb;
-
-        if (iss.fail() == true)
-        {
-            goto error_size_format;
-        }
     }
 
     state = LOCKED; //LOCKED till the ImageManager copies it to the Repository
@@ -206,21 +221,9 @@ int Image::insert(SqlDB *db, string& error_str)
 error_type:
     error_str = "Incorrect TYPE in template.";
     goto error_common;
-    
-error_no_path:
-    if ( type == DATABLOCK )
-    {
-        error_str = "A DATABLOCK type IMAGE has to declare a PATH, or both "
-                    "SIZE and FSTYPE.";
-    }
-    else
-    {
-        error_str = "No PATH in template.";
-    }
-    goto error_common;
 
-error_size_format:
-    error_str = "Wrong number in SIZE.";
+error_no_path:
+    error_str = "No PATH in template.";
     goto error_common;
 
 error_path_and_source:
@@ -334,17 +337,20 @@ string& Image::to_xml(string& xml) const
     string          template_xml;
     string          perms_xml;
     ostringstream   oss;
+    string          vm_collection_xml;
+    string          clone_collection_xml;
 
     oss <<
         "<IMAGE>" <<
             "<ID>"             << oid             << "</ID>"          <<
             "<UID>"            << uid             << "</UID>"         <<
             "<GID>"            << gid             << "</GID>"         <<
-            "<UNAME>"          << uname           << "</UNAME>"       << 
+            "<UNAME>"          << uname           << "</UNAME>"       <<
             "<GNAME>"          << gname           << "</GNAME>"       <<
             "<NAME>"           << name            << "</NAME>"        <<
             perms_to_xml(perms_xml)                                   <<
             "<TYPE>"           << type            << "</TYPE>"        <<
+            "<DISK_TYPE>"      << disk_type       << "</DISK_TYPE>"   <<
             "<PERSISTENT>"     << persistent_img  << "</PERSISTENT>"  <<
             "<REGTIME>"        << regtime         << "</REGTIME>"     <<
             "<SOURCE>"         << source          << "</SOURCE>"      <<
@@ -353,8 +359,12 @@ string& Image::to_xml(string& xml) const
             "<SIZE>"           << size_mb         << "</SIZE>"        <<
             "<STATE>"          << state           << "</STATE>"       <<
             "<RUNNING_VMS>"    << running_vms     << "</RUNNING_VMS>" <<
+            "<CLONING_OPS>"    << cloning_ops     << "</CLONING_OPS>" <<
+            "<CLONING_ID>"     << cloning_id      << "</CLONING_ID>"  <<
             "<DATASTORE_ID>"   << ds_id           << "</DATASTORE_ID>"<<
             "<DATASTORE>"      << ds_name         << "</DATASTORE>"   <<
+            vm_collection.to_xml(vm_collection_xml)                   <<
+            img_clone_collection.to_xml(clone_collection_xml)         <<
             obj_template->to_xml(template_xml)                        <<
         "</IMAGE>";
 
@@ -371,6 +381,7 @@ int Image::from_xml(const string& xml)
     vector<xmlNodePtr> content;
     int int_state;
     int int_type;
+    int int_disk_type;
 
     int rc = 0;
 
@@ -378,26 +389,29 @@ int Image::from_xml(const string& xml)
     update_from_str(xml);
 
     // Get class base attributes
-    rc += xpath(oid, "/IMAGE/ID",  -1);
-    rc += xpath(uid, "/IMAGE/UID", -1);
-    rc += xpath(gid, "/IMAGE/GID", -1);
+    rc += xpath(oid,    "/IMAGE/ID",  -1);
+    rc += xpath(uid,    "/IMAGE/UID", -1);
+    rc += xpath(gid,    "/IMAGE/GID", -1);
 
-    rc += xpath(uname, "/IMAGE/UNAME", "not_found");
-    rc += xpath(gname, "/IMAGE/GNAME", "not_found");
+    rc += xpath(uname,  "/IMAGE/UNAME", "not_found");
+    rc += xpath(gname,  "/IMAGE/GNAME", "not_found");
 
-    rc += xpath(name, "/IMAGE/NAME", "not_found");
+    rc += xpath(name,   "/IMAGE/NAME", "not_found");
 
-    rc += xpath(int_type, "/IMAGE/TYPE", 0);
-    rc += xpath(persistent_img, "/IMAGE/PERSISTENT", 0);
-    rc += xpath(regtime, "/IMAGE/REGTIME", 0);
+    rc += xpath(int_type,       "/IMAGE/TYPE",      0);
+    rc += xpath(int_disk_type,  "/IMAGE/DISK_TYPE", 0);
+    rc += xpath(persistent_img, "/IMAGE/PERSISTENT",0);
+    rc += xpath(regtime,        "/IMAGE/REGTIME",   0);
 
-    rc += xpath(source, "/IMAGE/SOURCE", "not_found");
-    rc += xpath(size_mb, "/IMAGE/SIZE", 0);
-    rc += xpath(int_state, "/IMAGE/STATE", 0);
-    rc += xpath(running_vms, "/IMAGE/RUNNING_VMS", -1);
+    rc += xpath(source,         "/IMAGE/SOURCE",        "not_found");
+    rc += xpath(size_mb,        "/IMAGE/SIZE",          0);
+    rc += xpath(int_state,      "/IMAGE/STATE",         0);
+    rc += xpath(running_vms,    "/IMAGE/RUNNING_VMS",   -1);
+    rc += xpath(cloning_ops,    "/IMAGE/CLONING_OPS",   -1);
+    rc += xpath(cloning_id,     "/IMAGE/CLONING_ID",    -1);
 
-    rc += xpath(ds_id,  "/IMAGE/DATASTORE_ID", -1);
-    rc += xpath(ds_name,"/IMAGE/DATASTORE", "not_found");
+    rc += xpath(ds_id,          "/IMAGE/DATASTORE_ID",  -1);
+    rc += xpath(ds_name,        "/IMAGE/DATASTORE",     "not_found");
 
     // Permissions
     rc += perms_from_xml();
@@ -406,8 +420,9 @@ int Image::from_xml(const string& xml)
     xpath(path,"/IMAGE/PATH", "");
     xpath(fs_type,"/IMAGE/FSTYPE","");
 
-    type  = static_cast<ImageType>(int_type);
-    state = static_cast<ImageState>(int_state);
+    type      = static_cast<ImageType>(int_type);
+    disk_type = static_cast<DiskType>(int_disk_type);
+    state     = static_cast<ImageState>(int_state);
 
     // Get associated classes
     ObjectXML::get_nodes("/IMAGE/TEMPLATE", content);
@@ -421,6 +436,33 @@ int Image::from_xml(const string& xml)
 
     ObjectXML::free_nodes(content);
 
+    content.clear();
+
+    ObjectXML::get_nodes("/IMAGE/VMS", content);
+
+    if (content.empty())
+    {
+        return -1;
+    }
+
+    rc += vm_collection.from_xml_node(content[0]);
+
+    ObjectXML::free_nodes(content);
+
+    content.clear();
+
+    ObjectXML::get_nodes("/IMAGE/CLONES", content);
+
+    if (content.empty())
+    {
+        return -1;
+    }
+
+    rc += img_clone_collection.from_xml_node(content[0]);
+
+    ObjectXML::free_nodes(content);
+
+
     if (rc != 0)
     {
         return -1;
@@ -432,116 +474,175 @@ int Image::from_xml(const string& xml)
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
-int Image::disk_attribute(  VectorAttribute * disk,
-                            int *             index,
-                            ImageType*        img_type)
+int Image::disk_attribute(  VectorAttribute *       disk,
+                            ImageType&              img_type,
+                            string&                 dev_prefix,
+                            const vector<string>&   inherit_attrs)
 {
-    string  bus;
-    string  target;
-    string  driver;
+    string target;
+    string driver;
+    string disk_attr_type;
+    string inherit_val;
 
-    ostringstream  iid;
+    bool ro;
 
-    *img_type = type;
-    bus       = disk->vector_value("BUS");
-    target    = disk->vector_value("TARGET");
-    driver    = disk->vector_value("DRIVER");
+    ostringstream iid;
+
+    vector<string>::const_iterator it;
+
+    img_type   = type;
+    target     = disk->vector_value("TARGET");
+    driver     = disk->vector_value("DRIVER");
+    dev_prefix = disk->vector_value("DEV_PREFIX");
     iid << oid;
 
-    string template_bus;
     string template_target;
-    string prefix;
     string template_driver;
+    string template_ptype;
 
-    get_template_attribute("BUS", template_bus);
     get_template_attribute("TARGET", template_target);
     get_template_attribute("DRIVER", template_driver);
+    get_template_attribute("PERSISTENT_TYPE", template_ptype);
 
-    get_template_attribute("DEV_PREFIX", prefix);
+    TO_UPPER(template_ptype);
 
-    if (prefix.empty())//Removed from image template, get it again from defaults
+    //---------------------------------------------------------------------------
+    //                       DEV_PREFIX ATTRIBUTE
+    //---------------------------------------------------------------------------
+    if ( dev_prefix.empty() ) //DEV_PEFIX not in DISK, check for it in IMAGE
     {
-        prefix = ImagePool::default_dev_prefix();
+        get_template_attribute("DEV_PREFIX", dev_prefix);
+
+        if (dev_prefix.empty())//Removed from image template, get it again
+        {
+            if ( type == CDROM )
+            {
+                dev_prefix = ImagePool::default_cdrom_dev_prefix();
+            }
+            else
+            {
+                dev_prefix = ImagePool::default_dev_prefix();
+            }
+        }
+
+        disk->replace("DEV_PREFIX", dev_prefix);
     }
 
-   //---------------------------------------------------------------------------
-   //                       BASE DISK ATTRIBUTES
-   //---------------------------------------------------------------------------
-
+    //--------------------------------------------------------------------------
+    //                       BASE DISK ATTRIBUTES
+    //--------------------------------------------------------------------------
     disk->replace("IMAGE",    name);
     disk->replace("IMAGE_ID", iid.str());
     disk->replace("SOURCE",   source);
-
-    if (bus.empty() && !template_bus.empty()) //BUS in Image, not in DISK
-    {
-        disk->replace("BUS",template_bus);
-    }
+    disk->replace("SIZE",     size_mb);
 
     if (driver.empty() && !template_driver.empty())//DRIVER in Image,not in DISK
     {
         disk->replace("DRIVER",template_driver);
     }
 
-   //---------------------------------------------------------------------------
-   //   TYPE, READONLY, CLONE, and SAVE attributes
-   //---------------------------------------------------------------------------
+    //--------------------------------------------------------------------------
+    //   READONLY attribute
+    //--------------------------------------------------------------------------
+    if ( type == CDROM || template_ptype == "IMMUTABLE" )
+    {
+        disk->replace("READONLY", "YES");
+    }
+    else if ( disk->vector_value("READONLY", ro) != 0 )
+    {
+        if ( get_template_attribute("READONLY", ro) )
+        {
+            disk->replace("READONLY", ro);
+        }
+        else
+        {
+            disk->replace("READONLY", "NO");
+        }
+    }
 
+    //--------------------------------------------------------------------------
+    //   CLONE & SAVE attributes
+    //--------------------------------------------------------------------------
     if ( persistent_img )
     {
-        disk->replace("CLONE","NO");
-        disk->replace("SAVE","YES");
-        disk->replace("PERSISTENT","YES");
+        disk->replace("PERSISTENT", "YES");
+        disk->replace("CLONE", "NO");
+
+        if ( template_ptype == "IMMUTABLE" )
+        {
+            disk->replace("SAVE", "NO");
+        }
+        else
+        {
+            disk->replace("SAVE", "YES");
+        }
     }
     else
     {
-        disk->replace("CLONE","YES");
-        disk->replace("SAVE","NO");
+        if ( type == CDROM )
+        {
+            disk->replace("CLONE", "NO");
+        }
+        else
+        {
+            disk->replace("CLONE", "YES");
+        }
+
+        disk->replace("SAVE", "NO");
     }
 
+    //--------------------------------------------------------------------------
+    //   TYPE attribute
+    //--------------------------------------------------------------------------
     switch(type)
     {
         case OS:
         case DATABLOCK:
-          disk->replace("TYPE","DISK");
-          disk->replace("READONLY","NO");
-        break;
+            disk_attr_type = disk_type_to_str(disk_type);
+            break;
 
-        case CDROM:
-          disk->replace("TYPE","CDROM");
-          disk->replace("READONLY","YES");
-        break;
-    }
+        case CDROM: //Always use CDROM type for these ones
+            DiskType new_disk_type;
 
-   //---------------------------------------------------------------------------
-   //   TARGET attribute
-   //---------------------------------------------------------------------------
-
-    if (target.empty()) //No TARGET in DISK attribute
-    {
-        if (!template_target.empty())
-        {
-            disk->replace("TARGET", template_target);
-        }
-        else
-        {
-            switch(type)
+            switch(disk_type)
             {
-                case OS:
-                    prefix += "a";
-                break;
+                case RBD:
+                    new_disk_type = RBD_CDROM;
+                    break;
 
-                case CDROM:
-                    prefix += "c"; // b is for context
-                break;
-
-                case DATABLOCK:
-                    prefix += static_cast<char>(('e'+ *index));
-                    *index  = *index + 1;
-                break;
+                case GLUSTER:
+                    new_disk_type = GLUSTER_CDROM;
+                    break;
 
+                default:
+                    new_disk_type = CD_ROM;
             }
 
-            disk->replace("TARGET", prefix);
+            disk_attr_type = disk_type_to_str(new_disk_type);
+            break;
+
+        default: //Other file types should not be never a DISK
+            break;
+    }
+
+    disk->replace("TYPE", disk_attr_type);
+
+    //--------------------------------------------------------------------------
+    //   TARGET attribute
+    //--------------------------------------------------------------------------
+    // TARGET defined in the Image template, but not in the DISK attribute
+    if ( target.empty() && !template_target.empty() )
+    {
+        disk->replace("TARGET", template_target);
+    }
+
+    for (it = inherit_attrs.begin(); it != inherit_attrs.end(); it++)
+    {
+        get_template_attribute((*it).c_str(), inherit_val);
+
+        if (!inherit_val.empty())
+        {
+            disk->replace(*it, inherit_val);
         }
     }
 
@@ -569,6 +670,18 @@ int Image::set_type(string& _type)
     {
         type = DATABLOCK;
     }
+    else if ( _type == "KERNEL" )
+    {
+        type = KERNEL;
+    }
+    else if ( _type == "RAMDISK" )
+    {
+        type = RAMDISK;
+    }
+    else if ( _type == "CONTEXT" )
+    {
+        type = CONTEXT;
+    }
     else
     {
         rc = -1;
@@ -579,3 +692,102 @@ int Image::set_type(string& _type)
 
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
+
+ImageTemplate * Image::clone_template(const string& new_name) const
+{
+
+    ImageTemplate * tmpl = new ImageTemplate(
+            *(static_cast<ImageTemplate *>(obj_template)));
+
+    tmpl->replace("NAME",   new_name);
+    tmpl->replace("TYPE",   type_to_str(type));
+    tmpl->replace("PATH",   source);
+    tmpl->replace("FSTYPE", fs_type);
+    tmpl->replace("SIZE",   size_mb);
+
+    if ( isPersistent() )
+    {
+        tmpl->replace("PERSISTENT", "YES");
+    }
+    else
+    {
+        tmpl->replace("PERSISTENT", "NO");
+    }
+
+    return tmpl;
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+Image::ImageType Image::str_to_type(string& str_type)
+{
+    Image::ImageType it = OS;
+
+    if (str_type.empty())
+    {
+        str_type = ImagePool::default_type();
+    }
+
+    TO_UPPER(str_type);
+
+    if ( str_type == "OS" )
+    {
+        it = OS;
+    }
+    else if ( str_type == "CDROM" )
+    {
+        it = CDROM;
+    }
+    else if ( str_type == "DATABLOCK" )
+    {
+        it = DATABLOCK;
+    }
+    else if ( str_type == "KERNEL" )
+    {
+        it = KERNEL;
+    }
+    else if ( str_type == "RAMDISK" )
+    {
+        it = RAMDISK;
+    }
+    else if ( str_type == "CONTEXT" )
+    {
+        it = CONTEXT;
+    }
+
+    return it;
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+Image::DiskType Image::str_to_disk_type(string& s_disk_type)
+{
+    Image::DiskType type = NONE;
+
+    one_util::toupper(s_disk_type);
+
+    if (s_disk_type == "FILE")
+    {
+        type = Image::FILE;
+    }
+    else if (s_disk_type == "BLOCK")
+    {
+        type = Image::BLOCK;
+    }
+    else if (s_disk_type == "CDROM")
+    {
+        type = Image::CD_ROM;
+    }
+    else if (s_disk_type == "RBD")
+    {
+        type = Image::RBD;
+    }
+    else if (s_disk_type == "GLUSTER")
+    {
+        type = Image::GLUSTER;
+    }
+
+    return type;
+}
