@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -108,7 +108,7 @@ public abstract class Pool{
         return xmlrpcInfo(client, infoMethod, MINE_GROUP, -1, -1);
     }
 
-    private static OneResponse xmlrpcInfo(Client client, String infoMethod, Object...args)
+    protected static OneResponse xmlrpcInfo(Client client, String infoMethod, Object...args)
     {
         return client.call(infoMethod, args);
     }
@@ -153,7 +153,7 @@ public abstract class Pool{
      * representation of the pool.
      * @param info The XML-RPC *pool.info response
      */
-    public void processInfo(OneResponse info)
+    protected void processInfo(OneResponse info)
     {
         if (info.isError())
         {
