@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,6 +52,7 @@ public class Acl extends PoolElement{
         tmpUsers.put("#", 0x0000000100000000L);
         tmpUsers.put("@", 0x0000000200000000L);
         tmpUsers.put("*", 0x0000000400000000L);
+        tmpUsers.put("%", 0x0000000800000000L);
 
         USERS = Collections.unmodifiableMap(tmpUsers);
 
@@ -66,6 +67,8 @@ public class Acl extends PoolElement{
         tmpResources.put("GROUP"    , 0x0000040000000000L);
         tmpResources.put("DATASTORE", 0x0000100000000000L);
         tmpResources.put("CLUSTER"  , 0x0000200000000000L);
+        tmpResources.put("DOCUMENT" , 0x0000400000000000L);
+        tmpResources.put("ZONE"     , 0x0000800000000000L);
 
         RESOURCES = Collections.unmodifiableMap(tmpResources);
 
@@ -338,7 +341,7 @@ public class Acl extends PoolElement{
      */
     private static long calculateIds(String id) throws RuleParseException
     {
-        if( !id.matches("^([#@]\\d+|\\*)$") )
+        if( !id.matches("^([#@%]\\d+|\\*)$") )
         {
             throw new RuleParseException("ID string '" + id + "' malformed");
         }
