@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,26 +28,30 @@ public class Image extends PoolElement
 {
 
     private static final String METHOD_PREFIX = "image.";
-    private static final String ALLOCATE = METHOD_PREFIX + "allocate";
-    private static final String INFO     = METHOD_PREFIX + "info";
-    private static final String DELETE   = METHOD_PREFIX + "delete";
-    private static final String UPDATE   = METHOD_PREFIX + "update";
-    private static final String ENABLE   = METHOD_PREFIX + "enable";
-    private static final String CHOWN    = METHOD_PREFIX + "chown";
-    private static final String CHMOD    = METHOD_PREFIX + "chmod";
-    private static final String CHTYPE   = METHOD_PREFIX + "chtype";
+    private static final String ALLOCATE    = METHOD_PREFIX + "allocate";
+    private static final String INFO        = METHOD_PREFIX + "info";
+    private static final String DELETE      = METHOD_PREFIX + "delete";
+    private static final String UPDATE      = METHOD_PREFIX + "update";
+    private static final String ENABLE      = METHOD_PREFIX + "enable";
+    private static final String PERSISTENT  = METHOD_PREFIX + "persistent";
+    private static final String CHOWN       = METHOD_PREFIX + "chown";
+    private static final String CHMOD       = METHOD_PREFIX + "chmod";
+    private static final String CHTYPE      = METHOD_PREFIX + "chtype";
+    private static final String CLONE       = METHOD_PREFIX + "clone";
+    private static final String RENAME      = METHOD_PREFIX + "rename";
 
     private static final String[] IMAGE_STATES =
-        {"INIT", "READY", "USED", "DISABLED"};
+        {"INIT", "READY", "USED", "DISABLED", "LOCKED",
+        "ERROR", "CLONE", "DELETE", "USED_PERS"};
 
     private static final String[] SHORT_IMAGE_STATES =
-        {"init", "rdy", "used", "disa"};
+        {"init", "rdy", "used", "disa", "lock", "err", "clon", "dele", "used"};
 
     private static final String[] IMAGE_TYPES =
-        {"OS", "CDROM", "DATABLOCK"};
+        {"OS", "CDROM", "DATABLOCK", "KERNEL", "RAMDISK", "CONTEXT"};
 
     private static final String[] SHORT_IMAGE_TYPES =
-        {"OS", "CD", "DB"};
+        {"OS", "CD", "DB", "KL", "RD", "CX"};
 
     /**
      * Creates a new Image representation.
@@ -76,7 +80,7 @@ public class Image extends PoolElement
      *
      * @param client XML-RPC Client.
      * @param description A string containing the template of the image.
-     * @param clusterId The cluster ID. If it is -1, this image
+     * @param datastoreId The cluster ID. If it is -1, this image
      * won't be added to any cluster.
      *
      * @return If successful the message contains the associated
@@ -85,22 +89,9 @@ public class Image extends PoolElement
     public static OneResponse allocate(
             Client client,
             String description,
-            int    clusterId)
+            int    datastoreId)
     {
-        return client.call(ALLOCATE, description, clusterId);
-    }
-
-    /**
-     * Allocates a new Image in OpenNebula.
-     *
-     * @param client XML-RPC Client.
-     * @param description A string containing the template of the image.
-     * @return If successful the message contains the associated
-     * id generated for this Image.
-     */
-    public static OneResponse allocate(Client client, String description)
-    {
-        return allocate(client, description, -1);
+        return client.call(ALLOCATE, description, datastoreId);
     }
 
     /**
@@ -134,11 +125,13 @@ public class Image extends PoolElement
      * @param client XML-RPC Client.
      * @param id The image id of the target image we want to modify.
      * @param new_template New template contents
+     * @param append True to append new attributes instead of replace the whole template
      * @return If successful the message contains the image id.
      */
-    public static OneResponse update(Client client, int id, String new_template)
+    public static OneResponse update(Client client, int id, String new_template,
+        boolean append)
     {
-        return client.call(UPDATE, id, new_template);
+        return client.call(UPDATE, id, new_template, append ? 1 : 0);
     }
 
     /**
@@ -154,6 +147,19 @@ public class Image extends PoolElement
         return client.call(ENABLE, id, enable);
     }
 
+    /**
+     * Sets the Image as persistent or not persistent.
+     *
+     * @param client XML-RPC Client.
+     * @param id The image id of the target image we want to modify.
+     * @param persistent True to make it persistent, false non-persistent
+     * @return If successful the message contains the image id.
+     */
+    public static OneResponse persistent(Client client, int id, boolean persistent)
+    {
+        return client.call(PERSISTENT, id, persistent);
+    }
+
     /**
      * Publishes or unpublishes an image.
      *
@@ -249,6 +255,32 @@ public class Image extends PoolElement
         return client.call(CHTYPE, id, type);
     }
 
+    /**
+     * Clones this Image into a new one
+     *
+     * @param client XML-RPC Client.
+     * @param id The Image id of the target Image.
+     * @param name Name for the new Image.
+     * @return If successful the message contains the new Image ID.
+     */
+    public static OneResponse clone(Client client, int id, String name)
+    {
+        return client.call(CLONE, id, name);
+    }
+
+    /**
+     * Renames this Image
+     *
+     * @param client XML-RPC Client.
+     * @param id The Image id of the target Image.
+     * @param name New name for the Image.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse rename(Client client, int id, String name)
+    {
+        return client.call(RENAME, id, name);
+    }
+
     // =================================
     // Instanced object XML-RPC methods
     // =================================
@@ -284,7 +316,19 @@ public class Image extends PoolElement
      */
     public OneResponse update(String new_template)
     {
-        return update(client, id, new_template);
+        return update(new_template, false);
+    }
+
+    /**
+     * Replaces the template contents.
+     *
+     * @param new_template New template contents
+     * @param append True to append new attributes instead of replace the whole template
+     * @return If successful the message contains the image id.
+     */
+    public OneResponse update(String new_template, boolean append)
+    {
+        return update(client, id, new_template, append);
     }
 
     /**
@@ -318,6 +362,37 @@ public class Image extends PoolElement
         return enable(false);
     }
 
+    /**
+     * Sets the Image as persistent or not persistent.
+     *
+     * @param persistent True for enabling, false for disabling.
+     * @return If successful the message contains the image id.
+     */
+    public OneResponse persistent(boolean persistent)
+    {
+        return persistent(client, id, persistent);
+    }
+
+    /**
+     * Sets the Image as persistent
+     *
+     * @return If successful the message contains the image id.
+     */
+    public OneResponse persistent()
+    {
+        return persistent(true);
+    }
+
+    /**
+     * Sets the Image as persistent or not persistent.
+     *
+     * @return If successful the message contains the image id.
+     */
+    public OneResponse nonpersistent()
+    {
+        return persistent(false);
+    }
+
     /**
      * Publishes or unpublishes the image.
      *
@@ -440,6 +515,28 @@ public class Image extends PoolElement
         return chtype(client, id, type);
     }
 
+    /**
+     * Clones this Image into a new one
+     *
+     * @param name Name for the new Image.
+     * @return If successful the message contains the new Image ID.
+     */
+    public OneResponse clone(String name)
+    {
+        return clone(client, id, name);
+    }
+
+    /**
+     * Renames this Image
+     *
+     * @param name New name for the Image.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse rename(String name)
+    {
+        return rename(client, id, name);
+    }
+
     // =================================
     // Helpers
     // =================================
