@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)
+ * Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@ public class VirtualNetwork extends PoolElement{
     private static final String UPDATE          = METHOD_PREFIX + "update";
     private static final String HOLD            = METHOD_PREFIX + "hold";
     private static final String RELEASE         = METHOD_PREFIX + "release";
+    private static final String RENAME          = METHOD_PREFIX + "rename";
 
     /**
      * Creates a new virtual network representation.
@@ -263,13 +264,28 @@ public class VirtualNetwork extends PoolElement{
      * Replaces the VirtualNetwork template contents.
      *
      * @param client XML-RPC Client.
-     * @param id The user id of the target vnet we want to modify.
+     * @param id The vnet id of the target vnet we want to modify.
      * @param new_template New template contents.
+     * @param append True to append new attributes instead of replace the whole template
      * @return If successful the message contains the vnet id.
      */
-    public static OneResponse update(Client client, int id, String new_template)
+    public static OneResponse update(Client client, int id, String new_template,
+        boolean append)
     {
-        return client.call(UPDATE, id, new_template);
+        return client.call(UPDATE, id, new_template, append ? 1 : 0);
+    }
+
+    /**
+     * Renames this VirtualNetwork
+     *
+     * @param client XML-RPC Client.
+     * @param id The VirtualNetwork id of the target VirtualNetwork.
+     * @param name New name for the VirtualNetwork.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public static OneResponse rename(Client client, int id, String name)
+    {
+        return client.call(RENAME, id, name);
     }
 
     // =================================
@@ -488,7 +504,30 @@ public class VirtualNetwork extends PoolElement{
      */
     public OneResponse update(String new_template)
     {
-        return update(client, id, new_template);
+        return update(new_template, false);
+    }
+
+    /**
+     * Replaces the VirtualNetwork template contents.
+     *
+     * @param new_template New template contents.
+     * @param append True to append new attributes instead of replace the whole template
+     * @return If successful the message contains the vnet id.
+     */
+    public OneResponse update(String new_template, boolean append)
+    {
+        return update(client, id, new_template, append);
+    }
+
+    /**
+     * Renames this VirtualNetwork
+     *
+     * @param name New name for the VirtualNetwork.
+     * @return If an error occurs the error message contains the reason.
+     */
+    public OneResponse rename(String name)
+    {
+        return rename(client, id, name);
     }
 
     // =================================
