@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -29,12 +29,6 @@ module OpenNebulaJSON
 
             if vm_hash['vm_raw']
                 template = vm_hash['vm_raw']
-            elsif vm_hash['template_id']
-                template_id = vm_hash['template_id']
-
-                template = "TEMPLATE_ID = #{template_id}"
-                template << "\nNAME = #{vm_hash['vm_name']}" if vm_hash['vm_name']
-
             else
                 template = template_to_str(vm_hash)
             end
@@ -53,19 +47,35 @@ module OpenNebulaJSON
                  when "deploy"       then self.deploy(action_hash['params'])
                  when "finalize"     then self.finalize
                  when "hold"         then self.hold
-                 when "livemigrate"  then self.live_migrate(action_hash['params'])
-                 when "migrate"      then self.migrate(action_hash['params'])
+                 when "livemigrate"  then self.migrate(action_hash['params'], true)
+                 when "migrate"      then self.migrate(action_hash['params'], false)
                  when "resume"       then self.resume
                  when "release"      then self.release
                  when "stop"         then self.stop
                  when "suspend"      then self.suspend
                  when "restart"      then self.restart
+                 when "reset"        then self.reset
                  when "saveas"       then self.save_as(action_hash['params'])
+                 when "snapshot_create"       then self.snapshot_create(action_hash['params'])
+                 when "snapshot_revert"       then self.snapshot_revert(action_hash['params'])
+                 when "snapshot_delete"       then self.snapshot_delete(action_hash['params'])
                  when "shutdown"     then self.shutdown
                  when "reboot"       then self.reboot
+                 when "poweroff"     then self.poweroff(action_hash['params'])
                  when "resubmit"     then self.resubmit
                  when "chown"        then self.chown(action_hash['params'])
                  when "chmod"        then self.chmod_octet(action_hash['params'])
+                 when "resize"       then self.resize(action_hash['params'])
+                 when "attachdisk"   then self.disk_attach(action_hash['params'])
+                 when "detachdisk"   then self.disk_detach(action_hash['params'])
+                 when "attachnic"    then self.nic_attach(action_hash['params'])
+                 when "detachnic"    then self.nic_detach(action_hash['params'])
+                 when "update"       then self.update(action_hash['params'])
+                 when "rename"       then self.rename(action_hash['params'])
+                 when "undeploy"     then self.undeploy(action_hash['params'])
+                 when "resched"      then self.resched
+                 when "unresched"    then self.unresched
+                 when "recover"      then self.recover(action_hash['params'])
                  else
                      error_msg = "#{action_hash['perform']} action not " <<
                          " available for this resource"
@@ -74,23 +84,39 @@ module OpenNebulaJSON
         end
 
         def delete
-            self.finalize
+            super()
         end
 
         def deploy(params=Hash.new)
-            super(params['host_id'])
+            super(params['host_id'], params['enforce'], params['ds_id'])
         end
 
-        def live_migrate(params=Hash.new)
-            super(params['host_id'])
+        def undeploy(params=Hash.new)
+            super(params['hard'])
         end
 
-        def migrate(params=Hash.new)
-            super(params['host_id'])
+        def poweroff(params=Hash.new)
+            super(params['hard'])
+        end
+
+        def migrate(params=Hash.new, live=false)
+            super(params['host_id'], live, params['enforce'])
         end
 
         def save_as(params=Hash.new)
-            super(params['disk_id'].to_i, params['image_name'], params['type'])
+            super(params['disk_id'].to_i, params['image_name'], params['type'], params['hot'])
+        end
+
+        def snapshot_create(params=Hash.new)
+            super(params['snapshot_name'])
+        end
+
+        def snapshot_revert(params=Hash.new)
+            super(params['snapshot_id'].to_i)
+        end
+
+        def snapshot_delete(params=Hash.new)
+            super(params['snapshot_id'].to_i)
         end
 
         def chown(params=Hash.new)
@@ -100,5 +126,44 @@ module OpenNebulaJSON
         def chmod_octet(params=Hash.new)
             super(params['octet'])
         end
+
+        def resize(params=Hash.new)
+            template_json = params['vm_template']
+            template = template_to_str(template_json)
+            super(template, params['enforce'])
+        end
+
+        def disk_attach(params=Hash.new)
+            template_json = params['disk_template']
+            template = template_to_str(template_json)
+            super(template)
+        end
+
+        def disk_detach(params=Hash.new)
+            super(params['disk_id'].to_i)
+        end
+
+        def nic_attach(params=Hash.new)
+            template_json = params['nic_template']
+            template = template_to_str(template_json)
+            super(template)
+        end
+
+        def nic_detach(params=Hash.new)
+            super(params['nic_id'].to_i)
+        end
+
+        def update(params=Hash.new)
+            super(params['template_raw'])
+        end
+
+        def rename(params=Hash.new)
+            super(params['name'])
+        end
+
+        def recover(params=Hash.new)
+            result = params['with'] == "success" ? true : false
+            super(result)
+        end
     end
 end
