@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -19,11 +19,12 @@ require 'CloudServer'
 require 'OpenNebulaJSON'
 include OpenNebulaJSON
 
-require 'acct/watch_client'
 require 'OpenNebulaVNC'
 require 'OpenNebulaJSON/JSONUtils'
 include JSONUtils
 
+require 'SunstoneMarketplace'
+
 class SunstoneServer < CloudServer
     # FLAG that will filter the elements retrieved from the Pools
     POOL_FILTER = Pool::INFO_ALL
@@ -31,6 +32,8 @@ class SunstoneServer < CloudServer
     # Secs to sleep between checks to see if image upload to repo is finished
     IMAGE_POLL_SLEEP_TIME = 5
 
+    include SunstoneMarketplace
+
     def initialize(client, config, logger)
         super(config, logger)
         @client = client
@@ -39,7 +42,9 @@ class SunstoneServer < CloudServer
     ############################################################################
     #
     ############################################################################
-    def get_pool(kind,gid)
+    def get_pool(kind,gid, client=nil)
+        client = @client if !client
+
         if gid == "0"
             user_flag = Pool::INFO_ALL
         else
@@ -47,27 +52,28 @@ class SunstoneServer < CloudServer
         end
 
         pool = case kind
-            when "group"      then GroupPoolJSON.new(@client)
-            when "cluster"    then ClusterPoolJSON.new(@client)
-            when "host"       then HostPoolJSON.new(@client)
-            when "image"      then ImagePoolJSON.new(@client, user_flag)
-            when "vmtemplate" then TemplatePoolJSON.new(@client, user_flag)
-            when "vm"         then VirtualMachinePoolJSON.new(@client, user_flag)
-            when "vnet"       then VirtualNetworkPoolJSON.new(@client, user_flag)
-            when "user"       then UserPoolJSON.new(@client)
-            when "acl"        then AclPoolJSON.new(@client)
-            when "datastore"  then DatastorePoolJSON.new(@client)
+            when "group"      then GroupPoolJSON.new(client)
+            when "cluster"    then ClusterPoolJSON.new(client)
+            when "host"       then HostPoolJSON.new(client)
+            when "image"      then ImagePoolJSON.new(client, user_flag)
+            when "vmtemplate" then TemplatePoolJSON.new(client, user_flag)
+            when "vm"         then VirtualMachinePoolJSON.new(client, user_flag)
+            when "vnet"       then VirtualNetworkPoolJSON.new(client, user_flag)
+            when "user"       then UserPoolJSON.new(client)
+            when "acl"        then AclPoolJSON.new(client)
+            when "datastore"  then DatastorePoolJSON.new(client)
+            when "zone"       then ZonePoolJSON.new(client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return [404, error.to_json]
         end
 
-        rc = pool.info
+        rc = pool.get_hash
 
         if OpenNebula.is_error?(rc)
             return [500, rc.to_json]
         else
-            return [200, pool.to_json]
+            return [200, rc.to_json]
         end
     end
 
@@ -111,6 +117,7 @@ class SunstoneServer < CloudServer
             when "user"       then UserJSON.new(User.build_xml, @client)
             when "acl"        then AclJSON.new(Acl.build_xml, @client)
             when "datastore"  then DatastoreJSON.new(Acl.build_xml, @client)
+            when "zone"       then ZoneJSON.new(Zone.build_xml, @client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return [404, error.to_json]
@@ -140,7 +147,7 @@ class SunstoneServer < CloudServer
         if OpenNebula.is_error?(ds_id)
             return [500, ds_id.to_json]
         end
-        
+
         new_template = {
             :image => image_hash,
             :ds_id => ds_id,
@@ -208,7 +215,7 @@ class SunstoneServer < CloudServer
             if !ONE_LOCATION
                 vm_log_file = LOG_LOCATION + "/#{id}.log"
             else
-                vm_log_file = LOG_LOCATION + "/#{id}/vm.log"
+                vm_log_file = LOG_LOCATION + "/vms/#{id}/vm.log"
             end
 
             begin
@@ -225,65 +232,142 @@ class SunstoneServer < CloudServer
     ########################################################################
     # VNC
     ########################################################################
-    def startvnc(id, config)
+    def startvnc(id, vnc)
         resource = retrieve_resource("vm", id)
         if OpenNebula.is_error?(resource)
             return [404, resource.to_json]
         end
 
-        vnc_proxy = OpenNebulaVNC.new(config, logger)
-        return vnc_proxy.start(resource)
+        return vnc.proxy(resource)
     end
 
-    ############################################################################
-    #
-    ############################################################################
-    def stopvnc(pipe)
-        begin
-            OpenNebulaVNC.stop(pipe)
-        rescue Exception => e
-            logger.error {e.message}
-            error = Error.new("Error stopping VNC. Please check server logs.")
-            return [500, error.to_json]
-        end
-
-        return [200, nil]
-    end
-
-    ############################################################################
-    #
-    ############################################################################
-    def get_monitoring(id, resource, monitor_resources, opts={})
-        watch_client = case resource
-            when "vm","VM"
-                OneWatchClient::VmWatchClient.new
-            when "host","HOST"
-                OneWatchClient::HostWatchClient.new
+    ########################################################################
+    # Accounting & Monitoring
+    ########################################################################
+    def get_pool_monitoring(resource, meters)
+        #pool_element
+        pool = case resource
+            when "vm", "VM"
+                VirtualMachinePool.new(@client)
+            when "host", "HOST"
+                HostPool.new(@client)
             else
-                error = Error.new("Monitoring not supported for this resource: #{resource}")
+                error = Error.new("Monitoring not supported for #{resource}")
                 return [200, error.to_json]
             end
 
-        filter = {}
-        filter[:uid] = opts[:uid] if opts[:gid]!=0
+        meters_a = meters.split(',')
 
-        columns = monitor_resources.split(',')
-        columns.map!{|e| e.to_sym}
+        rc = pool.monitoring(meters_a)
 
-        if id
-            rc = watch_client.resource_monitoring(id.to_i, columns, filter)
-        else
-            rc = watch_client.total_monitoring(columns, filter)
-        end
-
-        if rc.nil?
-            error = Error.new("There is no monitoring information for #{resource} #{id}")
+        if OpenNebula.is_error?(rc)
+            error = Error.new(rc.message)
             return [500, error.to_json]
         end
 
+        rc[:resource] = resource
+
         return [200, rc.to_json]
     end
 
+    def get_resource_monitoring(id, resource, meters)
+        pool_element = case resource
+            when "vm", "VM"
+                VirtualMachine.new_with_id(id, @client)
+            when "host", "HOST"
+                Host.new_with_id(id, @client)
+            else
+                error = Error.new("Monitoring not supported for #{resource}")
+                return [403, error.to_json]
+            end
+
+        meters_a = meters.split(',')
+
+        rc = pool_element.monitoring(meters_a)
+
+        if OpenNebula.is_error?(rc)
+            error = Error.new(rc.message)
+            return [500, error.to_json]
+        end
+
+        meters_h = Hash.new
+        meters_h[:resource]   = resource
+        meters_h[:id]         = id
+        meters_h[:monitoring] = rc
+
+        return [200, meters_h.to_json]
+    end
+
+
+    # returns a { monitoring : meter1 : [[ts1, agg_value],[ts2, agg_value]...]
+    #                          meter2 : [[ts1, agg_value],[ts2, agg_value]...]}
+    # with this information we can paint historical graphs of usage
+    def get_user_accounting(options)
+        uid      = options[:id].to_i
+        tstart   = options[:start].to_i
+        tend     = options[:end].to_i
+        interval = options[:interval].to_i
+        meters   = options[:monitor_resources]
+        gid      = options[:gid].to_i
+
+        acct_options = {:start_time => tstart,
+                        :end_time => tend}
+
+        # If we want acct per group, we ask for all VMs visible to user
+        # and then filter by group.
+        if gid
+            uid = Pool::INFO_ALL
+            acct_options[:group] = gid
+        end
+
+        # Init results and request accounting
+        result   = {}
+        meters_a = meters.split(',')
+        meters_a.each do | meter |
+            result[meter] = []
+        end
+        pool     = VirtualMachinePool.new(@client)
+        acct_xml = pool.accounting_xml(uid, acct_options)
+
+        if OpenNebula.is_error?(acct_xml)
+            error = Error.new(acct_xml.message)
+            return [500, error.to_json]
+        end
+
+        xml = XMLElement.new
+        xml.initialize_xml(acct_xml, 'HISTORY_RECORDS')
+
+        # We aggregate the accounting values for each interval withing
+        # the given timeframe
+        while tstart < tend
+
+            tstep = tstart + interval
+            count = Hash.new
+
+            # We count machines which have started before the end of
+            # this interval AND have not finished yet OR machines which
+            # have started before the end of this interval AND
+            # have finished anytime after the start of this interval
+            xml.each("HISTORY[STIME<=#{tstep} and ETIME=0 or STIME<=#{tstep} and ETIME>=#{tstart}]") do |hr|
+
+                meters_a.each do | meter |
+                    count[meter] ||= 0
+                    count[meter] += hr["VM/#{meter}"].to_i if hr["VM/#{meter}"]
+                end
+            end
+
+            # We have aggregated values for this interval
+            # Then we just add them to the results along with a timestamp
+            count.each do | mname, mcount |
+                result[mname] << [tstart, mcount]
+            end
+
+            tstart = tstep
+        end
+
+        return [200, {:monitoring => result}.to_json]
+    end
+
     private
 
     ############################################################################
@@ -301,6 +385,7 @@ class SunstoneServer < CloudServer
             when "user"       then UserJSON.new_with_id(id, @client)
             when "acl"        then AclJSON.new_with_id(id, @client)
             when "datastore"  then DatastoreJSON.new_with_id(id, @client)
+            when "zone"       then ZoneJSON.new_with_id(id, @client)
             else
                 error = Error.new("Error: #{kind} resource not supported")
                 return error
