@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             #
+# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -23,11 +23,13 @@ if !ONE_LOCATION
     LOG_LOCATION = "/var/log/one"
     VAR_LOCATION = "/var/lib/one"
     ETC_LOCATION = "/etc/one"
+    SHARE_LOCATION = "/usr/share/one"
     RUBY_LIB_LOCATION = "/usr/lib/one/ruby"
 else
     VAR_LOCATION = ONE_LOCATION + "/var"
     LOG_LOCATION = ONE_LOCATION + "/var"
     ETC_LOCATION = ONE_LOCATION + "/etc"
+    SHARE_LOCATION = ONE_LOCATION + "/share"
     RUBY_LIB_LOCATION = ONE_LOCATION+"/lib/ruby"
 end
 
@@ -44,6 +46,10 @@ $: << RUBY_LIB_LOCATION+'/cloud'
 $: << SUNSTONE_ROOT_DIR
 $: << SUNSTONE_ROOT_DIR+'/models'
 
+SESSION_EXPIRE_TIME = 60*60
+
+DISPLAY_NAME_XPATH = 'TEMPLATE/SUNSTONE_DISPLAY_NAME'
+
 ##############################################################################
 # Required libraries
 ##############################################################################
@@ -54,7 +60,7 @@ require 'yaml'
 
 require 'CloudAuth'
 require 'SunstoneServer'
-require 'SunstonePlugins'
+require 'SunstoneViews'
 
 
 ##############################################################################
@@ -62,40 +68,72 @@ require 'SunstonePlugins'
 ##############################################################################
 
 begin
-    conf = YAML.load_file(CONFIGURATION_FILE)
+    $conf = YAML.load_file(CONFIGURATION_FILE)
 rescue Exception => e
     STDERR.puts "Error parsing config file #{CONFIGURATION_FILE}: #{e.message}"
     exit 1
 end
 
-conf[:debug_level] ||= 3
+$conf[:debug_level] ||= 3
 
-CloudServer.print_configuration(conf)
+CloudServer.print_configuration($conf)
 
 #Sinatra configuration
 
-set :config, conf
-set :bind, settings.config[:host]
-set :port, settings.config[:port]
+set :config, $conf
+set :bind, $conf[:host]
+set :port, $conf[:port]
 
-use Rack::Session::Pool, :key => 'sunstone'
+case $conf[:sessions]
+when 'memory', nil
+    use Rack::Session::Pool, :key => 'sunstone'
+when 'memcache'
+    memcache_server=$conf[:memcache_host]+':'<<
+        $conf[:memcache_port].to_s
+
+    STDERR.puts memcache_server
+
+    use Rack::Session::Memcache,
+        :memcache_server => memcache_server,
+        :namespace => $conf[:memcache_namespace]
+else
+    STDERR.puts "Wrong value for :sessions in configuration file"
+    exit(-1)
+end
+
+use Rack::Deflater
 
 # Enable logger
 
 include CloudLogger
-enable_logging SUNSTONE_LOG, settings.config[:debug_level].to_i
+logger=enable_logging(SUNSTONE_LOG, $conf[:debug_level].to_i)
 
 begin
     ENV["ONE_CIPHER_AUTH"] = SUNSTONE_AUTH
-    cloud_auth = CloudAuth.new(settings.config)
+    $cloud_auth = CloudAuth.new($conf, logger)
 rescue => e
-    settings.logger.error {
+    logger.error {
         "Error initializing authentication system" }
-    settings.logger.error { e.message }
+    logger.error { e.message }
     exit -1
 end
 
-set :cloud_auth, cloud_auth
+set :cloud_auth, $cloud_auth
+
+
+$views_config = SunstoneViews.new
+
+#start VNC proxy
+
+$vnc = OpenNebulaVNC.new($conf, logger)
+
+configure do
+    set :run, false
+    set :vnc, $vnc
+    set :erb, :trim => '-'
+end
+
+DEFAULT_TABLE_ORDER = "desc"
 
 ##############################################################################
 # Helpers
@@ -107,17 +145,17 @@ helpers do
 
     def build_session
         begin
-            result = settings.cloud_auth.auth(request.env, params)
+            result = $cloud_auth.auth(request.env, params)
         rescue Exception => e
-            error 500, ""
             logger.error { e.message }
+            return [500, ""]
         end
 
         if result.nil?
             logger.info { "Unauthorized login attempt" }
             return [401, ""]
         else
-            client  = settings.cloud_auth.client(result)
+            client  = $cloud_auth.client(result)
             user_id = OpenNebula::User::SELF
 
             user    = OpenNebula::User.new_with_id(user_id, client)
@@ -127,39 +165,62 @@ helpers do
                 return [500, ""]
             end
 
-            session[:user]       = user['NAME']
-            session[:user_id]    = user['ID']
-            session[:user_gid]   = user['GID']
-            session[:user_gname] = user['GNAME']
-            session[:ip]         = request.ip
-            session[:remember]   = params[:remember]
+            session[:user]         = user['NAME']
+            session[:user_id]      = user['ID']
+            session[:user_gid]     = user['GID']
+            session[:user_gname]   = user['GNAME']
+            session[:ip]           = request.ip
+            session[:remember]     = params[:remember]
+            session[:display_name] = user[DISPLAY_NAME_XPATH] || user['NAME']
 
             #User IU options initialization
             #Load options either from user settings or default config.
             # - LANG
             # - WSS CONECTION
+            # - TABLE ORDER
 
             if user['TEMPLATE/LANG']
                 session[:lang] = user['TEMPLATE/LANG']
             else
-                session[:lang] = settings.config[:lang]
+                session[:lang] = $conf[:lang]
             end
 
             if user['TEMPLATE/VNC_WSS']
-                session[:wss] = user['TEMPLATE/VNC_WSS']
+                session[:vnc_wss] = user['TEMPLATE/VNC_WSS']
             else
-                wss = settings.config[:vnc_proxy_support_wss]
+                wss = $conf[:vnc_proxy_support_wss]
                 #limit to yes,no options
-                session[:wss] = (wss == true || wss == "yes" || wss == "only" ?
+                session[:vnc_wss] = (wss == true || wss == "yes" || wss == "only" ?
                                  "yes" : "no")
             end
 
+            if user['TEMPLATE/TABLE_ORDER']
+                session[:table_order] = user['TEMPLATE/TABLE_ORDER']
+            else
+                session[:table_order] = $conf[:table_order] || DEFAULT_TABLE_ORDER
+            end
+
+            if user['TEMPLATE/DEFAULT_VIEW']
+                session[:default_view] = user['TEMPLATE/DEFAULT_VIEW']
+            else
+                session[:default_view] = $views_config.available_views(session[:user], session[:user_gname]).first
+            end
+
             #end user options
 
-            if params[:remember]
-                env['rack.session.options'][:expire_after] = 30*60*60*24
+            if params[:remember] == "true"
+                env['rack.session.options'][:expire_after] = 30*60*60*24-1
             end
 
+            serveradmin_client = $cloud_auth.client()
+            rc = OpenNebula::System.new(serveradmin_client).get_configuration
+            return [500, rc.message] if OpenNebula.is_error?(rc)
+            return [500, "Couldn't find out zone identifier"] if !rc['FEDERATION/ZONE_ID']
+
+            zone = OpenNebula::Zone.new_with_id(rc['FEDERATION/ZONE_ID'].to_i, client)
+            zone.info
+            session[:zone_name] = zone.name
+
             return [204, ""]
         end
     end
@@ -171,67 +232,82 @@ helpers do
 end
 
 before do
-    unless request.path=='/login' || request.path=='/'
+    cache_control :no_store
+    content_type 'application/json', :charset => 'utf-8'
+    unless request.path=='/login' || request.path=='/' || request.path=='/vnc'
         halt 401 unless authorized?
-
-        @SunstoneServer = SunstoneServer.new(
-                              settings.cloud_auth.client(session[:user]),
-                              settings.config,
-                              settings.logger)
     end
+
+    if env['HTTP_ZONE_NAME']
+        client=$cloud_auth.client(session[:user])
+        zpool = ZonePoolJSON.new(client)
+
+        rc = zpool.info
+
+        return [500, rc.to_json] if OpenNebula.is_error?(rc)
+
+        zpool.each{|z|
+            if z.name == env['HTTP_ZONE_NAME']
+              session[:active_zone_endpoint] = z['TEMPLATE/ENDPOINT']
+              session[:zone_name] = env['HTTP_ZONE_NAME']
+            end 
+         }
+    end
+
+    client=$cloud_auth.client(session[:user],
+                              session[:active_zone_endpoint])
+
+    @SunstoneServer = SunstoneServer.new(client,$conf,logger)
 end
 
 after do
-    unless request.path=='/login' || request.path=='/'
-        unless session[:remember]
-            if params[:timeout] == true
+    unless request.path=='/login' || request.path=='/' || request.path=='/'
+        unless session[:remember] == "true"
+            if params[:timeout] == "true"
                 env['rack.session.options'][:defer] = true
             else
-                env['rack.session.options'][:expire_after] = 60*10
+                env['rack.session.options'][:expire_after] = SESSION_EXPIRE_TIME
             end
         end
     end
 end
 
+##############################################################################
+# Custom routes
+##############################################################################
+if $conf[:routes]
+    $conf[:routes].each { |route|
+        require "routes/#{route}"
+    }
+end
+
 ##############################################################################
 # HTML Requests
 ##############################################################################
 get '/' do
+    content_type 'text/html', :charset => 'utf-8'
     if !authorized?
-        if settings.config[:auth] == "x509"
-            templ = "login_x509.html"
-        else
-            templ = "login.html"
-        end
-
-        return File.read(File.dirname(__FILE__)+'/templates/'+templ)
+        return erb :login
     end
-    time = Time.now + 60*10
-    response.set_cookie("one-user",
-                        :value=>"#{session[:user]}",
-                        :expires=>time)
-    response.set_cookie("one-user_id",
-                        :value=>"#{session[:user_id]}",
-                        :expires=>time)
-    response.set_cookie("one-user_gid",
-                        :value=>"#{session[:user_gid]}",
-                        :expires=>time)
 
-    p = SunstonePlugins.new
-    @plugins = p.authorized_plugins(session[:user], session[:user_gname])
+    response.set_cookie("one-user", :value=>"#{session[:user]}")
 
     erb :index
 end
 
 get '/login' do
+    content_type 'text/html', :charset => 'utf-8'
     if !authorized?
-        if settings.config[:auth] == "x509"
-            templ = "login_x509.html"
-        else
-            templ = "login.html"
-        end
+        erb :login
+    end
+end
 
-        return File.read(File.dirname(__FILE__)+'/templates/'+templ)
+get '/vnc' do
+    content_type 'text/html', :charset => 'utf-8'
+    if !authorized?
+        erb :login
+    else
+        erb :vnc
     end
 end
 
@@ -254,7 +330,11 @@ get '/config' do
     uconf = {
         :user_config => {
             :lang => session[:lang],
-            :wss  => session[:wss]
+            :vnc_wss  => session[:vnc_wss],
+        },
+        :system_config => {
+            :marketplace_url => $conf[:marketplace_url],
+            :vnc_proxy_port => $vnc.proxy_port
         }
     }
 
@@ -262,21 +342,27 @@ get '/config' do
 end
 
 post '/config' do
-    begin
-        body = JSON.parse(request.body.read)
-    rescue Exception => e
-        msg = "Error parsing configuration JSON"
-        logger.error { msg }
-        logger.error { e.message } 
-        [500, OpenNebula::Error.new(msg).to_json]
+    @SunstoneServer.perform_action('user',
+                               OpenNebula::User::SELF,
+                               request.body.read)
+
+    user = OpenNebula::User.new_with_id(
+                OpenNebula::User::SELF,
+                $cloud_auth.client(session[:user]))
+
+    rc = user.info
+    if OpenNebula.is_error?(rc)
+        logger.error { rc.message }
+        error 500, ""
     end
 
-    body.each do | key,value |
-        case key
-            when "lang" then session[:lang]= value
-            when "wss"  then session[:wss] = value
-        end
-    end
+    session[:lang]         = user['TEMPLATE/LANG']
+    session[:vnc_wss]      = user['TEMPLATE/VNC_WSS']
+    session[:default_view] = user['TEMPLATE/DEFAULT_VIEW']
+    session[:table_order]  = user['TEMPLATE/TABLE_ORDER']
+    session[:display_name] = user[DISPLAY_NAME_XPATH] || user['NAME']
+
+    [200, ""]
 end
 
 get '/vm/:id/log' do
@@ -284,34 +370,60 @@ get '/vm/:id/log' do
 end
 
 ##############################################################################
-# Monitoring
+# Accounting & Monitoring
 ##############################################################################
 
 get '/:resource/monitor' do
-    @SunstoneServer.get_monitoring(
-        nil,
+    @SunstoneServer.get_pool_monitoring(
         params[:resource],
-        params[:monitor_resources],
-        :uid => session[:user_id].to_i,
-        :gid => session[:user_gid].to_i)
+        params[:monitor_resources])
+end
+
+get '/user/:id/monitor' do
+    @SunstoneServer.get_user_accounting(params)
+end
+
+get '/group/:id/monitor' do
+    params[:gid] = params[:id]
+    @SunstoneServer.get_user_accounting(params)
 end
 
 get '/:resource/:id/monitor' do
-    @SunstoneServer.get_monitoring(
+    @SunstoneServer.get_resource_monitoring(
         params[:id],
         params[:resource],
-        params[:monitor_resources],
-        :uid => session[:user_id].to_i,
-        :gid => session[:user_gid].to_i)
+        params[:monitor_resources])
 end
 
+##############################################################################
+# Marketplace
+##############################################################################
+get '/marketplace' do
+    @SunstoneServer.get_appliance_pool
+end
+
+get '/marketplace/:id' do
+    @SunstoneServer.get_appliance(params[:id])
+end
 
 ##############################################################################
 # GET Pool information
 ##############################################################################
 get '/:pool' do
-    @SunstoneServer.get_pool(params[:pool],
-                             session[:user_gid])
+    zone_client = nil
+
+    if params[:zone_id]
+        zone = OpenNebula::Zone.new_with_id(params[:zone_id].to_i,
+                                            $cloud_auth.client(session[:user]))
+        rc   = zone.info
+        return [500, rc.message] if OpenNebula.is_error?(rc)
+        zone_client = $cloud_auth.client(session[:user],
+                                         zone['TEMPLATE/ENDPOINT'])
+    end
+
+    @SunstoneServer.get_pool(params[:pool], 
+                             session[:user_gid], 
+                             zone_client)
 end
 
 ##############################################################################
@@ -337,7 +449,27 @@ end
 # Upload image
 ##############################################################################
 post '/upload'do
-    @SunstoneServer.upload(params[:img], request.env['rack.input'].path)
+
+    tmpfile = nil
+    rackinput = request.env['rack.input']
+
+    if (rackinput.class == Tempfile)
+        tmpfile = rackinput
+    elsif rackinput.respond_to?('read')
+        tmpfile = Tempfile.open('sunstone-upload')
+        tmpfile.write rackinput.read
+        tmpfile.flush
+    else
+        logger.error { "Unexpected rackinput class #{rackinput.class}" }
+        [500, ""]
+    end
+
+    if tmpfile.size == 0
+        [500, OpenNebula::Error.new("There was a problem uploading the file, " \
+                "please check the permissions on the file").to_json]
+    else
+        @SunstoneServer.upload(params[:img], tmpfile.path)
+    end
 end
 
 ##############################################################################
@@ -348,55 +480,11 @@ post '/:pool' do
 end
 
 ##############################################################################
-# Stop the VNC Session of a target VM
-##############################################################################
-post '/vm/:id/stopvnc' do
-    vm_id = params[:id]
-    vnc_hash = session['vnc']
-
-    if !vnc_hash || !vnc_hash[vm_id]
-        msg = "It seems there is no VNC proxy running for this machine"
-        return [403, OpenNebula::Error.new(msg).to_json]
-    end
-
-    rc = @SunstoneServer.stopvnc(vnc_hash[vm_id][:pipe])
-
-    if rc[0] == 200
-        session['vnc'].delete(vm_id)
-    end
-
-    rc
-end
-
-##############################################################################
-# Start a VNC Session for a target VM
+# Start VNC Session for a target VM
 ##############################################################################
 post '/vm/:id/startvnc' do
     vm_id = params[:id]
-
-    vnc_hash = session['vnc']
-
-    if !vnc_hash
-        session['vnc']= {}
-    elsif vnc_hash[vm_id]
-        #return existing information
-        info = vnc_hash[vm_id].clone
-        info.delete(:pipe)
-
-        return [200, info.to_json]
-    end
-
-    rc = @SunstoneServer.startvnc(vm_id,settings.config)
-
-    if rc[0] == 200
-        info = rc[1]
-        session['vnc'][vm_id] = info.clone
-        info.delete(:pipe)
-
-        [200, info.to_json]
-    else
-        rc
-    end
+    @SunstoneServer.startvnc(vm_id, $vnc)
 end
 
 ##############################################################################
@@ -407,3 +495,6 @@ post '/:resource/:id/action' do
                                    params[:id],
                                    request.body.read)
 end
+
+Sinatra::Application.run! if(!defined?(WITH_RACKUP))
+
