@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,7 +22,7 @@
 #include "NebulaLog.h"
 #include "Nebula.h"
 #include "AuthManager.h"
-#include "SSLTools.h"
+#include "NebulaUtil.h"
 
 #include <fstream>
 #include <sys/types.h>
@@ -53,8 +53,11 @@ string UserPool::oneadmin_name;
 /* -------------------------------------------------------------------------- */
 
 UserPool::UserPool(SqlDB * db,
-                   time_t  __session_expiration_time):
-                       PoolSQL(db, User::table, true)
+                   time_t  __session_expiration_time,
+                   vector<const Attribute *> hook_mads,
+                   const string&             remotes_location,
+                   bool                      is_federation_slave):
+                       PoolSQL(db, User::table, !is_federation_slave, true)
 {
     int           one_uid    = -1;
     int           server_uid = -1;
@@ -70,14 +73,19 @@ UserPool::UserPool(SqlDB * db,
     const char *  one_auth;
     ifstream      file;
 
-    string        filenames[3];
+    string        filenames[5];
     string        error_str;
-    stringstream  sstr;
 
     Nebula& nd   = Nebula::instance();
 
     _session_expiration_time = __session_expiration_time;
 
+    //Federation slaves do not need to init the pool
+    if (is_federation_slave)
+    {
+        return;
+    }
+
     User * oneadmin_user = get(0, true);
 
     if (oneadmin_user != 0)
@@ -85,6 +93,8 @@ UserPool::UserPool(SqlDB * db,
         oneadmin_name = oneadmin_user->get_name();
         oneadmin_user->unlock();
 
+        register_hooks(hook_mads, remotes_location);
+
         return;
     }
 
@@ -115,7 +125,7 @@ UserPool::UserPool(SqlDB * db,
     if (!file.good())
     {
         goto error_file;
-    } 
+    }
 
     getline(file,one_token);
 
@@ -138,18 +148,17 @@ UserPool::UserPool(SqlDB * db,
         goto error_one_name;
     }
 
-    srand(time(0));
-    sstr << rand();
-
-    random = SSLTools::sha1_digest( sstr.str() );
+    random = one_util::random_password();
 
     filenames[0] = nd.get_var_location() + "/.one/sunstone_auth";
     filenames[1] = nd.get_var_location() + "/.one/occi_auth";
     filenames[2] = nd.get_var_location() + "/.one/ec2_auth";
+    filenames[3] = nd.get_var_location() + "/.one/onegate_auth";
+    filenames[4] = nd.get_var_location() + "/.one/oneflow_auth";
 
     mkdir(string(nd.get_var_location() + "/.one").c_str(), S_IRWXU);
 
-    for (i=0 ; i < 3; i++)
+    for (i=0 ; i < 5; i++)
     {
         int cfile = creat(filenames[i].c_str(), S_IRUSR | S_IWUSR);
         close(cfile);
@@ -184,7 +193,7 @@ UserPool::UserPool(SqlDB * db,
              GroupPool::ONEADMIN_ID,
              SERVER_NAME,
              GroupPool::ONEADMIN_NAME,
-             SSLTools::sha1_digest(random),
+             one_util::sha1_digest(random),
              "server_cipher",
              true,
              error_str);
@@ -194,6 +203,8 @@ UserPool::UserPool(SqlDB * db,
         goto error_serveradmin;
     }
 
+    register_hooks(hook_mads, remotes_location);
+
     return;
 
 error_no_file:
@@ -256,13 +267,22 @@ int UserPool::allocate (
 
     ostringstream   oss;
 
+    if (nd.is_federation_slave())
+    {
+        NebulaLog::log("ONE",Log::ERROR,
+                "UserPool::allocate called, but this "
+                "OpenNebula is a federation slave");
+
+        return -1;
+    }
+
     // Check username and password
     if ( !User::pass_is_valid(password, error_str) )
     {
         goto error_pass;
     }
 
-    if ( !User::name_is_valid(uname, error_str) )
+    if (!PoolObjectSQL::name_is_valid(uname,User::INVALID_NAME_CHARS,error_str))
     {
         goto error_name;
     }
@@ -283,17 +303,23 @@ int UserPool::allocate (
 
     if (auth_driver == UserPool::CORE_AUTH)
     {
-        upass = SSLTools::sha1_digest(password);
+        upass = one_util::sha1_digest(password);
     }
 
     // Build a new User object
     user = new User(-1, gid, uname, gname, upass, auth_driver, enabled);
 
+    // Add the primary group to the collection
+    user->add_collection_id(gid);
+
+    // Set a password for the OneGate tokens
+    user->add_template_attribute("TOKEN_PASSWORD", one_util::random_password());
+
     // Insert the Object in the pool
     *oid = PoolSQL::allocate(user, error_str);
 
     if ( *oid < 0 )
-    { 
+    {
         return *oid;
     }
 
@@ -336,12 +362,55 @@ error_common:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+int UserPool::drop(PoolObjectSQL * objsql, string& error_msg)
+{
+    if (Nebula::instance().is_federation_slave())
+    {
+        NebulaLog::log("ONE",Log::ERROR,
+                "UserPool::drop called, but this "
+                "OpenNebula is a federation slave");
+
+        return -1;
+    }
+
+    return PoolSQL::drop(objsql, error_msg);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int UserPool::update(User * user)
+{
+    if (Nebula::instance().is_federation_slave())
+    {
+        NebulaLog::log("ONE",Log::ERROR,
+                "UserPool::update called, but this "
+                "OpenNebula is a federation slave");
+
+        return -1;
+    }
+
+    return user->update(db);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int UserPool::update_quotas(User * user)
+{
+    return user->update_quotas(db);
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 bool UserPool::authenticate_internal(User *        user,
                                      const string& token,
                                      int&          user_id,
                                      int&          group_id,
                                      string&       uname,
-                                     string&       gname)
+                                     string&       gname,
+                                     set<int>&     group_ids)
 {
     bool result = false;
 
@@ -360,6 +429,8 @@ bool UserPool::authenticate_internal(User *        user,
     user_id  = user->oid;
     group_id = user->gid;
 
+    group_ids = user->get_groups();
+
     uname  = user->name;
     gname  = user->gname;
 
@@ -374,7 +445,7 @@ bool UserPool::authenticate_internal(User *        user,
         return true;
     }
 
-    AuthRequest ar(user_id, group_id);
+    AuthRequest ar(user_id, group_ids);
 
     if ( auth_driver == UserPool::CORE_AUTH )
     {
@@ -438,6 +509,8 @@ auth_failure:
     user_id  = -1;
     group_id = -1;
 
+    group_ids.clear();
+
     uname = "";
     gname = "";
 
@@ -452,7 +525,8 @@ bool UserPool::authenticate_server(User *        user,
                                    int&          user_id,
                                    int&          group_id,
                                    string&       uname,
-                                   string&       gname)
+                                   string&       gname,
+                                   set<int>&     group_ids)
 {
     bool result = false;
 
@@ -473,7 +547,7 @@ bool UserPool::authenticate_server(User *        user,
 
     auth_driver = user->auth_driver;
 
-    AuthRequest ar(user->oid, user->gid);
+    AuthRequest ar(user->oid, user->get_groups());
 
     user->unlock();
 
@@ -495,6 +569,8 @@ bool UserPool::authenticate_server(User *        user,
     user_id  = user->oid;
     group_id = user->gid;
 
+    group_ids = user->get_groups();
+
     uname  = user->name;
     gname  = user->gname;
 
@@ -564,6 +640,8 @@ auth_failure:
     user_id  = -1;
     group_id = -1;
 
+    group_ids.clear();
+
     uname = "";
     gname = "";
 
@@ -572,13 +650,14 @@ auth_failure:
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
-    
-bool UserPool::authenticate_external(const string& username,
-                                     const string& token,
-                                     int&    user_id,
-                                     int&    group_id,
-                                     string& uname,
-                                     string& gname)
+
+bool UserPool::authenticate_external(const string&  username,
+                                     const string&  token,
+                                     int&           user_id,
+                                     int&           group_id,
+                                     string&        uname,
+                                     string&        gname,
+                                     set<int>&      group_ids)
 {
     ostringstream oss;
     istringstream is;
@@ -591,7 +670,9 @@ bool UserPool::authenticate_external(const string& username,
     Nebula&     nd      = Nebula::instance();
     AuthManager * authm = nd.get_authm();
 
-    AuthRequest ar(-1,-1);
+    set<int> empty_set;
+
+    AuthRequest ar(-1,empty_set);
 
     if (authm == 0)
     {
@@ -605,7 +686,7 @@ bool UserPool::authenticate_external(const string& username,
     ar.wait();
 
     if (ar.result != true) //User was not authenticated
-    {   
+    {
         goto auth_failure_driver;
     }
 
@@ -646,14 +727,15 @@ bool UserPool::authenticate_external(const string& username,
     }
 
     group_id = GroupPool::USERS_ID;
+    group_ids.insert( GroupPool::USERS_ID );
 
     uname = mad_name;
     gname = GroupPool::USERS_NAME;
 
     return true;
-        
+
 auth_failure_user:
-    oss << "Can't create user: " << error_str << ". Driver response: " 
+    oss << "Can't create user: " << error_str << ". Driver response: "
         << ar.message;
     NebulaLog::log("AuM",Log::ERROR,oss);
 
@@ -674,6 +756,8 @@ auth_failure:
     user_id  = -1;
     group_id = -1;
 
+    group_ids.clear();
+
     uname = "";
     gname = "";
 
@@ -682,12 +766,13 @@ auth_failure:
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
- 
-bool UserPool::authenticate(const string& session, 
-                            int&          user_id, 
+
+bool UserPool::authenticate(const string& session,
+                            int&          user_id,
                             int&          group_id,
                             string&       uname,
-                            string&       gname)
+                            string&       gname,
+                            set<int>&     group_ids)
 {
     User * user = 0;
     string username;
@@ -711,16 +796,16 @@ bool UserPool::authenticate(const string& session,
 
         if ( fnmatch(UserPool::SERVER_AUTH, driver.c_str(), 0) == 0 )
         {
-            ar = authenticate_server(user,token,user_id,group_id,uname,gname);
+            ar = authenticate_server(user,token,user_id,group_id,uname,gname,group_ids);
         }
         else
         {
-            ar = authenticate_internal(user,token,user_id,group_id,uname,gname);
+            ar = authenticate_internal(user,token,user_id,group_id,uname,gname,group_ids);
         }
     }
     else
     {
-        ar = authenticate_external(username,token,user_id,group_id,uname,gname);
+        ar = authenticate_external(username,token,user_id,group_id,uname,gname,group_ids);
     }
 
    return ar;
@@ -735,7 +820,7 @@ int UserPool::authorize(AuthRequest& ar)
     AuthManager * authm = nd.get_authm();
     int           rc    = -1;
 
-    if (authm == 0)
+    if (authm == 0 || !authm->is_authz_enabled())
     {
         if (ar.core_authorize())
         {
@@ -763,3 +848,72 @@ int UserPool::authorize(AuthRequest& ar)
     return rc;
 }
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int UserPool::dump(ostringstream& oss, const string& where, const string& limit)
+{
+    int     rc;
+    string  def_quota_xml;
+
+    ostringstream cmd;
+
+    cmd << "SELECT " << User::table << ".body, "
+        << UserQuotas::db_table << ".body"<< " FROM " << User::table
+        << " LEFT JOIN " << UserQuotas::db_table << " ON "
+        << User::table << ".oid=" << UserQuotas::db_table << ".user_oid";
+
+    if ( !where.empty() )
+    {
+        cmd << " WHERE " << where;
+    }
+
+    cmd << " ORDER BY oid";
+
+    if ( !limit.empty() )
+    {
+        cmd << " LIMIT " << limit;
+    }
+
+    oss << "<USER_POOL>";
+
+    set_callback(static_cast<Callbackable::Callback>(&UserPool::dump_cb),
+                 static_cast<void *>(&oss));
+
+    rc = db->exec(cmd, this);
+
+    unset_callback();
+
+    oss << Nebula::instance().get_default_user_quota().to_xml(def_quota_xml);
+
+    oss << "</USER_POOL>";
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int UserPool::dump_cb(void * _oss, int num, char **values, char **names)
+{
+    ostringstream * oss;
+
+    oss = static_cast<ostringstream *>(_oss);
+
+    if ( (!values[0]) || (num != 2) )
+    {
+        return -1;
+    }
+
+    *oss << values[0];
+
+    if (values[1] != NULL)
+    {
+        *oss << values[1];
+    }
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
