@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -24,12 +24,21 @@
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
-                                       vector<const Attribute *> hook_mads,
-                                       const string& hook_location,
-                                       const string& remotes_location,
-                                       vector<const Attribute *>& restricted_attrs)
-    : PoolSQL(db, VirtualMachine::table, false)
+time_t VirtualMachinePool::_monitor_expiration;
+bool   VirtualMachinePool::_submit_on_hold;
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+VirtualMachinePool::VirtualMachinePool(
+        SqlDB *                     db,
+        vector<const Attribute *>   hook_mads,
+        const string&               hook_location,
+        const string&               remotes_location,
+        vector<const Attribute *>&  restricted_attrs,
+        time_t                      expire_time,
+        bool                        on_hold)
+    : PoolSQL(db, VirtualMachine::table, true, false)
 {
     const VectorAttribute * vattr;
 
@@ -37,11 +46,18 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
     string on;
     string cmd;
     string arg;
-    string rmt;
     bool   remote;
 
     bool state_hook = false;
 
+    _monitor_expiration = expire_time;
+    _submit_on_hold = on_hold;
+
+    if ( _monitor_expiration == 0 )
+    {
+        clean_all_monitoring();
+    }
+
     for (unsigned int i = 0 ; i < hook_mads.size() ; i++ )
     {
         vattr = static_cast<const VectorAttribute *>(hook_mads[i]);
@@ -50,7 +66,7 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
         on   = vattr->vector_value("ON");
         cmd  = vattr->vector_value("COMMAND");
         arg  = vattr->vector_value("ARGUMENTS");
-        rmt  = vattr->vector_value("REMOTE");
+        vattr->vector_value("REMOTE", remote);
 
         transform (on.begin(),on.end(),on.begin(),(int(*)(int))toupper);
 
@@ -70,39 +86,27 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
             name = cmd;
         }
 
-        remote = false;
-
-        if ( !rmt.empty() )
-        {
-            transform(rmt.begin(),rmt.end(),rmt.begin(),(int(*)(int))toupper);
-
-            if ( rmt == "YES" )
-            {
-                remote = true;
-            }
-        }
-
         if (cmd[0] != '/')
         {
             ostringstream cmd_os;
 
             if ( remote )
             {
-                cmd_os << hook_location << "/" << cmd;     
+                cmd_os << hook_location << "/" << cmd;
             }
             else
             {
                 cmd_os << remotes_location << "/hooks/" << cmd;
-            } 
+            }
 
             cmd = cmd_os.str();
-        } 
+        }
 
         if ( on == "CREATE" )
         {
-            VirtualMachineAllocateHook * hook;
+            AllocateHook * hook;
 
-            hook = new VirtualMachineAllocateHook(name,cmd,arg);
+            hook = new AllocateHook(name, cmd, arg, false);
 
             add_hook(hook);
         }
@@ -166,6 +170,51 @@ VirtualMachinePool::VirtualMachinePool(SqlDB *                   db,
 
             state_hook = true;
         }
+        else if ( on == "UNKNOWN" )
+        {
+            VirtualMachineStateHook * hook;
+
+            hook = new VirtualMachineStateHook(name, cmd, arg, remote,
+                            VirtualMachine::UNKNOWN, VirtualMachine::ACTIVE);
+            add_hook(hook);
+
+            state_hook = true;
+        }
+        else if ( on == "CUSTOM" )
+        {
+            VirtualMachineStateHook * hook;
+
+            string lcm_str = vattr->vector_value("LCM_STATE");
+            string vm_str  = vattr->vector_value("STATE");
+
+            VirtualMachine::LcmState lcm_state;
+            VirtualMachine::VmState vm_state;
+
+            if ( VirtualMachine::lcm_state_from_str(lcm_str, lcm_state) != 0 )
+            {
+                ostringstream oss;
+                oss << "Wrong LCM_STATE: "<< lcm_str <<". Hook not registered!";
+
+                NebulaLog::log("VM",Log::WARNING,oss);
+                continue;
+            }
+
+            if ( VirtualMachine::vm_state_from_str(vm_str, vm_state) != 0 )
+            {
+                ostringstream oss;
+                oss << "Wrong STATE: "<< vm_str <<". Hook not registered!";
+
+                NebulaLog::log("VM",Log::WARNING,oss);
+                continue;
+            }
+
+            hook = new VirtualMachineStateHook(name, cmd, arg, remote,
+                    lcm_state, vm_state);
+
+            add_hook(hook);
+
+            state_hook = true;
+        }
         else
         {
             ostringstream oss;
@@ -196,6 +245,7 @@ int VirtualMachinePool::allocate (
     int            gid,
     const string&  uname,
     const string&  gname,
+    int            umask,
     VirtualMachineTemplate * vm_template,
     int *          oid,
     string&        error_str,
@@ -206,9 +256,9 @@ int VirtualMachinePool::allocate (
     // ------------------------------------------------------------------------
     // Build a new Virtual Machine object
     // ------------------------------------------------------------------------
-    vm = new VirtualMachine(-1, uid, gid, uname, gname, vm_template);
+    vm = new VirtualMachine(-1, uid, gid, uname, gname, umask, vm_template);
 
-    if (on_hold == true)
+    if ( _submit_on_hold == true || on_hold )
     {
         vm->state = VirtualMachine::HOLD;
     }
@@ -216,12 +266,12 @@ int VirtualMachinePool::allocate (
     {
         vm->state = VirtualMachine::PENDING;
     }
-    
+
     // ------------------------------------------------------------------------
     // Insert the Object in the pool
     // ------------------------------------------------------------------------
 
-    *oid = PoolSQL::allocate(vm,error_str);
+    *oid = PoolSQL::allocate(vm, error_str);
 
     return *oid;
 }
@@ -266,3 +316,100 @@ int VirtualMachinePool::get_pending(
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
+
+int VirtualMachinePool::dump_acct(ostringstream& oss,
+                                  const string&  where,
+                                  int            time_start,
+                                  int            time_end)
+{
+    ostringstream cmd;
+
+    cmd << "SELECT " << History::table << ".body FROM " << History::table
+        << " INNER JOIN " << VirtualMachine::table
+        << " WHERE vid=oid";
+
+    if ( !where.empty() )
+    {
+        cmd << " AND " << where;
+    }
+
+    if ( time_start != -1 || time_end != -1 )
+    {
+        if ( time_start != -1 )
+        {
+            cmd << " AND (etime > " << time_start << " OR  etime = 0)";
+        }
+
+        if ( time_end != -1 )
+        {
+            cmd << " AND stime < " << time_end;
+        }
+    }
+
+    cmd << " ORDER BY vid,seq";
+
+    return PoolSQL::dump(oss, "HISTORY_RECORDS", cmd);
+};
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualMachinePool::clean_expired_monitoring()
+{
+    if ( _monitor_expiration == 0 )
+    {
+        return 0;
+    }
+
+    time_t          max_last_poll;
+    int             rc;
+    ostringstream   oss;
+
+    max_last_poll = time(0) - _monitor_expiration;
+
+    oss << "DELETE FROM " << VirtualMachine::monit_table
+        << " WHERE last_poll < " << max_last_poll;
+
+    rc = db->exec(oss);
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualMachinePool::clean_all_monitoring()
+{
+    ostringstream   oss;
+    int             rc;
+
+    oss << "DELETE FROM " << VirtualMachine::monit_table;
+
+    rc = db->exec(oss);
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualMachinePool::dump_monitoring(
+        ostringstream& oss,
+        const string&  where)
+{
+    ostringstream cmd;
+
+    cmd << "SELECT " << VirtualMachine::monit_table << ".body FROM "
+        << VirtualMachine::monit_table
+        << " INNER JOIN " << VirtualMachine::table
+        << " WHERE vmid = oid";
+
+    if ( !where.empty() )
+    {
+        cmd << " AND " << where;
+    }
+
+    cmd << " ORDER BY vmid, " << VirtualMachine::monit_table << ".last_poll;";
+
+    return PoolSQL::dump(oss, "MONITORING_DATA", cmd);
+}
