@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -20,6 +20,59 @@
 #include <sstream>
 #include <fstream>
 #include <libgen.h>
+#include <math.h>
+
+const float LibVirtDriver::CGROUP_BASE_CPU_SHARES = 1024;
+
+/**
+ *  This function generates the <host> element for network disks
+ */
+static void do_network_hosts(ofstream& file,
+                             const string& cg_host,
+                             const string& transport)
+{
+    if (cg_host.empty())
+    {
+        file << "'/>" << endl;
+        return;
+    }
+
+    vector<string>::const_iterator it;
+    vector<string> hosts;
+
+    hosts = one_util::split(cg_host, ' ');
+
+    file << "'>" << endl;
+
+    for (it = hosts.begin(); it != hosts.end(); it++)
+    {
+        vector<string> parts = one_util::split(*it, ':');
+
+        if (parts.empty())
+        {
+            continue;
+        }
+
+        file << "\t\t\t\t<host name='" << parts[0];
+
+        if (parts.size() > 1)
+        {
+            file << "' port='" << parts[1];
+        }
+
+        if (!transport.empty())
+        {
+            file << "' transport='" << transport;
+        }
+
+        file << "'/>" << endl;
+    }
+
+    file << "\t\t\t</source>" << endl;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
 int LibVirtDriver::deployment_description_kvm(
         const VirtualMachine *  vm,
@@ -31,9 +84,8 @@ int LibVirtDriver::deployment_description_kvm(
     vector<const Attribute *>   attrs;
 
     string  vcpu;
-    string  memory;
-
-    int     memory_in_kb = 0;
+    float   cpu;
+    int     memory;
 
     string  emulator_path = "";
 
@@ -44,6 +96,9 @@ int LibVirtDriver::deployment_description_kvm(
     string  kernel_cmd = "";
     string  bootloader = "";
     string  arch       = "";
+    string  machine    = "";
+
+    vector<string> boots;
 
     const VectorAttribute * disk;
     const VectorAttribute * context;
@@ -54,8 +109,19 @@ int LibVirtDriver::deployment_description_kvm(
     string  ro         = "";
     string  driver     = "";
     string  cache      = "";
-    string  default_driver       = "";
-    string  default_driver_cache = "";
+    string  disk_io    = "";
+    string  source     = "";
+    string  clone      = "";
+    string  ceph_host  = "";
+    string  ceph_secret= "";
+    string  ceph_user  = "";
+    string  gluster_host   = "";
+    string  gluster_volume = "";
+
+    int     disk_id;
+    string  default_driver          = "";
+    string  default_driver_cache    = "";
+    string  default_driver_disk_io  = "";
     bool    readonly;
 
     const VectorAttribute * nic;
@@ -66,23 +132,38 @@ int LibVirtDriver::deployment_description_kvm(
     string  model      = "";
     string  ip         = "";
     string  filter     = "";
+
     string  default_filter = "";
+    string  default_model  = "";
 
     const VectorAttribute * graphics;
 
-    string  listen     = "";
-    string  port       = "";
-    string  passwd     = "";
-    string  keymap     = "";
+    string  listen          = "";
+    string  port            = "";
+    string  passwd          = "";
+    string  keymap          = "";
+    string  spice_options   = "";
 
     const VectorAttribute * input;
 
     const VectorAttribute * features;
 
-    string     pae     = "";
-    string     acpi    = "";
+    bool pae        = false;
+    bool acpi       = false;
+    bool apic       = false;
+    bool hyperv     = false;
+    bool localtime  = false;
+
+    int pae_found       = -1;
+    int acpi_found      = -1;
+    int apic_found      = -1;
+    int hyperv_found    = -1;
+    int localtime_found = -1;
+
+    string hyperv_options = "";
 
     const VectorAttribute * raw;
+    string default_raw;
     string data;
 
     // ------------------------------------------------------------------------
@@ -110,7 +191,7 @@ int LibVirtDriver::deployment_description_kvm(
     file << "\t<name>one-" << vm->get_oid() << "</name>" << endl;
 
     // ------------------------------------------------------------------------
-    // CPU
+    // CPU & Memory
     // ------------------------------------------------------------------------
 
     vm->get_template_attribute("VCPU", vcpu);
@@ -125,22 +206,19 @@ int LibVirtDriver::deployment_description_kvm(
         file << "\t<vcpu>" << vcpu << "</vcpu>" << endl;
     }
 
-    // ------------------------------------------------------------------------
-    // Memory
-    // ------------------------------------------------------------------------
-
-    vm->get_template_attribute("MEMORY",memory);
-
-    if (memory.empty())
+    //Every process gets 1024 shares by default (cgroups), scale this with CPU
+    if(vm->get_template_attribute("CPU", cpu))
     {
-        get_default("MEMORY",memory);
+        file << "\t<cputune>" << endl
+             << "\t\t<shares>"<< ceil( cpu * CGROUP_BASE_CPU_SHARES )
+             << "</shares>"   << endl
+             << "\t</cputune>"<< endl;
     }
 
-    if (!memory.empty())
+    // Memory must be expressed in Kb
+    if (vm->get_template_attribute("MEMORY",memory))
     {
-        memory_in_kb = atoi(memory.c_str()) * 1024;
-
-        file << "\t<memory>" << memory_in_kb << "</memory>" << endl;
+        file << "\t<memory>" << memory * 1024 << "</memory>" << endl;
     }
     else
     {
@@ -172,6 +250,7 @@ int LibVirtDriver::deployment_description_kvm(
             kernel_cmd = os->vector_value("KERNEL_CMD");
             bootloader = os->vector_value("BOOTLOADER");
             arch       = os->vector_value("ARCH");
+            machine    = os->vector_value("MACHINE");
         }
     }
 
@@ -185,11 +264,20 @@ int LibVirtDriver::deployment_description_kvm(
         }
     }
 
-    if (emulator == "kvm")
+    if ( machine.empty() )
     {
-        file << "\t\t<type arch='" << arch << "'>hvm</type>" << endl;
+        get_default("OS", "MACHINE", machine);
     }
 
+    file << "\t\t<type arch='" << arch << "'";
+
+    if ( !machine.empty() )
+    {
+        file << " machine='" << machine << "'";
+    }
+
+    file << ">hvm</type>" << endl;
+
     if ( kernel.empty() )
     {
         get_default("OS","KERNEL",kernel);
@@ -251,8 +339,12 @@ int LibVirtDriver::deployment_description_kvm(
         file << "\t\t<bootloader>" << bootloader << "</bootloader>" << endl;
     }
 
+    boots = one_util::split(boot, ',');
 
-    file << "\t\t<boot dev='" << boot << "'/>" << endl;
+    for (vector<string>::const_iterator it=boots.begin(); it!=boots.end(); it++)
+    {
+        file << "\t\t<boot dev='" << *it << "'/>" << endl;
+    }
 
     file << "\t</os>" << endl;
 
@@ -263,18 +355,15 @@ int LibVirtDriver::deployment_description_kvm(
     // ------------------------------------------------------------------------
     file << "\t<devices>" << endl;
 
-    if (emulator == "kvm")
+    get_default("EMULATOR",emulator_path);
+
+    if(emulator_path.empty())
     {
-        get_default("EMULATOR",emulator_path);
-
-        if(emulator_path.empty())
-        {
-            emulator_path = "/usr/bin/kvm";
-        }
-
-        file << "\t\t<emulator>" << emulator_path << "</emulator>" << endl;
+        emulator_path = "/usr/bin/kvm";
     }
 
+    file << "\t\t<emulator>" << emulator_path << "</emulator>" << endl;
+
     // ------------------------------------------------------------------------
     // Disks
     // ------------------------------------------------------------------------
@@ -292,6 +381,7 @@ int LibVirtDriver::deployment_description_kvm(
        default_driver_cache = "default";
     }
 
+    get_default("DISK","IO",default_driver_disk_io);
     // ------------------------------------------------------------------------
 
     num = vm->get_template_attribute("DISK",attrs);
@@ -302,15 +392,24 @@ int LibVirtDriver::deployment_description_kvm(
 
         if ( disk == 0 )
         {
-         continue;
+            continue;
         }
 
-        type   = disk->vector_value("TYPE");
-        target = disk->vector_value("TARGET");
-        ro     = disk->vector_value("READONLY");
-        bus    = disk->vector_value("BUS");
-        driver = disk->vector_value("DRIVER");
-        cache  = disk->vector_value("CACHE");
+        type        = disk->vector_value("TYPE");
+        target      = disk->vector_value("TARGET");
+        ro          = disk->vector_value("READONLY");
+        driver      = disk->vector_value("DRIVER");
+        cache       = disk->vector_value("CACHE");
+        disk_io     = disk->vector_value("IO");
+        source      = disk->vector_value("SOURCE");
+        clone       = disk->vector_value("CLONE");
+        ceph_host   = disk->vector_value("CEPH_HOST");
+        ceph_secret = disk->vector_value("CEPH_SECRET");
+        ceph_user   = disk->vector_value("CEPH_USER");
+        gluster_host    = disk->vector_value("GLUSTER_HOST");
+        gluster_volume  = disk->vector_value("GLUSTER_VOLUME");
+
+        disk->vector_value_str("DISK_ID", disk_id);
 
         if (target.empty())
         {
@@ -321,7 +420,7 @@ int LibVirtDriver::deployment_description_kvm(
 
         if ( !ro.empty() )
         {
-            transform(ro.begin(),ro.end(),ro.begin(),(int(*)(int))toupper);
+            one_util::toupper(ro);
 
             if ( ro == "YES" )
             {
@@ -331,44 +430,83 @@ int LibVirtDriver::deployment_description_kvm(
 
         // ---- Disk type and source for the image ----
 
-        if (type.empty() == false)
-        {
-            transform(type.begin(),type.end(),type.begin(),(int(*)(int))toupper);
-        }
+        one_util::toupper(type);
 
         if ( type == "BLOCK" )
         {
             file << "\t\t<disk type='block' device='disk'>" << endl
-                 << "\t\t\t<source dev='" << vm->get_remote_system_dir() 
-                 << "/disk." << i << "'/>" << endl;
+                 << "\t\t\t<source dev='" << vm->get_remote_system_dir()
+                 << "/disk." << disk_id << "'/>" << endl;
+        }
+        else if ( type == "RBD" || type == "RBD_CDROM" )
+        {
+            if (type == "RBD")
+            {
+                file << "\t\t<disk type='network' device='disk'>" << endl;
+            }
+            else
+            {
+                file << "\t\t<disk type='network' device='cdrom'>" << endl;
+            }
+
+            file << "\t\t\t<source protocol='rbd' name='" << source;
+
+            if ( clone == "YES" )
+            {
+                file << "-" << vm->get_oid() << "-" << disk_id;
+            }
+
+            do_network_hosts(file, ceph_host, "");
+
+            if ( !ceph_secret.empty() && !ceph_user.empty())
+            {
+                file << "\t\t\t<auth username='"<< ceph_user <<"'>" << endl
+                     << "\t\t\t\t<secret type='ceph' uuid='"
+                     << ceph_secret <<"'/>" << endl
+                     << "\t\t\t</auth>" << endl;
+            }
+        }
+        else if ( type == "GLUSTER" || type == "GLUSTER_CDROM" )
+        {
+            if ( type == "GLUSTER" )
+            {
+                file << "\t\t<disk type='network' device='disk'>" << endl;
+            }
+            else
+            {
+                file << "\t\t<disk type='network' device='cdrom'>" << endl;
+            }
+
+            file << "\t\t\t<source protocol='gluster' name='" << gluster_volume
+                 << "/";
+
+            if ( clone == "YES" )
+            {
+                file << vm->get_oid() << "/disk." << disk_id;
+            }
+            else
+            {
+                file << one_util::split(source, '/').back();
+            }
+
+            do_network_hosts(file, gluster_host, "tcp");
         }
         else if ( type == "CDROM" )
         {
             file << "\t\t<disk type='file' device='cdrom'>" << endl
-                 << "\t\t\t<source file='" << vm->get_remote_system_dir() 
-                 << "/disk." << i << "'/>" << endl;
+                 << "\t\t\t<source file='" << vm->get_remote_system_dir()
+                 << "/disk." << disk_id << "'/>" << endl;
         }
         else
         {
             file << "\t\t<disk type='file' device='disk'>" << endl
-                 << "\t\t\t<source file='" << vm->get_remote_system_dir() 
-                 << "/disk." << i << "'/>" << endl;
+                 << "\t\t\t<source file='" << vm->get_remote_system_dir()
+                 << "/disk." << disk_id << "'/>" << endl;
         }
 
         // ---- target device to map the disk ----
 
-        file << "\t\t\t<target dev='" << target << "'";
-
-        // ---- bus ----
-
-        if (!bus.empty())
-        {
-            file << " bus='" << bus << "'/>" << endl;
-        }
-        else
-        {
-            file << "/>" << endl;
-        }
+        file << "\t\t\t<target dev='" << target << "'/>" << endl;
 
         // ---- readonly attribute for the disk ----
 
@@ -381,7 +519,11 @@ int LibVirtDriver::deployment_description_kvm(
 
         file << "\t\t\t<driver name='qemu' type='";
 
-        if ( !driver.empty() )
+        if ( type == "CDROM" ) // Use driver raw for CD's
+        {
+            file << "raw";
+        }
+        else if ( !driver.empty() )
         {
             file << driver;
         }
@@ -394,14 +536,23 @@ int LibVirtDriver::deployment_description_kvm(
 
         if ( !cache.empty() )
         {
-            file << cache << "'/>" << endl;
+            file << cache << "'";
         }
         else
         {
-            file << default_driver_cache << "'/>" << endl;
+            file << default_driver_cache << "'";
         }
 
-        file << "\t\t</disk>" << endl;
+        if ( !disk_io.empty() )
+        {
+            file << " io='" << disk_io << "'";
+        }
+        else if ( !default_driver_disk_io.empty() )
+        {
+            file << " io='" << default_driver_disk_io << "'";
+        }
+
+        file << "/>" << endl << "\t\t</disk>" << endl;
     }
 
     attrs.clear();
@@ -416,26 +567,19 @@ int LibVirtDriver::deployment_description_kvm(
         target  = context->vector_value("TARGET");
         driver  = context->vector_value("DRIVER");
 
+        context->vector_value_str("DISK_ID", disk_id);
+
         if ( !target.empty() )
         {
             file << "\t\t<disk type='file' device='cdrom'>" << endl;
 
-            file << "\t\t\t<source file='" << vm->get_remote_system_dir() 
-                 << "/disk." << num << "'/>" << endl;
+            file << "\t\t\t<source file='" << vm->get_remote_system_dir()
+                 << "/disk." << disk_id << "'/>" << endl;
 
             file << "\t\t\t<target dev='" << target << "'/>" << endl;
             file << "\t\t\t<readonly/>" << endl;
 
-            file << "\t\t\t<driver name='qemu' type='";
-
-            if ( !driver.empty() )
-            {
-                file << driver << "'/>" << endl;
-            }
-            else
-            {
-                file << default_driver << "'/>" << endl;
-            }
+            file << "\t\t\t<driver name='qemu' type='raw'/>" << endl;
 
             file << "\t\t</disk>" << endl;
         }
@@ -452,9 +596,11 @@ int LibVirtDriver::deployment_description_kvm(
     // Network interfaces
     // ------------------------------------------------------------------------
 
-    get_default("NIC","FILTER",default_filter);
+    get_default("NIC", "FILTER", default_filter);
 
-    num = vm->get_template_attribute("NIC",attrs);
+    get_default("NIC", "MODEL", default_model);
+
+    num = vm->get_template_attribute("NIC", attrs);
 
     for(int i=0; i<num; i++)
     {
@@ -483,6 +629,11 @@ int LibVirtDriver::deployment_description_kvm(
             file << "\t\t\t<source bridge='" << bridge << "'/>" << endl;
         }
 
+        if ( vm->get_vnm_mad() == "ovswitch" )
+        {
+            file << "\t\t\t<virtualport type='openvswitch'/>" << endl;
+        }
+
         if( !mac.empty() )
         {
             file << "\t\t\t<mac address='" << mac << "'/>" << endl;
@@ -498,9 +649,20 @@ int LibVirtDriver::deployment_description_kvm(
             file << "\t\t\t<script path='" << script << "'/>" << endl;
         }
 
-        if( !model.empty() )
+        string * the_model = 0;
+
+        if (!model.empty())
         {
-            file << "\t\t\t<model type='" << model << "'/>" << endl;
+            the_model = &model;
+        }
+        else if (!default_model.empty())
+        {
+            the_model = &default_model;
+        }
+
+        if (the_model != 0)
+        {
+            file << "\t\t\t<model type='" << *the_model << "'/>" << endl;
         }
 
         if (!ip.empty() )
@@ -546,9 +708,11 @@ int LibVirtDriver::deployment_description_kvm(
             passwd = graphics->vector_value("PASSWD");
             keymap = graphics->vector_value("KEYMAP");
 
-            if ( type == "vnc" || type == "VNC" )
+            one_util::tolower(type);
+
+            if ( type == "vnc" || type == "spice" )
             {
-                file << "\t\t<graphics type='vnc'";
+                file << "\t\t<graphics type='" << type << "'";
 
                 if ( !listen.empty() )
                 {
@@ -571,11 +735,21 @@ int LibVirtDriver::deployment_description_kvm(
                 }
 
                 file << "/>" << endl;
+
+                if ( type == "spice" )
+                {
+                    get_default("SPICE_OPTIONS", spice_options);
+
+                    if ( spice_options != "" )
+                    {
+                        file << "\t\t" << spice_options << endl;
+                    }
+                }
             }
             else
             {
                 vm->log("VMM", Log::WARNING,
-                        "Not supported graphics type, ignored.");
+                        "Graphics not supported or undefined, ignored.");
             }
         }
     }
@@ -625,38 +799,75 @@ int LibVirtDriver::deployment_description_kvm(
 
         if ( features != 0 )
         {
-            pae  = features->vector_value("PAE");
-            acpi = features->vector_value("ACPI");
+            pae_found       = features->vector_value("PAE", pae);
+            acpi_found      = features->vector_value("ACPI", acpi);
+            apic_found      = features->vector_value("APIC", apic);
+            hyperv_found    = features->vector_value("HYPERV", hyperv);
+            localtime_found = features->vector_value("LOCALTIME", localtime);
         }
     }
 
-    if ( pae.empty() )
+    if ( pae_found != 0 )
     {
         get_default("FEATURES", "PAE", pae);
     }
 
-    if ( acpi.empty() )
+    if ( acpi_found != 0 )
     {
         get_default("FEATURES", "ACPI", acpi);
     }
 
-    if( acpi == "yes" || pae == "yes" )
+    if ( apic_found != 0 )
+    {
+        get_default("FEATURES", "APIC", apic);
+    }
+
+    if ( hyperv_found != 0 )
+    {
+        get_default("FEATURES", "HYPERV", hyperv);
+    }
+
+    if ( localtime_found != 0 )
+    {
+        get_default("FEATURES", "LOCALTIME", localtime);
+    }
+
+    if ( acpi || pae || apic || hyperv )
     {
         file << "\t<features>" << endl;
 
-        if ( pae == "yes" )
+        if ( pae )
         {
             file << "\t\t<pae/>" << endl;
         }
 
-        if ( acpi == "yes" )
+        if ( acpi )
         {
             file << "\t\t<acpi/>" << endl;
         }
 
+        if ( apic )
+        {
+            file << "\t\t<apic/>" << endl;
+        }
+
+        if ( hyperv )
+        {
+            get_default("HYPERV_OPTIONS", hyperv_options);
+
+            file << "\t\t<hyperv>" << endl;
+            file << hyperv_options << endl;
+            file << "\t\t</hyperv>" << endl;
+        }
+
         file << "\t</features>" << endl;
     }
 
+    if ( localtime )
+    {
+        file << "\t<clock offset='localtime'/>" << endl;
+    }
+
     attrs.clear();
 
     // ------------------------------------------------------------------------
@@ -676,7 +887,7 @@ int LibVirtDriver::deployment_description_kvm(
 
         type = raw->vector_value("TYPE");
 
-        transform(type.begin(),type.end(),type.begin(),(int(*)(int))toupper);
+        one_util::toupper(type);
 
         if ( type == "KVM" )
         {
@@ -685,6 +896,13 @@ int LibVirtDriver::deployment_description_kvm(
         }
     }
 
+    get_default("RAW", default_raw);
+
+    if ( !default_raw.empty() )
+    {
+        file << "\t" << default_raw << endl;
+    }
+
     file << "</domain>" << endl;
 
     file.close();
