@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,6 +19,7 @@
 #include "LifeCycleManager.h"
 
 #include "Nebula.h"
+#include "NebulaUtil.h"
 #include <sstream>
 
 VirtualMachineManagerDriver::VirtualMachineManagerDriver(
@@ -98,132 +99,30 @@ void VirtualMachineManagerDriver::get_default(
     }
 }
 
-/* ************************************************************************** */
-/* Driver ASCII Protocol Implementation                                       */
-/* ************************************************************************** */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
 
-void VirtualMachineManagerDriver::deploy (
-    const int     oid,
-    const string& drv_msg) const
+bool VirtualMachineManagerDriver::get_default(
+    const char *  name,
+    const char *  vname,
+    bool&         value) const
 {
-    ostringstream os;
+    string st;
 
-    os << "DEPLOY " << oid << " " << drv_msg << endl;
-    
-    write(os);
-};
+    get_default(name, vname, st);
 
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
+    if ( st == "" )
+    {
+        value = false;
+        return false;
+    }
 
-void VirtualMachineManagerDriver::shutdown (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
+    one_util::toupper(st);
 
-    os << "SHUTDOWN " << oid << " " << drv_msg << endl;
+    value = ( st == "YES" );
 
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::cancel (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os << "CANCEL " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::checkpoint (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os<< "CHECKPOINT " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::save (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os<< "SAVE " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::restore (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os << "RESTORE " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::migrate (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os<< "MIGRATE " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::poll (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os << "POLL " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void VirtualMachineManagerDriver::reboot (
-    const int     oid,
-    const string& drv_msg) const
-{
-    ostringstream os;
-
-    os << "REBOOT " << oid << " " << drv_msg << endl;
-
-    write(os);
-};
+    return true;
+}
 
 /* ************************************************************************** */
 /* MAD Interface                                                              */
@@ -233,8 +132,8 @@ void VirtualMachineManagerDriver::reboot (
 /* Helpers for the protocol function                                          */
 /* -------------------------------------------------------------------------- */
 
-static void log_error(VirtualMachine* vm, 
-                      ostringstream&  os, 
+static void log_error(VirtualMachine* vm,
+                      ostringstream&  os,
                       istringstream&  is,
                       const char *    msg)
 {
@@ -254,11 +153,33 @@ static void log_error(VirtualMachine* vm,
     vm->log("VMM",Log::ERROR,os);
 }
 
+/* -------------------------------------------------------------------------- */
+
+static void log_monitor_error(VirtualMachine* vm,
+                      ostringstream&  os,
+                      istringstream&  is,
+                      const char *    msg)
+{
+    string info;
+
+    getline(is,info);
+
+    os.str("");
+    os << msg;
+
+    if (!info.empty() && info[0] != '-')
+    {
+        os << ": " << info;
+        vm->set_template_monitor_error(os.str());
+    }
+
+    vm->log("VMM",Log::ERROR,os);
+}
+
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void VirtualMachineManagerDriver::protocol(
-    string&     message)
+void VirtualMachineManagerDriver::protocol(const string& message) const
 {
     istringstream is(message);
     ostringstream os;
@@ -313,8 +234,7 @@ void VirtualMachineManagerDriver::protocol(
         return;
     }
 
-    if ( vm->get_lcm_state() == VirtualMachine::CLEANUP ||
-         vm->get_lcm_state() == VirtualMachine::FAILURE ||
+    if ( vm->get_lcm_state() == VirtualMachine::FAILURE ||
          vm->get_lcm_state() == VirtualMachine::LCM_INIT )
     {
         os.str("");
@@ -334,12 +254,10 @@ void VirtualMachineManagerDriver::protocol(
         if (result == "SUCCESS")
         {
             string deploy_id;
-            time_t thetime = time(0);
 
             is >> deploy_id;
 
             vm->update_info(deploy_id);
-            vm->set_last_poll(thetime);
 
             vmpool->update(vm);
 
@@ -364,7 +282,7 @@ void VirtualMachineManagerDriver::protocol(
         }
         else
         {
-            log_error(vm,os,is,"Error shuting down VM");
+            log_error(vm,os,is,"Error shutting down VM");
             vmpool->update(vm);
 
             lcm->trigger(LifeCycleManager::SHUTDOWN_FAILURE, id);
@@ -432,7 +350,7 @@ void VirtualMachineManagerDriver::protocol(
         }
         else
         {
-            log_error(vm,os,is,"Error live migrating VM");
+            log_error(vm, os, is, "Error live migrating VM");
             vmpool->update(vm);
 
             lcm->trigger(LifeCycleManager::DEPLOY_FAILURE, id);
@@ -442,7 +360,7 @@ void VirtualMachineManagerDriver::protocol(
     {
         if (result == "SUCCESS")
         {
-            vm->log("VMM",Log::ERROR,"VM Successfully rebooted.");
+            vm->log("VMM",Log::INFO,"VM successfully rebooted.");
         }
         else
         {
@@ -450,162 +368,195 @@ void VirtualMachineManagerDriver::protocol(
             vmpool->update(vm);
         }
     }
+    else if ( action == "RESET" )
+    {
+        if (result == "SUCCESS")
+        {
+            vm->log("VMM",Log::INFO,"VM successfully rebooted-hard.");
+        }
+        else
+        {
+            log_error(vm,os,is,"Error rebooting-hard VM, assume it's still running");
+            vmpool->update(vm);
+        }
+    }
+    else if ( action == "ATTACHDISK" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM", Log::INFO, "VM Disk successfully attached.");
+
+            lcm->trigger(LifeCycleManager::ATTACH_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error attaching new VM Disk");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::ATTACH_FAILURE, id);
+        }
+    }
+    else if ( action == "DETACHDISK" )
+    {
+        Nebula              &ne  = Nebula::instance();
+        LifeCycleManager    *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM",Log::INFO,"VM Disk successfully detached.");
+
+            lcm->trigger(LifeCycleManager::DETACH_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm,os,is,"Error detaching VM Disk");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::DETACH_FAILURE, id);
+        }
+    }
+    else if ( action == "ATTACHNIC" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM", Log::INFO, "VM NIC Successfully attached.");
+
+            lcm->trigger(LifeCycleManager::ATTACH_NIC_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error attaching new VM NIC");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::ATTACH_NIC_FAILURE, id);
+        }
+    }
+    else if ( action == "DETACHNIC" )
+    {
+        Nebula              &ne  = Nebula::instance();
+        LifeCycleManager    *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM",Log::INFO, "VM NIC Successfully detached.");
+
+            lcm->trigger(LifeCycleManager::DETACH_NIC_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm,os,is,"Error detaching VM NIC");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::DETACH_NIC_FAILURE, id);
+        }
+    }
+    else if ( action == "SNAPSHOTCREATE" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            string hypervisor_id;
+
+            is >> hypervisor_id;
+
+            vm->update_snapshot_id(hypervisor_id);
+
+            vmpool->update(vm);
+
+            vm->log("VMM", Log::INFO, "VM Snapshot successfully created.");
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_CREATE_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error creating new VM Snapshot");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_CREATE_FAILURE, id);
+        }
+    }
+    else if ( action == "SNAPSHOTREVERT" )
+    {
+        Nebula              &ne  = Nebula::instance();
+        LifeCycleManager    *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM",Log::INFO,"VM Snapshot successfully reverted.");
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_REVERT_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm,os,is,"Error reverting VM Snapshot");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_REVERT_FAILURE, id);
+        }
+    }
+    else if ( action == "SNAPSHOTDELETE" )
+    {
+        Nebula              &ne  = Nebula::instance();
+        LifeCycleManager    *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM",Log::INFO,"VM Snapshot successfully deleted.");
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_DELETE_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm,os,is,"Error deleting VM Snapshot");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::SNAPSHOT_DELETE_FAILURE, id);
+        }
+    }
+    else if ( action == "CLEANUP" )
+    {
+        Nebula           &ne  = Nebula::instance();
+        LifeCycleManager *lcm = ne.get_lcm();
+
+        if ( result == "SUCCESS" )
+        {
+            vm->log("VMM", Log::INFO, "Host successfully cleaned.");
+
+            lcm->trigger(LifeCycleManager::CLEANUP_SUCCESS, id);
+        }
+        else
+        {
+            log_error(vm, os, is, "Error cleaning Host");
+            vmpool->update(vm);
+
+            lcm->trigger(LifeCycleManager::CLEANUP_FAILURE, id);
+        }
+    }
     else if ( action == "POLL" )
     {
         if (result == "SUCCESS")
         {
-            size_t          pos;
+            string monitor_str;
+            getline(is, monitor_str);
 
-            string          tmp;
-            string          var;
-            ostringstream   os;
-            istringstream   tiss;
-
-            int             cpu    = -1;
-            int             memory = -1;
-            int             net_tx = -1;
-            int             net_rx = -1;
-            char            state  = '-';
-
-            string monitor_str = is.str();
-            bool   parse_error = false;
-
-            while(is.good())
-            {
-                is >> tmp >> ws;
-
-                pos = tmp.find('=');
-
-                if ( pos == string::npos )
-                {
-                    parse_error = true;
-                    continue;
-                }
-
-                tmp.replace(pos,1," ");
-
-                tiss.clear();
-
-                tiss.str(tmp);
-
-                tiss >> var >> ws;
-
-                if (!tiss.good())
-                {
-                    parse_error = true;
-                    continue;
-                }
-
-                if (var == "USEDMEMORY")
-                {
-                    tiss >> memory;
-                }
-                else if (var == "USEDCPU")
-                {
-                    tiss >> cpu;
-                }
-                else if (var == "NETRX")
-                {
-                    tiss >> net_rx;
-                }
-                else if (var == "NETTX")
-                {
-                    tiss >> net_tx;
-                }
-                else if (var == "STATE")
-                {
-                    tiss >> state;
-                }
-                else if (!var.empty())
-                {
-                    string val;
-
-                    os.str("");
-                    os << "Adding custom monitoring attribute: " << tmp;
-
-                    vm->log("VMM",Log::WARNING,os);
-
-                    tiss >> val;
-
-                    vm->replace_template_attribute(var,val);
-                }
-            }
-
-            if (parse_error)
-            {
-                os.str("");
-                os << "Error parsing monitoring str:\"" << monitor_str <<"\"";
-
-                vm->log("VMM",Log::ERROR,os);
-
-                vm->set_template_error_message(os.str());
-                vmpool->update(vm);
-
-                vm->unlock();
-                return;
-            }
-
-            vm->update_info(memory,cpu,net_tx,net_rx);
-
-            vmpool->update(vm);
-
-            if (state != '-' &&
-                (vm->get_lcm_state() == VirtualMachine::RUNNING ||
-                 vm->get_lcm_state() == VirtualMachine::UNKNOWN))
-            {
-                Nebula              &ne  = Nebula::instance();
-                LifeCycleManager *  lcm = ne.get_lcm();
-
-                switch (state)
-                {
-                case 'a': // Still active, good!
-                    os.str("");
-                    os  << "Monitor Information:\n"
-                        << "\tCPU   : "<< cpu    << "\n"
-                        << "\tMemory: "<< memory << "\n"
-                        << "\tNet_TX: "<< net_tx << "\n"
-                        << "\tNet_RX: "<< net_rx;
-                    vm->log("VMM",Log::DEBUG,os);
-
-                    if ( vm->get_lcm_state() == VirtualMachine::UNKNOWN)
-                    {
-                        vm->log("VMM",Log::INFO,"VM was now found, new state is"
-                                " RUNNING");
-                        vm->set_state(VirtualMachine::RUNNING);
-                        vmpool->update(vm);
-                    }
-                    break;
-
-                case 'p': // It's paused
-                    vm->log("VMM",Log::INFO,"VM running but new state "
-                            "from monitor is PAUSED.");
-
-                    lcm->trigger(LifeCycleManager::MONITOR_SUSPEND, id);
-                    break;
-
-                case 'e': //Failed
-                    vm->log("VMM",Log::INFO,"VM running but new state "
-                            "from monitor is ERROR.");
-
-                    lcm->trigger(LifeCycleManager::MONITOR_FAILURE, id);
-                    break;
-
-                case 'd': //The VM was not found
-                    vm->log("VMM",Log::INFO,"VM running but it was not found."
-                            " Restart and delete actions available or try to"
-                            " recover it manually");
-
-                    lcm->trigger(LifeCycleManager::MONITOR_DONE, id);
-                    break;
-                }
-            }
+            process_poll(vm, monitor_str);
         }
         else
         {
-            log_error(vm,os,is,"Error monitoring VM");
-            vmpool->update(vm);
+            Nebula            &ne = Nebula::instance();
+            LifeCycleManager* lcm = ne.get_lcm();
 
-            vm->log("VMM",Log::ERROR,os);
+            log_monitor_error(vm, os, is, "Error monitoring VM");
+
+            lcm->trigger(LifeCycleManager::MONITOR_DONE, vm->get_oid());
         }
     }
     else if (action == "LOG")
@@ -622,6 +573,223 @@ void VirtualMachineManagerDriver::protocol(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void VirtualMachineManagerDriver::process_poll(
+        int id,
+        const string &monitor_str)
+{
+    // Get the VM from the pool
+    VirtualMachine* vm = Nebula::instance().get_vmpool()->get(id,true);
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    process_poll(vm, monitor_str);
+
+    vm->unlock();
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void VirtualMachineManagerDriver::process_poll(
+        VirtualMachine* vm,
+        const string&   monitor_str)
+{
+    int rc;
+
+    int        cpu;
+    int        memory;
+    long long  net_tx;
+    long long  net_rx;
+    char       state;
+
+    map<string, string> custom;
+    ostringstream oss;
+
+    Nebula &ne = Nebula::instance();
+
+    LifeCycleManager* lcm      = ne.get_lcm();
+    VirtualMachinePool* vmpool = ne.get_vmpool();
+
+    /* ---------------------------------------------------------------------- */
+    /* Parse VM info and update VM                                            */
+    /* ---------------------------------------------------------------------- */
+
+    rc = parse_vm_info(monitor_str, cpu, memory, net_tx, net_rx, state, custom);
+
+    if (rc == -1)
+    {
+        oss << "Error parsing monitoring information: " << monitor_str;
+        NebulaLog::log("VMM", Log::ERROR, oss);
+
+        vm->set_template_error_message(oss.str());
+        vm->log("VMM", Log::ERROR, oss);
+
+        vmpool->update(vm);
+
+        lcm->trigger(LifeCycleManager::MONITOR_DONE, vm->get_oid());
+
+        return;
+    }
+
+    oss << "VM " << vm->get_oid() << " successfully monitored: " << monitor_str;
+    NebulaLog::log("VMM", Log::DEBUG, oss);
+
+    vm->update_info(memory, cpu, net_tx, net_rx, custom);
+
+    vmpool->update(vm);
+
+    vmpool->update_history(vm);
+
+    vmpool->update_monitoring(vm);
+
+    /* ---------------------------------------------------------------------- */
+    /* Process the VM state from the monitoring info                          */
+    /* ---------------------------------------------------------------------- */
+
+    if (state == '-' || ( vm->get_lcm_state() != VirtualMachine::RUNNING &&
+        vm->get_lcm_state() != VirtualMachine::UNKNOWN))
+    {
+        return;
+    }
+
+    switch (state)
+    {
+        case 'a': // Still active, good!
+            if ( vm->get_lcm_state() == VirtualMachine::UNKNOWN)
+            {
+                vm->log("VMM", Log::INFO, "VM found again, state is RUNNING");
+
+                vm->set_state(VirtualMachine::RUNNING);
+                vmpool->update(vm);
+            }
+            break;
+
+        case 'p': // It's paused
+            vm->log("VMM",Log::INFO, "VM running but monitor state is PAUSED.");
+
+            lcm->trigger(LifeCycleManager::MONITOR_SUSPEND, vm->get_oid());
+            break;
+
+        case 'e': //Failed
+            vm->log("VMM", Log::INFO, "VM running but monitor state is ERROR.");
+
+            lcm->trigger(LifeCycleManager::MONITOR_FAILURE, vm->get_oid());
+            break;
+
+        case 'd': //The VM was not found
+            vm->log("VMM", Log::INFO, "VM running but it was not found."
+                    " Boot and delete actions available or try to"
+                    " recover it manually");
+
+            lcm->trigger(LifeCycleManager::MONITOR_DONE, vm->get_oid());
+            break;
+    }
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualMachineManagerDriver::parse_vm_info(
+    const string       &monitor_str,
+    int                &cpu,
+    int                &memory,
+    long long          &net_tx,
+    long long          &net_rx,
+    char               &state,
+    map<string,string> &custom)
+{
+    istringstream is;
+
+    int    parse_result = 0;
+    size_t pos;
+
+    string tmp;
+    string var;
+
+    ostringstream   os;
+    istringstream   tiss;
+
+    cpu    = -1;
+    memory = -1;
+    net_tx = -1;
+    net_rx = -1;
+    state  = '-';
+
+    custom.clear();
+
+    is.str(monitor_str);
+
+    while(parse_result == 0 && is.good())
+    {
+        is >> tmp >> ws;
+
+        pos = tmp.find('=');
+
+        if ( pos == string::npos )
+        {
+            parse_result = -1;
+            continue;
+        }
+
+        tmp.replace(pos,1," ");
+
+        tiss.clear();
+
+        tiss.str(tmp);
+
+        tiss >> var >> ws;
+
+        if (!tiss.good())
+        {
+            parse_result = -1;
+            continue;
+        }
+
+        if (var == "USEDMEMORY")
+        {
+            tiss >> memory;
+        }
+        else if (var == "USEDCPU")
+        {
+            tiss >> fixed >> cpu;
+        }
+        else if (var == "NETRX")
+        {
+            tiss >> net_rx;
+        }
+        else if (var == "NETTX")
+        {
+            tiss >> net_tx;
+        }
+        else if (var == "STATE")
+        {
+            tiss >> state;
+        }
+        else if (!var.empty())
+        {
+            string val;
+
+            tiss >> val;
+
+            custom.insert(make_pair(var, val));
+        }
+
+        if (tiss.fail())
+        {
+            parse_result = -1;
+            continue;
+        }
+    }
+
+    return parse_result;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 void VirtualMachineManagerDriver::recover()
 {
     NebulaLog::log("VMM",Log::INFO,"Recovering VMM drivers");
