@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2012, OpenNebula Project Leads (OpenNebula.org)             */
+/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -26,6 +26,8 @@
 #include "AuthManager.h"
 #include "ClusterPool.h"
 
+#include "NebulaUtil.h"
+
 #define TO_UPPER(S) transform(S.begin(),S.end(),S.begin(),(int(*)(int))toupper)
 
 /* ************************************************************************** */
@@ -36,6 +38,7 @@ VirtualNetwork::VirtualNetwork(int                      _uid,
                                int                      _gid,
                                const string&            _uname,
                                const string&            _gname,
+                               int                      _umask,
                                int                      _cluster_id,
                                const string&            _cluster_name,
                                VirtualNetworkTemplate * _vn_template):
@@ -53,6 +56,11 @@ VirtualNetwork::VirtualNetwork(int                      _uid,
     {
         obj_template = new VirtualNetworkTemplate;
     }
+
+    set_umask(_umask);
+
+    global_bin[0] = global_bin[1] = 0;
+    site_bin[0] = site_bin[1] = 0;
 };
 
 /* -------------------------------------------------------------------------- */
@@ -60,15 +68,8 @@ VirtualNetwork::VirtualNetwork(int                      _uid,
 
 VirtualNetwork::~VirtualNetwork()
 {
-    if (leases != 0)
-    {
-        delete leases;
-    }
-
-    if (obj_template != 0)
-    {
-        delete obj_template;
-    }
+    delete leases;
+    delete obj_template;
 }
 
 /* ************************************************************************** */
@@ -78,12 +79,13 @@ VirtualNetwork::~VirtualNetwork()
 const char * VirtualNetwork::table        = "network_pool";
 
 const char * VirtualNetwork::db_names =
-        "oid, name, body, uid, gid, owner_u, group_u, other_u";
+        "oid, name, body, uid, gid, owner_u, group_u, other_u, cid";
 
 const char * VirtualNetwork::db_bootstrap = "CREATE TABLE IF NOT EXISTS"
     " network_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128),"
-    " body TEXT, uid INTEGER, gid INTEGER, "
-    "owner_u INTEGER, group_u INTEGER, other_u INTEGER, UNIQUE(name,uid))";
+    " body MEDIUMTEXT, uid INTEGER, gid INTEGER,"
+    " owner_u INTEGER, group_u INTEGER, other_u INTEGER,"
+    " cid INTEGER, UNIQUE(name,uid))";
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -127,7 +129,7 @@ int VirtualNetwork::select_leases(SqlDB * db)
 
     string          network_address;
 
-    unsigned int mac_prefix   = VirtualNetworkPool::mac_prefix();
+    unsigned int mac_prefix = VirtualNetworkPool::mac_prefix();
 
     //Get the leases
     if (type == RANGED)
@@ -135,6 +137,8 @@ int VirtualNetwork::select_leases(SqlDB * db)
         leases = new RangedLeases(db,
                                   oid,
                                   mac_prefix,
+                                  global_bin,
+                                  site_bin,
                                   ip_start,
                                   ip_end);
     }
@@ -142,7 +146,9 @@ int VirtualNetwork::select_leases(SqlDB * db)
     {
         leases = new  FixedLeases(db,
                                   oid,
-                                  mac_prefix);
+                                  mac_prefix,
+                                  global_bin,
+                                  site_bin);
     }
     else
     {
@@ -178,11 +184,11 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
     ostringstream   ose;
     int             rc;
 
-    string          vlan_attr;
-    string          s_type;
-    string          ranged_error_str;
+    bool b_vlan;
+    string s_type;
+    string ranged_error_str;
 
-    unsigned int mac_prefix   = VirtualNetworkPool::mac_prefix();
+    unsigned int mac_prefix = VirtualNetworkPool::mac_prefix();
 
     //--------------------------------------------------------------------------
     // VirtualNetwork Attributes from the template
@@ -221,19 +227,30 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
 
     // ------------ PHYDEV --------------------
 
-    erase_template_attribute("PHYDEV",phydev);
+    erase_template_attribute("PHYDEV", phydev);
+
+    add_template_attribute("PHYDEV", phydev);
 
     // ------------ VLAN_ID -------------------
 
-    erase_template_attribute("VLAN_ID",vlan_id);
+    erase_template_attribute("VLAN_ID", vlan_id);
+
+    add_template_attribute("VLAN_ID", vlan_id);
 
     // ------------ VLAN ----------------------
 
-    erase_template_attribute("VLAN", vlan_attr);
+    erase_template_attribute("VLAN", b_vlan);
 
-    TO_UPPER(vlan_attr);
-
-    vlan = (vlan_attr == "YES") || (vlan_attr.empty() && !phydev.empty());
+    if (b_vlan || !phydev.empty())
+    {
+        vlan = 1;
+        add_template_attribute("VLAN", "YES");
+    }
+    else
+    {
+        vlan = 0;
+        add_template_attribute("VLAN", "NO");
+    }
 
     // ------------ BRIDGE --------------------
 
@@ -249,12 +266,39 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
         {
             ostringstream oss;
 
-            oss << "onebr" << oid;
- 
+            oss << "onebr";
+
+            if (!vlan_id.empty())
+            {
+                oss << "." << vlan_id;
+            }
+            else
+            {
+                oss << oid;
+            }
+
             bridge = oss.str();
         }
     }
 
+    add_template_attribute("BRIDGE", bridge);
+
+    // ------------ IP6 PREFIX ---------------
+
+    erase_template_attribute("GLOBAL_PREFIX", global);
+
+    if (Leases::Lease::prefix6_to_number(global, global_bin) != 0)
+    {
+        goto error_global;
+    }
+
+    erase_template_attribute("SITE_PREFIX", site);
+
+    if (Leases::Lease::prefix6_to_number(site, site_bin) != 0)
+    {
+        goto error_site;
+    }
+
     //--------------------------------------------------------------------------
     // Get the leases
     //--------------------------------------------------------------------------
@@ -274,6 +318,8 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
         leases = new RangedLeases(db,
                                   oid,
                                   mac_prefix,
+                                  global_bin,
+                                  site_bin,
                                   ip_start,
                                   ip_end);
     }
@@ -286,6 +332,8 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
         leases = new FixedLeases(db,
                                  oid,
                                  mac_prefix,
+                                 global_bin,
+                                 site_bin,
                                  vector_leases);
 
         remove_template_attribute("LEASES");
@@ -332,6 +380,16 @@ error_ranged:
     ose << ranged_error_str;
     goto error_common;
 
+error_global:
+    ose << global
+        << " does not contain a character string representing a valid prefix";
+    goto error_common;
+
+error_site:
+    ose << site
+        << " does not contain a character string representing a valid prefix";
+    goto error_common;
+
 error_null_leases:
     ose << "Error getting Virtual Network leases.";
 
@@ -344,6 +402,76 @@ error_common:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+int VirtualNetwork::replace_template(const string& tmpl_str, string& error_str)
+{
+    string new_bridge;
+    bool   b_vlan;
+
+    /* ---------------------------------------------------------------------- */
+    /* Parse & Update VirtualNetwork Template                                 */
+    /* ---------------------------------------------------------------------- */
+
+    Template * new_tmpl  = new VirtualNetworkTemplate;
+
+    if ( new_tmpl == 0 )
+    {
+        error_str = "Cannot allocate a new template";
+        return -1;
+    }
+
+    if ( new_tmpl->parse_str_or_xml(tmpl_str, error_str) != 0 )
+    {
+        delete new_tmpl;
+        return -1;
+    }
+
+    delete obj_template;
+
+    obj_template = new_tmpl;
+
+    /* ---------------------------------------------------------------------- */
+    /* Update Configuration Attributes (class & template)                     */
+    /*  - PHYDEV                                                              */
+    /*  - VLAN_ID                                                             */
+    /*  - VLAN                                                                */
+    /*  - BRIDGE                                                              */
+    /* ---------------------------------------------------------------------- */
+    erase_template_attribute("PHYDEV", phydev);
+
+    add_template_attribute("PHYDEV", phydev);
+
+    erase_template_attribute("VLAN_ID", vlan_id);
+
+    add_template_attribute("VLAN_ID", vlan_id);
+
+    erase_template_attribute("VLAN", b_vlan);
+
+    if (b_vlan || !phydev.empty())
+    {
+        vlan = 1;
+        add_template_attribute("VLAN", "YES");
+    }
+    else
+    {
+        vlan = 0;
+        add_template_attribute("VLAN", "NO");
+    }
+
+    erase_template_attribute("BRIDGE",new_bridge);
+
+    if (!new_bridge.empty())
+    {
+        bridge = new_bridge;
+    }
+
+    add_template_attribute("BRIDGE", bridge);
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int VirtualNetwork::insert_replace(SqlDB *db, bool replace, string& error_str)
 {
     ostringstream   oss;
@@ -393,7 +521,8 @@ int VirtualNetwork::insert_replace(SqlDB *db, bool replace, string& error_str)
         <<          gid         << ","
         <<          owner_u     << ","
         <<          group_u     << ","
-        <<          other_u     << ")";
+        <<          other_u     << ","
+        <<          cluster_id  << ")";
 
     rc = db->exec(oss);
 
@@ -452,6 +581,24 @@ string& VirtualNetwork::to_xml(string& xml) const
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+string& VirtualNetwork::to_xml64(string &xml64, bool extended)
+{
+    string *str64;
+
+    to_xml_extended(xml64, extended);
+
+    str64 = one_util::base64_encode(xml64);
+
+    xml64 = *str64;
+
+    delete str64;
+
+    return xml64;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 string& VirtualNetwork::to_xml_extended(string& xml) const
 {
     return to_xml_extended(xml,true);
@@ -509,6 +656,24 @@ string& VirtualNetwork::to_xml_extended(string& xml, bool extended) const
         os << "<VLAN_ID/>";
     }
 
+    if (!global.empty())
+    {
+        os << "<GLOBAL_PREFIX>" << global << "</GLOBAL_PREFIX>";
+    }
+    else
+    {
+        os << "<GLOBAL_PREFIX/>";
+    }
+
+    if (!site.empty())
+    {
+        os << "<SITE_PREFIX>" << site << "</SITE_PREFIX>";
+    }
+    else
+    {
+        os << "<SITE_PREFIX/>";
+    }
+
     if ( type == RANGED )
     {
         string st_ip_start;
@@ -581,6 +746,12 @@ int VirtualNetwork::from_xml(const string &xml_str)
     xpath(phydev,  "/VNET/PHYDEV", "");
     xpath(vlan_id, "/VNET/VLAN_ID","");
 
+    xpath(global,  "/VNET/GLOBAL_PREFIX", "");
+    xpath(site, "/VNET/SITE_PREFIX","");
+
+    Leases::Lease::prefix6_to_number(site, site_bin);
+    Leases::Lease::prefix6_to_number(global, global_bin);
+
     type = static_cast<NetworkType>(int_type);
 
     // Get associated classes
@@ -620,14 +791,23 @@ int VirtualNetwork::from_xml(const string &xml_str)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
+int VirtualNetwork::nic_attribute(
+        VectorAttribute *       nic,
+        int                     vid,
+        const vector<string>&   inherit_attrs)
 {
     int rc;
 
     string  ip;
     string  mac;
+    string  inherit_val;
 
-    ostringstream  oss;
+    unsigned int eui64[2];
+    unsigned int prefix[2] = {0, 0};
+
+    ostringstream oss;
+
+    vector<string>::const_iterator it;
 
     ip    = nic->vector_value("IP");
     oss << oid;
@@ -638,11 +818,11 @@ int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
 
     if (ip.empty())
     {
-        rc = leases->get(vid,ip,mac);
+        rc = leases->get(vid, ip, mac, eui64);
     }
     else
     {
-        rc = leases->set(vid,ip,mac);
+        rc = leases->set(vid,ip,mac, eui64);
     }
 
     if ( rc != 0 )
@@ -654,11 +834,27 @@ int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
     //                       NEW NIC ATTRIBUTES
     //--------------------------------------------------------------------------
 
-    nic->replace("NETWORK"   ,name);
+    nic->replace("NETWORK", name);
     nic->replace("NETWORK_ID",oss.str());
-    nic->replace("BRIDGE"    ,bridge);
-    nic->replace("MAC"       ,mac);
-    nic->replace("IP"        ,ip);
+    nic->replace("BRIDGE", bridge);
+    nic->replace("MAC", mac);
+    nic->replace("IP", ip);
+
+    Leases::Lease::ip6_to_string(eui64, prefix, ip);
+    nic->replace("IP6_LINK", ip);
+
+    if (!global.empty())
+    {
+        Leases::Lease::ip6_to_string(eui64, global_bin, ip);
+        nic->replace("IP6_GLOBAL", ip);
+    }
+
+    if (!site.empty())
+    {
+        Leases::Lease::ip6_to_string(eui64, site_bin, ip);
+        nic->replace("IP6_SITE", ip);
+    }
+
 
     if ( vlan == 1 )
     {
@@ -687,6 +883,16 @@ int VirtualNetwork::nic_attribute(VectorAttribute *nic, int vid)
         nic->replace("CLUSTER_ID", oss.str());
     }
 
+    for (it = inherit_attrs.begin(); it != inherit_attrs.end(); it++)
+    {
+        get_template_attribute((*it).c_str(), inherit_val);
+
+        if (!inherit_val.empty())
+        {
+            nic->replace(*it, inherit_val);
+        }
+    }
+
     return 0;
 }
 
